/*
 * Copyright, 2011, Teradek
 * 
 * All rights reserved.
 * Contact Teradek for licensing.
 */

#ifndef _BRC_H_
#define _BRC_H_

#include <stdarg.h>
#include <syslog.h>
#include <sys/time.h>

#define MAX_BRC_ALGO 4

typedef void * BCR_POLICY_HANDLE;
typedef void * BCR_POLICY_DATA;

typedef void (*reset_func)(BCR_POLICY_HANDLE *brc);
typedef void (*manage_func)(BCR_POLICY_HANDLE *brc, int size, int util, int buf_len_ms, int curr_meas_bitrate, int long_meas_bitrate, int *emerg_reduction);
typedef void (*set_bitrate_func)(void *handle, int bitrate);

typedef struct 
{
    void                *handle;
    set_bitrate_func    func;
} SET_BITRATE_CONFIG;

typedef struct
{
    char            *name;
    reset_func      reset;
    manage_func     manage;
    BCR_POLICY_DATA *data;
    
} BRC_POLICY_ALGO;

typedef struct
{
    int max_buf_len_ms;
    int max_br;
    int print_stats;
    
    SET_BITRATE_CONFIG  set_bitrate;    
    BRC_POLICY_ALGO     *algo;
    
	int (*logCallback)(void *handle, int level, const char* fmt, va_list args);
	void* logHandler;

} BRC_POLICY;

BRC_POLICY_ALGO bcr_algo_ilya;

void print_log_brc(BRC_POLICY *h, int level, const char* fmt, ...);

BRC_POLICY *brc_configure(int algo, int max_buf_len_ms, int max_br, int print_stats,
		int (*logCallback)(void *handle, int level, const char* fmt, va_list args), void* logHandler);
void brc_close(BRC_POLICY *h);

#endif /* _BRC_H_ */
