/*
 * Teradek 2013
 *
 * Closed captions support
 * 
 * ilya@teradek.com
 *
 */

#ifndef _CC_GEN_H_
#define _CC_GEN_H_

#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>
#include <stdint.h>
#include <time.h>
#include <pthread.h>

#ifdef __cplusplus
#include <list>

using namespace std;

class Caption
{
public:
	Caption(int type608or708, uint8_t line, char char1, char char2, int timeOffsetMs);
	int64_t getTimestamp();
	bool is708();
	int getField();
	uint8_t* getDataPtr();
	uint8_t getCcType();
	bool isTooEarlyToRelease();
private:
	bool is708_;
	uint8_t data_[3];
	int64_t timestamp_;
};

class CaptionSet
{
public:
	static const int MAX_LIST_LENGTH_MS = 1000;

	CaptionSet();
	~CaptionSet();
	void addCaption608(uint8_t field, char char1, char char2, int timeOffsetMs);
	void addCaption708(char* buf, int len, int timeOffsetMs);
	void getSeiPacket(uint8_t** buf, int* len, uint32_t frame_duration_ms);
private:
	void convertThreeBytePreventionSequence();
	void removeExpiredCaptions();
	list<Caption*> captionList608_;
	list<Caption*> captionList708_;
	uint8_t packet_[512];
	int packetLength_;
	pthread_mutex_t mutex_;
	bool first608Caption_;
	bool first708Caption_;
	bool first608in708Caption_;
	int64_t last608in708received_;
	bool isUsing608captions_;
	int timeSinceLast608TxMs_;
};

#endif


#ifdef __cplusplus
  extern "C" {
#endif

void *cc_gen_init();
void cc_gen_close(void *handle);
void add_caption608(void *handle, uint8_t line, char char1, char char2, int timeOffsetMs);
void add_caption708(void *handle, char* buf, int len, int timeOffsetMs);
void get_sei_packet(void *handle, uint8_t** buf, int* len, uint32_t frame_duration_ms);

#ifdef __cplusplus
  }
#endif


#endif
