/*
 * Teradek 2013
 *
 * File recording support
 * 
 * ilya@teradek.com
 *
 */

#ifndef _FILE_REC_H_
#define _FILE_REC_H_

#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>
#include <stdint.h>
#include <sys/time.h>
#include <pthread.h>

#ifdef __cplusplus
#include <string>

using namespace std;

class FileRec
{
public:
	FileRec();
	~FileRec();
	void setRecMaxLengthMb(int recMaxLengthMb);
	void setRecMaxTimeS(int recMaxTimeS);
	void setRecMultipleFilesEnabled(bool recMultipleFilesEnabled);
	void setRecFilename(string recFilename);
	void setVideoPid(uint32_t videoPid);
	void start();
	void stop();
	void recordData(char *buf, int len);
	int ioctl(uint32_t cmd, void *arg);
	void print_log(int level, const char* fmt, ...);
	void setLogging(ioctlLogCallbackFunction ioctlLogCallback, ioctlLogCallbackHandler ioctlLogHandler);
private:
	bool recEnabled_;
	int recMaxLengthMb_;
	int recMaxTimeS_;
	bool recMultipleFilesEnabled_;
	string recFileName_;
	string realFileName_;
	int64_t recStartTimeMs_;
	int64_t recSizeBytes_;
	int fd_;
	int64_t lastOpenAttemptMs_;
	bool openError_;
	uint32_t clipNo_;
	pthread_mutex_t mutex_;
	uint32_t videoPid_;
	bool isWaitingForIFrame_;
	int waitningIFrameCount_;
	ioctlLogCallbackFunction ioctlLogCallback_;
	ioctlLogCallbackHandler ioctlLogHandler_;

	string getFileName();
};

#endif


#ifdef __cplusplus
  extern "C" {
#endif

void *file_rec_init();
void file_rec_close(void *handle);
void set_rec_max_length_mb(void *handle, int rec_max_length_mb);
void set_rec_max_time_s(void *handle, int rec_max_time_s);
void set_rec_multiple_files_enabled(void *handle, int rec_multiple_files_enabled);
void set_rec_filename(void *handle, char *rec_filename);
void set_video_pid(void *handle, uint32_t videoPid);
void rec_start(void *handle);
void rec_stop(void *handle);
void rec_data(void *handle, char *buf, int len);
int rec_ioctl(void *handle, uint32_t cmd, void *arg);
void set_file_rec_logging(void *handle, ioctlLogCallbackFunction ioctlLogCallback, ioctlLogCallbackHandler ioctlLogHandler);

#ifdef __cplusplus
  }
#endif


#endif
