/*******************************************************************************
* 
* Teradek(C) 2010
* TCP listening server
*
* Questions: ilya.issenin@teradek.com
* 
*******************************************************************************/

#ifndef LTCP_SERV_H
#define LTCP_SERV_H

#include <stdint.h>
#include <sys/time.h>
#include <unistd.h>
#include <stdlib.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <netinet/in.h>
#include <netdb.h> 
#include <pthread.h>
#include <stdarg.h>
#include <syslog.h>

#define NETW_OK 0
#define NETW_NOT_OK 1

#define MAX_FRAME_SIZE (2*1024*1024)

#define V1 if (h->verb >= 1)
#define V2 if (h->verb >= 2)

struct ltcp_senddata_info_struct
{
	char *buf;
	int size;
	uint32_t seq_num;
	uint32_t time;
	struct ltcp_senddata_info_struct *next;
	struct ltcp_senddata_info_struct *prev;
}; 
typedef struct ltcp_senddata_info_struct ltcp_senddata_info_t;

struct ltcp_thread_args_struct
{
	void *handle;
	int conn; 
};

typedef struct
{
	int sock;
	uint32_t cur_seq_num;
	ltcp_senddata_info_t *data;
	struct ltcp_thread_args_struct thread_arg;
	pthread_t threadId;
	char *buf;
	int used;
	struct sockaddr_in from;
} client_struct_t;

typedef struct
{
	int lport;
	int sock;
	int listen_sock;
	int verb;	// verbosity level: 0 - no messages; 1 - error/info messages; 2 - periodical statistics
	int max_clients;
	client_struct_t *sock_list;
	int clients_connected;
	ltcp_senddata_info_t *send_start;
	ltcp_senddata_info_t *send_end;
	uint32_t cur_seq_num;
	pthread_rwlock_t rwlock_senddata;
	pthread_mutex_t mutex_new_data;
	pthread_cond_t cond_new_data;
	pthread_mutex_t mutex_clients_conn;
	int max_queue_len_ms;   // how much data to keep for slow tcp server connections
	int closing;
	struct sockaddr_in from;
	int (*logCallback)(void *handle, int level, const char* fmt, va_list args);
	void* logHandler;
} LTCP_SERV_HANDLE;

void ltcp_set_logging(LTCP_SERV_HANDLE *h, int (*logCallback)(void *handle, int level, const char* fmt, va_list args), void* logHandler);
int ltcp_serv_init(LTCP_SERV_HANDLE *h, int port, int max_queue_len_ms, int max_clients, int verbosity);
int ltcp_serv_send(LTCP_SERV_HANDLE *h, char *bf, int sz, char *bf2, int sz2, uint32_t time);
int ltcp_serv_close(LTCP_SERV_HANDLE *h);
int ltcp_serv_change_port(LTCP_SERV_HANDLE *h, int port);
void ltcp_get_clients_list(LTCP_SERV_HANDLE *h, char *arg);

#endif
