#include <stdint.h>

#define TO_FOURCC_LE(d,c,b,a) (a<<24|b<<16|c<<8|d)
#define TO_FOURCC_BE(a,b,c,d) (a<<24|b<<16|c<<8|d)
	
// Usefule struct
typedef struct
{
	uint32_t size;
	uint32_t type;	
} mp4_atom_box ;


typedef struct
{
	uint32_t version : 8;
	uint32_t flag : 24;
} mp4_atom_flag;

typedef struct
{
	uint32_t size;
	uint32_t type;
	union {
		uint32_t value;
		mp4_atom_flag field;
	} flags	;
} mp4_atom_full_box ;



// MP4 atoms for Video

typedef struct  
{
	mp4_atom_box atom ;
	uint32_t major_brand;
	uint32_t minor_version;
	uint32_t compatible_brands[4];
} mp4_atom_ftyp ;

typedef struct
{
	mp4_atom_box atom;
} mp4_atom_moov;

typedef struct
{
	mp4_atom_box atom;
} mp4_atom_mdat;

typedef struct
{
	uint32_t size;
	uint32_t type;
	uint64_t largesize;
} mp4_atom_mdat64;

typedef struct
{
	mp4_atom_box atom;
} mp4_atom_free;

typedef struct
{
  mp4_atom_box atom;
} mp4_atom_edts;

typedef struct
{
  uint32_t segment_duration;
  uint32_t media_time;
  uint16_t media_rate_int;
  uint16_t media_rate_fraction;
} elst_entry;

typedef struct
{
  mp4_atom_full_box atom;
  uint32_t entry_count;
  elst_entry entry[2];
} mp4_atom_elst2entries;

typedef struct
{
	mp4_atom_full_box atom;
	uint32_t entry_count;
	uint32_t next_media_time;
	elst_entry * table;
} mp4_atom_elst;

typedef struct
{
	mp4_atom_full_box atom;
	uint32_t creation_time;
	uint32_t modification_time;
	uint32_t timescale;
	uint32_t duration;
	uint32_t rate;
	uint32_t volume;
	uint32_t reserved[2];
	uint32_t matrix[9];
	uint32_t pre_defined[6];
	uint32_t next_track_ID;
} mp4_atom_mvhd;
typedef struct
{
  mp4_atom_full_box atom;
  uint64_t creation_time;
  uint64_t modification_time;
  uint32_t timescale;
  uint64_t duration;
  uint32_t rate;
  uint32_t volume;
  uint32_t reserved[2];
  uint32_t matrix[9];
  uint32_t pre_defined[6];
  uint32_t next_track_ID;
} mp4_atom_mvhd64;

typedef struct 
{
	mp4_atom_box atom;
} mp4_atom_trak;

typedef struct 
{
	mp4_atom_full_box atom;
	uint32_t creation_time;
	uint32_t modification_time;
	uint32_t track_ID;
	uint32_t reserved;
	uint32_t duration;
	uint32_t reserved2[2];
	char layer[2];
	char alternate_group[2];
	uint32_t volume ;
	uint32_t matrix[9];
	uint32_t width;
	uint32_t height;
} mp4_atom_tkhd;

typedef struct 
{
  mp4_atom_full_box atom;
  uint64_t creation_time;
  uint64_t modification_time;
  uint32_t track_ID;
  uint32_t reserved;
  uint64_t duration;
  uint32_t reserved2[2];
  char layer[2];
  char alternate_group[2];
  uint32_t volume ;
  uint32_t matrix[9];
  uint32_t width;
  uint32_t height;
} mp4_atom_tkhd64;

typedef struct 
{
	mp4_atom_box atom;
} mp4_atom_mdia;

typedef struct
{
	mp4_atom_full_box atom;
	uint32_t creation_time;
	uint32_t modification_time;
	uint32_t timescale;
	uint32_t duration;
	uint32_t language;
} mp4_atom_mdhd;
typedef struct
{
  mp4_atom_full_box atom;
  uint64_t creation_time;
  uint64_t modification_time;
  uint32_t timescale;
  uint64_t duration;
  uint32_t language;
} mp4_atom_mdhd64;


typedef struct
{
	mp4_atom_full_box atom;
	uint32_t pre_defined ;
	uint32_t handler_type;
	uint32_t reserved[3];
	char name[18];
} mp4_atom_hdlr;

typedef struct 
{
	mp4_atom_box atom;
} mp4_atom_minf;

typedef struct
{
	mp4_atom_full_box atom;
	uint32_t graphicsmode;
	uint32_t opcolor;
} mp4_atom_vmhd;

typedef struct
{
	mp4_atom_box atom;
} mp4_atom_dinf;

typedef struct
{
	mp4_atom_full_box atom;
	//char * location;
	uint32_t location;
} mp4_atom_url;

typedef struct
{
	mp4_atom_full_box atom;
	uint32_t entry_count;
} mp4_atom_dref;

typedef struct
{
	mp4_atom_box atom;
} mp4_atom_stbl;

typedef struct 
{
	mp4_atom_full_box atom;
	uint32_t entry_count;
} mp4_atom_stsd;

typedef struct 
{
	uint32_t sample_count;
	uint32_t sample_delta;
} stts_entry;

typedef struct
{
	mp4_atom_full_box atom;
	uint32_t entry_count;	
	stts_entry * table;
} mp4_atom_stts;
#define STTS_SIZE (sizeof(mp4_atom_full_box)+4)

typedef struct
{ 
	mp4_atom_full_box atom;
	uint32_t entry_count;
	uint32_t * sample_number;	
} mp4_atom_stss;
#define STTS_SIZE (sizeof(mp4_atom_full_box)+4)

typedef struct 
{
	uint32_t first_chunk;
	uint32_t samples_per_chunk;
	uint32_t sample_description_index;
} stsc_entry;

typedef struct
{
	mp4_atom_full_box atom;
	uint32_t entry_count;
	stsc_entry * table;
} mp4_atom_stsc;
#define STSC_SIZE (sizeof(mp4_atom_full_box)+4)

typedef struct
{
	mp4_atom_full_box atom;
	uint32_t sample_size;
	uint32_t sample_count;
	uint32_t * entry_size;
} mp4_atom_stsz;
#define STSZ_SIZE (sizeof(mp4_atom_full_box)+8)

typedef struct
{
	mp4_atom_full_box atom;
	uint32_t entry_count;
	uint32_t * chunk_offset;
} mp4_atom_stco;
#define STCO_SIZE (sizeof(mp4_atom_full_box)+4)

typedef struct
{
	mp4_atom_full_box atom;
	uint32_t entry_count;
	uint64_t * chunk_offset;
} mp4_atom_co64;
#define CO64_SIZE (sizeof(mp4_atom_full_box)+4)


typedef struct
{
	mp4_atom_box atom;
	char configurationVersion;
	char AVCProfileIndication;
	char profile_compatibility;
	char AVCLevelIndication;
	char lengthSizeMinusOne;
	char numOfSequenceParameterSets;
	char sequenceParameterSetLength[2];
	char * sequence_entries;
	char numOfPictureParameterSets;
	char pictureParameterSetLength[2];
	char * picture_entries;
} mp4_atom_avcC;

typedef struct
{
	mp4_atom_box atom;
	uint32_t reserved;
	uint32_t data_reference_index;
	uint32_t pre_defined[4];
	uint32_t width_height;
	uint32_t horizresolution;
	uint32_t vertresolution;
	uint32_t reserved2;
	char frame_count[2];
	char compressorname[32];
	char depth[4];
} mp4_atom_avc1;

typedef struct
{
	uint32_t size;
	uint32_t type;
	uint32_t uuid[4];
	uint32_t data;
} mp4_atom_uuid;

typedef struct
{
	mp4_atom_box atom;
	mp4_atom_box atom2;
	unsigned int track_id;
} mp4_atom_tref;

typedef struct
{
  mp4_atom_ftyp  ftyp;
  mp4_atom_moov  moov;
  mp4_atom_mdat  mdat;
  int mdat_offset;
  mp4_atom_mvhd  mvhd;
  mp4_atom_trak  trak;
  mp4_atom_tkhd  tkhd;
  mp4_atom_tkhd64  tkhd64;
  mp4_atom_mdia  mdia;
  mp4_atom_mdhd  mdhd;
  mp4_atom_mdhd64  mdhd64;
  mp4_atom_hdlr  hdlr;
  mp4_atom_minf  minf;
  mp4_atom_vmhd  vmhd;
  mp4_atom_dinf  dinf;
  mp4_atom_dref  dref;
  mp4_atom_stbl  stbl;
  mp4_atom_stsd  stsd;
  mp4_atom_url  url;
  mp4_atom_avc1  avc1;
  mp4_atom_avcC  avcC;
  mp4_atom_uuid  uuid;
  mp4_atom_stts  stts;
  mp4_atom_stss  stss;
  mp4_atom_stsc  stsc;
  mp4_atom_stsz  stsz;
  mp4_atom_stco  stco;
  mp4_atom_co64  co64;
	mp4_atom_mdat64 mdat64;
  mp4_atom_mvhd64 mvhd64;
  mp4_atom_tref  tref;
  mp4_atom_edts  edts;
  mp4_atom_elst  elst;
  int timecode_enabled;
} mp4_header_video_handler;


// MP4 atoms for Audio

typedef struct
{
	mp4_atom_full_box atom;
	uint16_t balance;
	uint16_t reserved;
	
} mp4_atom_smhd;


typedef struct
{
  mp4_atom_box atom;
  unsigned char reserved1[6];
  uint16_t data_ref_index;
  uint16_t version;
  uint16_t revision;
  uint32_t vendor;
  uint16_t channelcount;
  uint16_t samplesize;
  uint16_t compression_id;
  uint16_t reserved3;
  unsigned int   samplerate;
  // for version 1 codec (mp4a):
  uint32_t samplesPerPacket;
  uint32_t bytesPerPacket;
  uint32_t bytesPerFrame;
  uint32_t bytesPerSample;
} mp4_atom_mp4a;

typedef struct
{
  mp4_atom_box atom;
  uint32_t data_format;
} mp4_atom_frma;

typedef struct
{
  mp4_atom_box atom;
} mp4_atom_wave;

typedef struct
{
  mp4_atom_box atom;
} mp4_atom_mp4a_end;

typedef struct _BaseDescriptor
{
  unsigned char tag;
  /* the first bit of each byte indicates if the next byte should be used */
  uint32_t size;
} BaseDescriptor;

typedef struct _DecoderSpecificInfoDescriptor
{
  BaseDescriptor base;
  unsigned char* data;
} DecoderSpecificInfoDescriptor;


typedef struct _DecoderConfigDescriptor {
  BaseDescriptor base;

  unsigned char object_type;

  /* following are condensed into streamType:
   * bit(6) streamType;
   * bit(1) upStream;
   * const bit(1) reserved=1;
  */
  unsigned char stream_type;

  unsigned char buffer_size_DB[3];
  uint32_t max_bitrate;
  uint32_t avg_bitrate;

  DecoderSpecificInfoDescriptor dec_specific_info;
} DecoderConfigDescriptor;

typedef struct _SLConfigDescriptor
{

  BaseDescriptor base;
  unsigned char predefined;              /* everything is supposed predefined */
} SLConfigDescriptor;

typedef struct _ESDescriptor
{
  BaseDescriptor base;

  uint16_t id;

  /* flags contains the following:
   * bit(1) streamDependenceFlag;
   * bit(1) URL_Flag;
   * bit(1) OCRstreamFlag;
   * bit(5) streamPriority;
   */
  unsigned char flags;

#if 0
  uint16_t depends_on_es_id;
  unsigned char  url_length;              /* only if URL_flag is set */
  unsigned char* *url_string;             /* size is url_length */
  uint16_t ocr_es_id;              /* only if OCRstreamFlag is set */
#endif

  DecoderConfigDescriptor dec_conf_desc;
  SLConfigDescriptor sl_conf_desc;

  /* optional remainder of ESDescriptor is not used */
} ESDescriptor;

typedef struct
{
  mp4_atom_full_box atom;
  //unsigned char   audiospecificconfig[2];
  ESDescriptor es_desc;

} mp4_atom_esds;

typedef struct
{
  mp4_atom_trak  trak;
  mp4_atom_tkhd  tkhd;
  mp4_atom_tkhd64  tkhd64;
  mp4_atom_mdia  mdia;
  mp4_atom_mdhd  mdhd;
  mp4_atom_mdhd64  mdhd64;
  mp4_atom_hdlr  hdlr;
  mp4_atom_minf  minf;
  mp4_atom_smhd  smhd;
  mp4_atom_dinf  dinf;
  mp4_atom_dref  dref;
  mp4_atom_url   url;
  mp4_atom_stbl  stbl;
  mp4_atom_stsd  stsd;
  mp4_atom_mp4a  mp4a;
  mp4_atom_esds  esds;
  mp4_atom_stts  stts;
  mp4_atom_stsc  stsc;
  mp4_atom_stsz  stsz;
  mp4_atom_stco  stco;
  mp4_atom_co64  co64;
  mp4_atom_frma  frma;
  mp4_atom_wave  wave;
  mp4_atom_mp4a_end end;
  mp4_atom_edts edts;
  mp4_atom_elst2entries elst;
} mp4_header_audio_handler;

// MOV atoms for timecode
typedef struct
{
	mp4_atom_box atom;
} mp4_atom_gmhd;

typedef struct
{
	mp4_atom_full_box atom;
	uint16_t graphics_mode;
	uint16_t opcolor_red;
	uint16_t opcolor_green;
	uint16_t opcolor_blue;
	uint16_t balance;
	uint16_t reserved;
} mp4_atom_gmin;

typedef struct
{
	mp4_atom_box atom;
} mp4_atom_tmcd;

typedef struct
{
	mp4_atom_full_box atom;
	uint16_t text_font;
	uint16_t text_face;
	uint16_t text_size;
	uint16_t text_color_red;
	uint16_t text_color_green;
	uint16_t text_color_blue;
	uint16_t background_color_red;
	uint16_t background_color_green;
	uint16_t background_color_blue;
	char font_name[8];
} mp4_atom_tcmi;

typedef struct
{
  mp4_atom_box atom;
  unsigned char reserved1[6];
  uint16_t data_ref_index;
  uint32_t reserved2[1];
  unsigned int  flags;
  unsigned int  timescale;
  unsigned int  frameduration;
  unsigned char counter_fps;
  unsigned char reserved3[1];
} mp4_atom_timecode;

typedef struct
{
  mp4_atom_trak  trak;
  mp4_atom_tkhd  tkhd;
  mp4_atom_tkhd64  tkhd64;
  mp4_atom_mdia  mdia;
  mp4_atom_mdhd  mdhd;
  mp4_atom_mdhd64  mdhd64;
  mp4_atom_hdlr  hdlr;
  mp4_atom_minf  minf;
  mp4_atom_dinf  dinf;
  mp4_atom_dref  dref;
  mp4_atom_url   url;
  mp4_atom_stbl  stbl;
  mp4_atom_stsd  stsd;
  mp4_atom_timecode  timecode;
//  mp4_atom_esds  esds;
  mp4_atom_stts  stts;
  mp4_atom_stsc  stsc;
  mp4_atom_stsz  stsz;
  mp4_atom_stco  stco;
  mp4_atom_co64  co64;
  mp4_atom_gmhd  gmhd;
  mp4_atom_gmin  gmin;
  mp4_atom_tmcd  tmcd;
  mp4_atom_tcmi  tcmi;
} mp4_header_timecode_handler;


// Prototypes :
mp4_header_video_handler* create_empty_video_header(uint32_t track_ID,
													uint32_t width, uint32_t height, int timecode_enabled, int encoder_freq);
mp4_header_audio_handler* create_empty_audio_header(uint32_t track_ID, int encoder_freq);
mp4_header_timecode_handler* create_empty_timecode_header(uint32_t track_ID, int encoder_freq);
int mp4_header_video_swap(mp4_header_video_handler * h);
int mp4_header_video_write(mp4_header_video_handler * h, int fd,int large_file, int no_edit_lists);
int mp4_header_audio_swap(mp4_header_audio_handler * h);
int mp4_header_timecode_swap(mp4_header_timecode_handler * h);
int mp4_header_audio_write(mp4_header_audio_handler * h, int fd, int large_file, int no_edit_lists);
int mp4_header_timecode_write(mp4_header_timecode_handler * h, int fd,int large_file);
void mp4_header_video_duration_update(mp4_header_video_handler * h,uint32_t duration);
void mp4_header_video_size_update(mp4_header_video_handler *h,int large_file, int no_edit_lists);
void mp4_audio_video_merge(mp4_header_video_handler *vh,mp4_header_audio_handler *ah,int large_file, int encoder_freq, int no_edit_lists);
void mp4_header_audio_duration_update(mp4_header_audio_handler * h,uint32_t duration);
void mp4_audio_header_size_update(mp4_header_audio_handler *h,int largefile, int no_edit_lists);
void mp4_timecode_header_size_update(mp4_header_timecode_handler *h,int large_file);
int mp4_header_movie_write(mp4_header_video_handler * h, int fd,int largefile);
void mp4_header_video_extended_duration_update(mp4_header_video_handler * h,uint32_t duration);
void mp4_header_audio_extended_duration_update(mp4_header_audio_handler * h,uint32_t duration);
void mp4_header_timecode_duration_update(mp4_header_timecode_handler * h,uint32_t duration);
void mp4_header_timecode_extended_duration_update(mp4_header_timecode_handler * h,uint32_t duration);
void mp4_timecode_video_merge(mp4_header_video_handler *vh,mp4_header_timecode_handler *ah,int large_file);
// Prototype for atoms
void mp4_stts_update(mp4_atom_stts * atom, uint32_t sample_count, uint32_t sample_delta);
void mp4_stts_update_duration(mp4_atom_stts * atom, uint32_t sample_count, uint32_t sample_delta);
void mp4_stss_update(mp4_atom_stss * atom, uint32_t sample_number);
void mp4_stsc_update(mp4_atom_stsc * atom, uint32_t first_chunk, uint32_t samples_per_chunk,uint32_t sample_description_index);
void mp4_stsz_update(mp4_atom_stsz * atom, uint32_t entry_size);
void mp4_stsz_update_fixedsize(mp4_atom_stsz * atom, uint32_t entry_size);
void mp4_stco_update(mp4_atom_stco * atom, uint32_t chunk_offset);
void mp4_co64_update(mp4_atom_co64 * atom, uint64_t chunk_offset);
void mp4_avcC_update(mp4_atom_avcC * atom, uint16_t spslen, char * sps , uint16_t ppslen, char * pps);
void mp4_elst_video_update(mp4_atom_elst * atom, uint32_t gap, uint32_t frame_duration);

