/*******************************************************************************
*
* The content of this file or document is CONFIDENTIAL and PROPRIETARY
* to Maxim Integrated Products.  It is subject to the terms of a
* License Agreement between Licensee and Maxim Integrated Products.
* restricting among other things, the use, reproduction, distribution
* and transfer.  Each of the embodiments, including this information and
* any derivative work shall retain this copyright notice.
*
* Copyright 2009 Maxim Integrated Products
* All rights reserved.
*
*******************************************************************************/

#ifndef __MP4WRITER_H
#define __MP4WRITER_H

#include <stdint.h>
#include "writestreamer.h"
#include <sys/types.h>
#include "mp4mux.h"
#include "tdprofiler.h"

// Error codes that threadedWriter can compare against and take appropriate action
// towards mp4 writer
#define MP4WRITER_ERR_INVALID_FD 2
#define MP4_CHECK_SPACE_EVERY_FRAMES 50

typedef enum
{
    MP4_WRITER_IOCTL_START = 0x01,
    MP4_WRITER_IOCTL_STOP = 0x02,
    MP4_WRITER_IOCTL_INFO = 0x03,
    MP4_WRITER_IOCTL_STATUS = 0x04,
    MP4_WRITER_IOCTL_FILENAME = 0x05,
} MP4_WRITER_IOCTL;


typedef enum
{
  MP4_WRITER_LARGE_FILE_ENABLE=10
}MP4WriterConfig;

//#define MP4WRITER_QUEUE_THREAD
#ifdef MP4WRITER_QUEUE_THREAD

#define MP4WRITER_QUEUE_LENGTH      38400
#define MP4WRITER_QUEUE_SEND_COUNT  15
#define RESTORE_VIDEO_QUEUE_COUNT   60
#define MAX_VIDEO_QUEUE_BYTES       (2*1024*1024)
#define AUDIO_BUF_LEN               (2*1024*1024)

#define TYPE_IMM 1
#define TYPE_AUDIO 2
#define TYPE_VIDEO 3

typedef struct
{
    int valid;
    int type;
    int len;

    char *buf;
    char data[8];

} MP4WRITER_QUEUE_DATA;

typedef struct
{
    int head;
    int tail;
    int max;
    int count;
    int finished;
    int video_sz;
    int audio_sz;

    pthread_mutex_t         lock;
    pthread_cond_t          ready;
    pthread_cond_t          done;

    MP4WRITER_QUEUE_DATA    data[MP4WRITER_QUEUE_LENGTH];

} MP4WRITER_QUEUE;

#endif

typedef struct
{
    int fd;
    int spsFound;
    uint16_t spslen;
    char * sps;
    int ppsFound;
    uint16_t ppslen;
    char * pps;
    int headerInit;
    int audhdrinit;
    int timeHeaderInit;
    unsigned int width ;
    unsigned int height;
    uint32_t frameTicks;
    uint32_t frameTicksPrev;
    uint32_t audioFrameTicksPrev;
    uint32_t audioTicks;
    int channelConfiguration;
    uint32_t QVideoHdrboxsize;
    uint32_t QAudioHdrboxsize;
    uint32_t frameNumber;
    uint32_t audsmplnumber;
    uint32_t timecodenumber;
    int samplingFrequencyIndex;
    uint64_t chunk_offset;
    mp4_header_video_handler * header_video;
    mp4_header_audio_handler * header_audio;
    mp4_header_timecode_handler * header_timecode;
    uint32_t audio_samplingfreq;
    uint32_t audio_channels;
    int large_file;
    uint64_t sizecheck;
    uint64_t sampleount;
    int timecode_enabled;
    char filename[256];
    char basefilename[256];
    char timecode[16];
    int tick_120;
    int pts_ticks;
    int dropframe;
    int fps;
    int rp188;
    int64_t first_pts_64;
    int64_t first_video_pts;
    int64_t first_audio_pts;
    int64_t prev_video_pts;
    int64_t prev_audio_pts;
    int close_req;
    int no_audio;
    int red_timecode;
    int first_video;
    int first_slice;
    int initial_p_frames;
    int no_edit_lists;
    int auto_start;

    int checkCount;
    tdProfiler profiler;

#ifdef MP4WRITER_QUEUE_THREAD
    MP4WRITER_QUEUE mp4writer_queue;
    int running;
    int sending_empty_frames;
    int last_audio_skipped;
    char audio_buf[AUDIO_BUF_LEN];
    char *audio_buf_ptr;
#endif

} MP4WRITER_HANDLE;

BITSTREAMWRITER_HANDLE  mp4WriterCreate();


// Writestreamer functions

int  mp4WriterOpen(BITSTREAMWRITER_HANDLE  h, const char *name);
int  mp4WriterClose(BITSTREAMWRITER_HANDLE  h);
int  mp4WriterNewBlock(BITSTREAMWRITER_HANDLE  h, char *buf, int len);
int  mp4WriterIOCTL(BITSTREAMWRITER_HANDLE  h, unsigned int cmd, void *arg);
int  mp4WriterShow(BITSTREAMWRITER_HANDLE h);
int  mp4WriterConfig(BITSTREAMWRITER_HANDLE h, int param, int value);
int  mp4WriterReset(MP4WRITER_HANDLE * h);

// Mp4 specific functions

int mp4OpenFile(MP4WRITER_HANDLE * h);
int mp4InitFile(MP4WRITER_HANDLE * h);
int mp4CloseFile(MP4WRITER_HANDLE * h);
int mp4MuxVideo(MP4WRITER_HANDLE * h,unsigned int sampleSize,int sampleType,int64_t cts);
int mp4MuxAudio(MP4WRITER_HANDLE * h,unsigned int sampleSize,int64_t cts);
int mp4MuxTimecodeInit(MP4WRITER_HANDLE * h);
int mp4MuxTimecode(MP4WRITER_HANDLE * h);
void mp4_esds_desc_write(int fd,mp4_atom_esds * esds);
void mp4_esds_desc_init(ESDescriptor * es_desc);
void mp4_esds_update_asc(mp4_atom_esds * esds,int asclength, unsigned char* asc);

#endif
