/*******************************************************************************
* 
* Teradek(C) 2009
* MPEG2 PES parsing routines
* 
* Questions: ilya.issenin@teradek.com
* 
*******************************************************************************/

#include <stdint.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdlib.h>
#include "ts_parse.h"
	
#ifndef PESPARSE_H
#define PESPARSE_H

#define PES_STREAM_ID_VIDEO 0xE0
#define PES_STREAM_ID_AUDIO_AAC 0xC0
#define PES_STREAM_ID_AUDIO_MP2 0xD0

#define PES_MAX_HEADER_LEN 270
	
// MPEG2 video elementary stream definitions:
#define MPEG2_SYNC_CODE							0x000001
#define MPEG2_SEQUENCE_HEADER_START_CODE              0xB3
#define MPEG2_SEQUENCE_END_CODE                       0xB7
#define MPEG2_PICTURE_START_CODE                      0x00
#define MPEG2_SLICE_START_CODE_FIRST                  0x01
#define MPEG2_SLICE_START_CODE_LAST                   0xAF
#define MPEG2_USER_DATA_START_CODE                    0xB2
#define MPEG2_SEQUENCE_HEADER_START_CODE              0xB3
#define MPEG2_SEQUENCE_ERROR_CODE                     0xB4
#define MPEG2_PICT_CODING_EXTENSION_START_CODE        0xB5
#define MPEG2_GOP_START_CODE                          0xB8
#define MPEG2_SYSTEM_START_CODE                       0xB9
#define MPEG2_SYSTEM_END_CODE                         0xFF
	
typedef struct {
	inp_info_t *inp_info;
	int (*status_update_func)(uint32_t status, inp_info_t *inp_info);
	uint8_t header_data[PES_MAX_HEADER_LEN];
	int header_filled;
	int do_sync;
	int same_pid;
	uint32_t pts33;
	uint32_t pts;
	uint16_t stream_id;
	int sync_point;
	int length;
	int cur_pos;
	ts_parse_stream_t ts_stream;
} pes_parse_stream_t;

// Init the stream descriptor
void pes_parse_init(pes_parse_stream_t *pes_stream, int (*inp_func)(uint8_t **buf_ret, int len, inp_info_t *inp_info), inp_info_t *inp_info,
		int (*tty_outp_func)(uint8_t *buf, int len, inp_info_t *inp_info),
		int (*status_update_func)(uint32_t status, inp_info_t *inp_info));

// Tells the parser to start from the beginning of PES that contains the first NAL of the frame
void sync_to_start_pes(pes_parse_stream_t *pes_stream);

// Tells the parser to start from the beginning of PES that contains the first NAL of the frame; look for the same TS PID as before
void sync_to_next_pes(pes_parse_stream_t *pes_stream);

// Tells the parser to init internal buffers since stream error was encountered
void pes_clean_on_error(pes_parse_stream_t *pes_stream);

// Extracts payload data from PES stream, then sets end_of_packet to 1, which means that this is the end of the frame (and QBOX packet).
// Returns the number of bytes actually read and pointer to the buffer where they are located.
// Call this function until end_of_packet gets set to 1.
// Retuned error values: 0 - no more data (end of file) or end_of_packet; < 0 - error in the stream (try to sync again)
// Limitaions: it is assumed that the whole PES header fits into 1 TS packet
int get_pes_data(pes_parse_stream_t *pes_stream, uint8_t **ptr, int *end_of_packet, int *pts_present, uint32_t *pts33, uint32_t *pts, 
	uint16_t *stream_id, int *sync_point);

#endif
