/*******************************************************************************
* 
* Teradek(C) 2009
* MPEG2 PES generation routines
* 
*******************************************************************************/

#include <stdint.h>
#include <unistd.h>

#include "cc_gen.h"

#ifndef PESGEN_H
#define PESGEN_H

#define PES_STREAM_ID_VIDEO 0xE0
//#define PES_STREAM_ID_AUDIO_AAC 0xC0
//#define PES_STREAM_ID_AUDIO_MP2 0xD0
#define PES_STREAM_ID_AUDIO_MP2 0xC0
#define PES_STREAM_ID_AUX 0xBF

#define ADTS_HEADER_LEN 7
#define PES_HEADER_LEN 14

typedef struct {
	uint8_t header_data[ADTS_HEADER_LEN];	// data to append before the beginning of payload (sync code for NALs or ADTS header for audio)
	int header_size;
	uint8_t adts_conf1, adts_conf2;
	uint8_t video_pes_id;
	uint8_t audio_pes_id;
	// captions
	void* captions;
	int cc_enabled;
	uint32_t prev_video_pts;
} pes_stream_t;

// init PES stuff
void pesInit(pes_stream_t *pes_stream/*, uint8_t video_pes_id, uint8_t audio_pes_id*/);
void pesClose(pes_stream_t *pes_stream);

// generates ADTS syncword and fixed and variable headers to add to AAC-LC raw data
// qmedAudioConfigData - pointer to the data after QMED header
int genAdtsHeader(pes_stream_t *pes_stream, uint8_t *ptr, int sampleSize, int audioConfigPresent, uint8_t *qmedAudioConfigData);

// Fills the buffer *ptr with PES packet(s) containing given NAL. 
// firstNAL: if the NAL is the first in QBOX (so that the timestamp from QBOX is valid)
// Returns number of bytes written
int genPesPacketsForNal(pes_stream_t *pes_stream, uint8_t *ptr, uint8_t *nalData, int nalDataSize, int streamID, uint32_t cts33, uint32_t cts, int firstNal);

// Fills the buffer *ptr with PES packet(s) containing given frame. Replaces NAL size bytes with sync code.
// firstNAL: if the NAL is the first in QBOX (so that the timestamp from QBOX is valid)
// Returns number of bytes written
int genPesPacketsForQboxFrame(pes_stream_t *pes_stream, uint8_t *ptr, uint8_t *data, int totalSize, int streamID, uint32_t cts33, uint32_t cts, int firstNal);

#endif
