/*******************************************************************************
* 
* Teradek(C) 2009
* MPEG2 transport stream library
* 
*******************************************************************************/

#ifndef __QBOXPARSE_H
#define __QBOXPARSE_H

#include <stdint.h>
#include <stdarg.h>
#include <syslog.h>
#include "tsgen.h"
#include "pesgen.h"
#include "refdesign.h"
#include "qboxutil.h"

typedef struct
{
	int spsFound;
	tstream_t *ts_stream;
	pes_stream_t *pes_stream;
	int firstVideo, firstAudioAAC, firstAudioMP2;
	uint8_t video_pes_id;
	uint8_t audio_pes_id;
	int buf_size;
	uint8_t *outbuf;
	int totalSz;
	char *buf_tmp;
	int buf_size_ticks;	// size of H.264 encoder (rate control) buffer in 90kHz ticks
	int one_pes_per_frame_mode;
	int ticks_120khz;
	uint64_t cts_64;
	int (*logCallback)(void *handle, int level, const char* fmt, va_list args);
	void* logHandler;
} qboxp_stream_t;

void qbox_parse_set_logging(qboxp_stream_t *qbox_stream, int (*logCallback)(void *handle, int level, const char* fmt, va_list args), void* logHandler);

void qbox_stream_init(qboxp_stream_t *qbox_stream, pes_stream_t *pes_stream, tstream_t *ts_stream, uint8_t video_pes_id, uint8_t audio_pes_id, 
	int buf_size_ticks, int buf_size, uint8_t *outbuf);

// parses one QBOX and generates PES packet(s)
int qbox_parse_new_block(qboxp_stream_t *qbox_stream, char *buf, int len);

// converts PES packet(s) to TS
void qbox_parse_gen_ts(qboxp_stream_t *qbox_stream, uint16_t pid);

// parses qbox and returns offset to the place where to insert closed captions
// also returns offset of the beginning of SEI NAL
// returns -1 if not found
int find_cc_insertion_place_in_qbox(char *qbox_buf, int qbox_len, int *nal_offset);

// parses data section of qbox and returns offset to the place where to insert closed captions
// also returns offset of the beginning of SEI NAL
// returns -1 if not found
int find_cc_insertion_place_in_h264_of_qbox(char *sample_buf, int sample_len, int *nal_offset);

// Adjusts the length of QBOX and NAL to accommodate for CC SEI record
void increase_qbox_and_nal_sizes(uint8_t *qbox_buf, int nal_offset, int cc_len);

// Adjusts the length of NAL to accommodate for CC SEI record
void increase_nal_size(uint8_t *buf, int nal_offset, int cc_len);

#endif
