/*******************************************************************************
* 
* Teradek(C) 2009
* QBOX generation routines
* 
* Questions: ilya.issenin@teradek.com
* 
*******************************************************************************/

#ifndef QBOXGEN_H
#define QBOXGEN_H

#define QBOX_BUFSIZE (2*1024*1024)
#define MAX_H264_HEADER_LEN 1024

#include <stdint.h>
#include "pes_parse.h"
#ifdef H264_PARSE
#include "h264_parse/h264_parse.h"
#endif
#ifdef MPEG2V_PARSE
#include "h264_parse/mp4av_mpeg3.h"
#endif

typedef struct {
	int (*outp_func)(uint8_t *buf, int len, inp_info_t *inp_info);
	int (*status_update_func)(uint32_t status, inp_info_t *inp_info);
	inp_info_t *inp_info;
	int high;
	pes_parse_stream_t pes_stream;
	uint8_t outbuf[QBOX_BUFSIZE];
	int sps_present;
	int qbox_stream_type;
	int qbox_stream_id;
	int err;
	uint8_t *saved_outptr;
	uint8_t *begin;
	uint8_t *end;
	uint8_t *next_frame_start;
	int saved_len;
	int saved_picts;
	int mpeg2_seq_header;
	uint32_t saved_code;
	int two_fields;
	int end_of_pes_state;
	int got_pict;
	uint32_t pts33;
	uint32_t pts;
	uint64_t pts_prev;
	uint16_t stream_id;
	int last_error;
	int video_qbox;
	int pts_present;
	uint8_t sps[MAX_H264_HEADER_LEN];
	int sps_cnt;
	uint8_t pps[MAX_H264_HEADER_LEN];
	int pps_cnt;
	int sei_cnt;
	int sls_cnt;
	uint8_t h264_header_tmp[MAX_H264_HEADER_LEN];
	uint32_t vmode_x;
	uint32_t vmode_y;
	int vmode_interlaced;
	float vmode_fps;
	char vmode_str[32];
	char vmode_str_mpeg2v[32];
	int mpeg2v_hdr_cnt;
	int c147_filter;
    int aac_packets_left; // 1(yes) or 0(no)
    int mp2audio_packets_left; // 1(yes) or 0(no)
    int mp2audio_slots_found;
    int mp2audio_slots_confidence;
    uint8_t adts[7];
#ifdef H264_PARSE
	h264_decode_t dec;
#endif
#ifdef MPEG2V_PARSE
	mpeg2v_decode_t mpeg2_dec;
#endif
} qbox_stream_t;


int gen_qbox(qbox_stream_t *qbox_stream);
int gen_single_qbox(qbox_stream_t *qbox_stream);
void qbox_reset_h264_parser(qbox_stream_t *qbox_stream);
void qbox_reset_mpeg2v_parser(qbox_stream_t *qbox_stream);

// Init the stream descriptor
void qbox_gen_init(qbox_stream_t *qbox_stream, 
		int (*inp_func)(uint8_t **buf_ret, int len, inp_info_t *inp_info),
		int (*outp_func)(uint8_t *buf, int len, inp_info_t *inp_info), 
		inp_info_t *inp_info,
		int (*tty_outp_func)(uint8_t *buf, int len, inp_info_t *inp_info));

void qbox_gen_init_stat(qbox_stream_t *qbox_stream, 
		int (*inp_func)(uint8_t **buf_ret, int len, inp_info_t *inp_info),
		int (*outp_func)(uint8_t *buf, int len, inp_info_t *inp_info), 
		inp_info_t *inp_info,
		int (*tty_outp_func)(uint8_t *buf, int len, inp_info_t *inp_info),
		int (*status_update_func)(uint32_t status, inp_info_t *inp_info));

#endif
