/*******************************************************************************
 * Copyright (c) 2010 eInfochips - All rights reserved.
 *
 * This software is authored by eInfochips and is eInfochips' intellectual
 * property, including the copyrights in all countries in the world.
 * This software is provided under a license to use only with all other rights,
 * including ownership rights, being retained by eInfochips.
 *
 * This file may not be distributed, copied, or reproduced in any manner,
 * electronic or otherwise, without the written consent of eInfochips.
 ******************************************************************************/

/*******************************************************************************
 *
 * File Name    :   ServerCommon.h
 *
 * Description  :   This file contains all enums, defines, structures, functions
 *                  used commonly between all modules - API Interface Module,
 *                  Chunk Module, AMF Module, FLV Module and Handshake Module
 *
 * History:
 *  MAY/13/2010, ARJ,   Created the file.
 *  MAY/14/2010, PRP,   Started RTMP implementation
 ******************************************************************************/

#ifndef _INCLUDE_SERVER_COMMON_H
#define _INCLUDE_SERVER_COMMON_H

/*********************
 * Include Files
 *********************/
#include "ServerInclude.h"
#include "ServerBasicTypes.h"
#include "RtmpServerApi.h"
#include "ServerAmf.h"

#include "RtmpQueue.h"
#include "RtmpPlatform.h"

/*********************
 * Defines and Macros
 *********************/
#include "ServerCommonDefines.h"

/*********************
 * IOCTL interface handler
 *********************/
#include "ioctrl.h"

/*********************
 *    Structures
 *********************/

typedef struct _RtmpServer          RtmpServer;
typedef struct _RtmpSession         RtmpSession;
typedef struct _RtmpClient          RtmpClient;
typedef struct _RtmpCodecInfo       RtmpCodecInfo;
typedef struct _RtmpVdCodecInfo     RtmpVdCodecInfo;
typedef struct _RtmpAdCodecInfo     RtmpAdCodecInfo;
typedef struct _TunnelRTMPReqMsg    TunnelRTMPReqMsg;
typedef struct _TunnelRTMPResMsg    TunnelRTMPResMsg;
typedef struct _RtmpRecvStrmPkt     RtmpRecvStrmPkt;
typedef struct _RtmpSendStrmPkt     RtmpSendStrmPkt;
typedef struct _HTTPMsgHeaders      HTTPMsgHeaders;
typedef struct _RtmpData            RtmpData;
typedef struct _PublishResult       PublishResult;
typedef struct _PublishError        PublishError;
typedef struct _PublishStatus       PublishStatus;
typedef struct _PublishServer       PublishServer;
typedef struct _PublishAuthentication       PublishAuthentication;
typedef struct _PublishPoint        PublishPoint;
typedef struct _RtmpPacketBuffer    RtmpPacketBuffer;
typedef struct _RtmpSenderQueue     RtmpSenderQueue;
typedef struct _RtmpDebug           RtmpDebug;
typedef struct _RtmpAdaptiveBitrate RtmpAdaptiveBitrate;
typedef struct _RtmpSenderMonitor   RtmpSenderMonitor;
typedef struct _RtmpSenderMonitorEntry      RtmpSenderMonitorEntry;
typedef struct _RtmpCC              RtmpCC;
typedef struct _PublishThreadArg    PublishThreadArg;
typedef struct _RtmpStatus          RtmpStatus;
typedef struct _RtmpLog             RtmpLog;

struct _RtmpLog
{
    ioctlLogCallbackFunction func;
    ioctlLogCallbackHandler  handler;
};

struct _RtmpStatus
{
    int self_id;
    int source_id;
    int status_sink_id;

    int redis_id;
    int redis_seq_num;

    char session[128];

    ioctlCallbackFunction ioctlCallback;
    void* ioctlCallbackHandler;
};

struct _PublishAuthentication
{
    INT32 required;
    UINT8 authmod[1024];
    UINT8 reason[1024];
    UINT8 user[1024];
    UINT8 password[1024];
    UINT8 salt[1024];
    UINT8 challenge[1024];
    UINT8 response[1024];
    UINT8 opaque[1024];
    UINT8 session_id[1024];
};

struct _PublishPoint
{
    UINT8 ip[1024];
    UINT8 file[1024];
    INT32 port;
};

struct _RtmpCC
{
    INT32 enable;
    void *captions;
};

struct _PublishResult
{
    DOUBLE transactionId;
};

struct _PublishError
{
    INT32 code;
    UINT8 level[1024];
    UINT8 shortStr[1024];
    UINT8 longStr[1024];
    DOUBLE transactionId;
};

struct _PublishStatus
{
    UINT8 level[1024];
    UINT8 shortStr[1024];
    UINT8 longStr[1024];
    DOUBLE transactionId;
};

struct _PublishServer
{
    PublishPoint pubPoints[2];
    INT32 pubPointInUse;

    UINT8 username[1024];
    UINT8 password[1024];
    UINT8 streamName[1024];
    UINT8 user_agent[1024];
    UINT8 copyright[1024];
    UINT8 device[1024];

    UINT8 platform[1024];
    UINT8 platformArgs[4096];

    UINT8 proxy[1024];

    INT32 auto_reconnect;
    INT32 auto_start;
    INT32 retries;

    INT32 wait;
    INT32 timeout;

    PublishState state;
    PublishLogLevel logLevel;

    PublishAuthentication auth;
    RtmpPlatform rtmpPlatform;

    PublishResult PublishResultPtr;
    PublishError PublishErrorPtr;
    PublishStatus PublishStatusPtr;

    RtmpStatus *status;
};

struct _PublishThreadArg
{
    RtmpServer *rtmpServer;
    PublishServer server;
    INT8 session[1024];
};

struct _RtmpPacketBuffer
{
    SHANDLE rtmpSession; // Pointer to rtmp Session
    UINT8 *dataBuff; // Pointer to audio/video data
    INT32 dataSize; // Size of audio/video data
    INT32 deviceFd; // Zero copy device fd
    VOID *vZeroPtr; // Zero copy pointer
    UINT64 prevTimeStamp; // time stamp from the encoder
    UINT64 timeStampDiff; // difference between last sent frame of the same type and this packet
    RTMP_EXACT_MSG_TYPE exactMsgTypeId; // msg type
    STREAM_TYPE strmType; // data type (audio/video)

    // Doubly linked list
    RtmpPacketBuffer *prevPacket;
    RtmpPacketBuffer *nextPacket;
};

struct _RtmpSenderMonitorEntry
{
    STREAM_TYPE                 type;
    struct timeval              time;
    INT32                       size;

    INT32                       currentTotal;
};

struct _RtmpSenderMonitor
{
    RtmpSenderMonitorEntry      *queue;
    INT32                       tail;
    INT32                       head;

    INT32                       size;
    INT32                       total;
    INT32                       average;

    INT32                       bitrate;

    INT32                       sumTotal;

    struct timeval              lastUpdate;
    struct timeval              lastMsg;
    struct timeval              firstUptime;
    struct timeval              lastUptime;
};

struct _RtmpDebug
{
    FILE    *fp;
    UINT64  first_pts;
    INT32   frameCount;
    UINT64  uptime;
    INT32   dropCount;
    INT32   temperature;
    INT32   qp;

    UINT8   file;
    UINT8   burnin;

    UINT8   path[MEDIUM_BUFFER_SIZE];
    INT32   intervalFrames;
};

struct _RtmpAdaptiveBitrate
{
    INT32   limit;
    INT32   current;
    UINT64  lastIncreaseTimestamp;
    UINT64  lastDecreaseTimestamp;
    UINT8   warningIssued;

    INT32   min; // bytes (absolute min)

    INT32   recoverThreshold; // ms
    INT32   recoverStepPercent; // %
    INT32   recoverMinStep; // bytes
    INT32   recoverInterval; // sec

    INT32   adjustThreshold; // ms
    INT32   adjustPercent; // % of measured bandwidth

    INT32   triggerThreshold; // ms
    INT32   decreasePercent; // % of measured bandwidth
    INT32   decreaseInterval; // sec

    INT32   margin; // % jitter margin (% of limit)
};

struct _RtmpSenderQueue
{
    SHANDLE rtmpSession;

    /* previous audio/video timestamp (to calculate deltas) */
    UINT64                      prevAudioTimeStamp;
    UINT64                      prevVideoTimeStamp;

    /* RTMP timestamps for audio/video packets */
    UINT64                      lastSentVideoTimeStamp;
    UINT64                      lastSentAudioTimeStamp;

    /* Video Packet buffer */
    RtmpPacketBuffer            *VideoBuffer; // Pointer to static mem
    INT32                       videoHead; //
    INT32                       videoTail;
    RtmpPacketBuffer            *VideoBufferPtr; // Moving pointer of rotating buffer
    RtmpPacketBuffer            *VideoBufferEndPtr;
    INT32                       VideoBufferSize; // Current size

    /* Audio Packet buffer */
    RtmpPacketBuffer            *AudioBuffer; // Pointer to static mem
    INT32                       audioHead;
    INT32                       audioTail;
    RtmpPacketBuffer            *AudioBufferPtr;
    RtmpPacketBuffer            *AudioBufferEndPtr;
    INT32                       AudioBufferSize;

    UINT8                       highWaterMark;
    UINT64                      highWaterMarkTimeStamp;
    struct timeval              highWaterMarkTime;

    RtmpDebug                   *rtmpDebug;     /* Debug settings */
    RtmpAdaptiveBitrate         *adaptiveBitrate;
    RtmpSenderMonitor           *bandwidthMonitor;
    SHANDLE                     avcencHandle;

    pthread_t                   ThreadId;
    pthread_mutex_t             ThreadLock;
    pthread_cond_t              ThreadReady;
};

struct _RtmpServer
{
    UINT8                       ServerIP[MAX_IP_LEN];/* Server IP address     */
    INT32                       WinAckSize;     /* Window ack. size           */
    INT32                       MaxVideoBitrate;/* Maximum video bitrate      */
    INT32                       StoCChunkSize;  /* Server to client chunk size*/
    INT32                       PreferedStoCChunkSize;  /* Server to client chunk size*/

    UINT16                      RtmpPort;       /* Rtmp server port           */
    SOCKET                      RtmpSocket;     /* Socket to accept Rtmp client
                                                   connection                 */
    THREAD_HANDLE               RtmpSrvThrdId;  /* Rtmp server thread ID      */

    UINT16                      RtmptPort;      /* Rtmpt server port          */
    SOCKET                      RtmptSocket;    /* Socket to accept Rtmpt client
                                                   Connection                 */
    THREAD_HANDLE               RtmptSrvThrdId; /* Rtmpt server thread ID     */

    UINT8                       connectThread;

    /* Live session List                                                      */
    INT32                       LiveSessionCount;/* Number of live session    */
    LOCK_HANDLE                 LiveSessionListLock;/* Live session list lock */
    RtmpSession                 *LiveSessionList;/* Live session list         */
    RtmpSession                 *LiveSessionListEndPtr;/* Live session list   */

    RtmpServer                  *PrevServer;    /* Next server in list        */
    RtmpServer                  *NextServer;    /* Previous server in list    */

    RtmpLog                     *log;           /* Logging handle             */
};

struct _RtmpSession
{
    UINT8                       *SessionName;   /* Name of session            */
    SESSION_TYPE                SessionType;    /* Type of session            */
    RTMP_SESSION_STATE          SessionState;   /* State of session           */
    AV_FLAG                     AvFlag;         /* Audio video flag           */
    RtmpCodecInfo               *CodecInfo;     /* Information of codec       */

    INT32                       ClientListCount;/* Number of connected clients*/
    RtmpClient                  *ClientList;    /* List of connected clients  */
    RtmpClient                  *ClientListEndPtr;/* List of connected clients*/
    LOCK_HANDLE                 ClientListLock; /* Client list lock           */

    RtmpServer                  *RtmpServerPtr; /* Pointer to Rtmp server     */

    RtmpCC                      *RtmpCC;

    INT32                       bwMonitor;      /* Flag to enable the monitor */
    INT32                       tick_120;
    UINT32                      dscp;
    UINT32                      avcencHandle;

    INT32                       senderType;      /* Flag to use the sender queue */

    RtmpSenderQueue             *rtmpQueue;     /* Sender queue */
    RtmpQueue                   *threadedQueue;

    PublishServer               publishServer; // Rtmp Session owns publishServer structure, so that we don't have to allocate it separately
    PublishServer               *currentPublishServer;
    PublishServer               *activePublishServer;

    RtmpSession                 *NextSession;   /* Next session in list       */
    RtmpSession                 *PrevSession;   /* Previous session in list   */
};

struct _RtmpCodecInfo
{
    VIDEO_CODEC_TYPE            VideoType;      /* Type of video codec        */
    INT32                       VdBitrate;      /* Video bitrate              */
    INT32                       VdConfigDataSize;/* Size of video config data */
    UINT8                       *VdConfigData;  /* Pointer to video config    */
    RtmpVdCodecInfo             *VdCodecInfo;   /* Video codec information    */

    AUDIO_CODEC_TYPE            AudioType;      /* Type of audio codec        */
    INT32                       AdBitrate;      /* Audio bitrate              */
    INT32                       AdConfigDataSize;/* Size of audio config data */
    UINT8                       *AdConfigData;  /* Pointer to audio config    */
    RtmpAdCodecInfo             *AdCodecInfo;   /* Audio codec information    */

    UINT8                       *MetaData;      /* Pointer to metadata info   */
    INT32                       MetaDataSize;   /* Size of metadata info      */
};

struct _RtmpAdCodecInfo
{
    INT32                       SampleRate;     /* Sample rate of audio       */
    INT32                       NoOfChannles;   /* Number of channels         */
    INT32                       Bitrate;        /* Bitrate of audio           */
    UINT8                       AacConfigPkt [4];/* Pointer to AAC config pkt  */
    INT32                       AacConfigPktSize;/* Size of AAC config packet */
};


struct _RtmpVdCodecInfo
{
    INT32                       Height;         /* Height of video            */
    INT32                       Width;          /* Width of video             */
    INT32                       Bitrate;        /* Bitrate of video           */
    INT32                       ProfileId;      /* Profile Id of video        */
    INT32                       ProfileCompat;  /* Profile compatibility      */
    INT32                       LevelId;        /* Level Id of video          */
    INT32                       fr1001;         /* Fractional framerate flag  */
    INT32                       fr;             /* Framerate                  */
    INT32                       kfi;            /* Key-Frame interval         */
    UINT8                       *AvccBox;       /* Pointer to avcC data       */
    INT32                       AvccBoxSize;    /* Size of avcC data          */
};

struct _RtmpClient
{
    UINT8                       *ClientIP;      /* IP address of
                                                            connected client  */
    CLIENT_TYPE                 ClientType;     /* Type of client connected   */
    SOCKET                      ClientSocket;   /* Socket of connected client */
    UINT16                      ClientPort;     /* Port of connected client   */
    CLIENT_STATE                ClientState;    /* State of client            */
    CLIENT_RTMPT_STATE          ClientRtmptState;/* RTMPT state of client     */
    THREAD_HANDLE               ClientThreadID; /* Thread ID of client        */
    INT32                       WinAckSize;     /* Window ack. size           */
    BOOL                        PauseFlag;      /* Flag to indicate pause
                                                   and resume                 */
    UINT64                      ClientStartTime;/* Up Time of client          */
    INT32                       CtoSChunkSize;  /* Client to Server chunk size*/
    UINT8                       ClientId [IDENTIFIER_LEN+1];/* Client ID for
                                                            RTMP session      */
    UINT8                       *ReqSessionName;/* Name of request session    */

    INT32                       NetConnStrmId;  /* NetConnection - Stream Id  */
    INT32                       NetStreamStrmId;/* NetStream - Streamd Id     */
    INT32                       NetStreamVdChunkId;/* Video stream chunk Id   */
    INT32                       NetStreamAdChunkId;/* Audio stream chunk Id   */

    TunnelRTMPReqMsg            *TunnelReqMsg;  /* RTMPT request message      */
    TunnelRTMPResMsg            *TunnelResMsg;  /* RTMPT response message     */

    INT32                       RecvStrmListCount;/* Total recv pkts in list  */
    RtmpRecvStrmPkt             *RecvStrmList;  /* Recv packet list           */
    RtmpRecvStrmPkt             *RecvStrmListEndPtr;/* Pointer to last packet */
    LOCK_HANDLE                 RecvStrmListLock;/* Lock for recv packet list */

    INT32                       SendStrmListCount;/* Total send pkts in list  */
    RtmpSendStrmPkt             *SendStrmList;  /* Send packet list           */
    RtmpSendStrmPkt             *SendStrmListEndPtr;/* Pointer to last packet */
    LOCK_HANDLE                 SendStrmListLock;/* Lock for send packet list */

    RtmpSession                 *RtmpSessionPtr;/* Pointer to rtmp session    */
    RtmpServer                  *RtmpServerPtr; /* Pointer to rtmp server     */

    RtmpClient                  *NextClient;    /* Pointer to next client     */
    RtmpClient                  *PrevClient;    /* Pointer to prev client     */

    PublishServer               *PublishPtr;
};

struct _RtmpRecvStrmPkt
{
    INT32                       ChunkStreamId;  /* Chunk stream identifier    */

    INT32                       PktCountId;     /* Packet count identifier    */
    PACKET_STATUS               PktStatus;      /* Packet status              */
    INT32                       PayloadBytesReceived; /* Paylod bytes received*/
    MESSAGE_HEADER_TYPE         ChunkMsgHdrType;/* Type of message header     */
    BASIC_HEADER_TYPE           ChunkBasicHdrType; /* Type of basic header    */

    RTMP_MESSAGE_TYPE           TypeId;         /* Type id of received packet */
    UINT8                       TimeStamp [3];  /* Timestamp of received pkt  */
    UINT8                       TimeStampDelta [3]; /* Timestamp delta        */
    UINT8                       ExtTimeStamp [4];/* Extented TS of recv pkt   */
    UINT8                       StrmId [4];     /* Streamd Id of recv pkt     */
    INT32                       PayloadSize;    /* Size of payload            */
    UINT8                       *PayloadData;   /* Pointer to payload data    */

    union
    {
        AmfObj                  *Object;        /* Amfobject type payload data*/
        UINT8                   *MiscData;      /* Misc payload data          */
        EventBuff               Event;          /* Event type payload data    */
    }ParsedPayload;

    RtmpRecvStrmPkt             *NextStrmPkt;   /* Pointer to next stream pkt */
    RtmpRecvStrmPkt             *PrevStrmPkt;   /* Pointer to prev stream pkt */
};

struct _RtmpSendStrmPkt
{
    INT32                       ChunkStreamId;   /* Chunk stream identifier   */

    RTMP_MESSAGE_TYPE           TypeId;         /* Type id of received packet */
    RTMP_EXACT_MSG_TYPE         ExactTypeId;    /* Exact message type Id      */

    UINT64                      PresTimestamp;  /* Present system time        */
    UINT8                       TimeStamp[3];   /* Timestamp of send stream
                                                   pkt                        */
    UINT8                       TimeStampDelta[3];/* Timestamp delta          */
    UINT8                       ExtTimeStamp[4];/* Extented TS of send stream
                                                   pkt                        */
    UINT8                       ExtTimeStampDelta[4];/* Extended TS delta of
                                                   send stream packet         */
    UINT8                       StrmId[4];      /* Streamd Id of send stream
                                                   pkt                        */
    INT32                       PayloadSize;    /* Size of payload            */
    UINT8                       *PayloadData;   /* Pointer to payload data    */

    struct _RtmpSendStrmPkt     *NextStrmPkt;   /* Pointer to next stream pkt */
    struct _RtmpSendStrmPkt     *PrevStrmPkt;   /* Pointer to prev stream pkt */
};

struct _RtmpData
{
    INT32                       Size;           /* Size of data               */
    UINT8                       *Data;          /* Pointer to data            */
};

struct _HTTPMsgHeaders
{
    UINT8                       *HdrName;       /* Message Header Name        */
    UINT8                       *HdrValue;      /* Message Header Value       */

};

struct _TunnelRTMPResMsg
{
    INT32                       StatusCode;     /* Status code                */
    INT32                       SeqNumber;      /* Sequence number            */
    INT32                       HeadersCount;   /* Number of headers          */
    UINT8                       *ReasonPhrase;  /* Reason of status           */
    UINT8                       *HttpVersion;   /* Http version               */
    HTTPMsgHeaders              MsgHeaders [MAX_HEADER_COUNT]; /* Headers     */
};

struct _TunnelRTMPReqMsg
{
    INT32                       SeqNumber;      /* Sequence number            */
    INT32                       HeadersCount;   /* Number of headers          */
    UINT8                       *MsgURI;        /* Message URI                */
    UINT8                       *HttpVersion;   /* Http version               */
    TUNNEL_RTMP_METHOD          Method;         /* RTMPT method name          */
    HTTPMsgHeaders              MsgHeaders [MAX_HEADER_COUNT];/* Headers      */

    INT32                       NoOfBytesReceived; /* No. of bytes received   */
    INT32                       ContentLength;  /* Content length             */
};

/************************
 * Function declarations
 ************************/

/* Thread for accepting connections from clients on RTMP                      */
INT32 AcceptRtmpConnectionThread (VOID *threadArgs);

/* Thread for accepting connections from clients on RTMPT                     */
INT32 AcceptRtmptConnectionThread (VOID *threadArgs);

/* Function to validate session handle                                        */
INT32 ValidateRtmpSessionHandle (RtmpServer *rtmpServer, RtmpSession *rtmpSession);

/* Function to validate stream name as received from client                   */
INT32 ValidateRtmpStreamName (RtmpClient *clientPtr, INT32 streamNameSize,
                              UINT8 *streamName, RtmpSession **validSessionPtr);

/* Function to check whether session name already exists or not               */
INT32 IsSessionNameExists (RtmpServer *rtmpServer, UINT8 *sessionName);

/* Function to read RTMP Message                                              */
RtmpRecvStrmPkt* ReadRtmpMessage (RtmpClient *clientPtr);

/* Function to check whether recv packet exists or not in list                */
INT32 IsRecvStrmPktExists (RtmpClient *clientPtr, RtmpRecvStrmPkt *recvStrmPkt);

/* Function to decide which type of message header to be generated            */
MESSAGE_HEADER_TYPE DecideMsgHdrType (RtmpSendStrmPkt *curSendPkt,
                                      RtmpSendStrmPkt *lastSendPkt);

/* Function to get received packet from list                                  */
RtmpRecvStrmPkt* GetRecvPktFromList (RtmpClient *clientPtr, INT32 chunkStrmId);

/* Function to get send packet from list                                      */
RtmpSendStrmPkt *GetSendPktFromList (RtmpClient *clientPtr, INT32 chunkStrmId);

/* Function to add Rtmp receive packet in list                                */
INT32 AddRecvPkt (RtmpClient *clientPtr, RtmpRecvStrmPkt *recvStrmPkt);

/* Function to add Rtmp send packet in list                                   */
INT32 AddSendPkt (RtmpClient *clientPtr, RtmpSendStrmPkt *sendStrmPkt);

/* Function to remove Rtmp receive packet in list                             */
INT32 RemoveRecvPkt (RtmpClient *clientPtr, RtmpRecvStrmPkt *recvStrmPkt);

/* Function to remove Rtmp send packet in list                                */
INT32 RemoveSendPkt (RtmpClient *clientPtr, RtmpSendStrmPkt *sendStrmPkt);

/* Function to create new rtmp packet                                         */
RtmpRecvStrmPkt* CreateNewRtmpPkt (RtmpClient *clientPtr,
                                   UINT32 dataBuffSize, UINT8 *dataBuff,
                                   INT32 payloadSizeInHdr, INT32 chunkMsgHdrType,
                                   BASIC_HEADER_TYPE chunkBasicHdrType,
                                   INT32 chunkStrmId);

/* Function to get payload header size from last same stream id packet        */
INT32 GetPayloadHdrSize (RtmpClient *clientPtr, INT32 chunkStrmId);

/* Function to get message type id from last same stream id packet            */
RTMP_MESSAGE_TYPE GetMsgTypeId (RtmpClient *clientPtr, INT32 chunkStrmId);

/* Function to parse received request message                                 */
INT32 ParseRequestMsg (RtmpClient *clientPtr, RtmpRecvStrmPkt *recvStrmPkt,
                       RTMP_EXACT_MSG_TYPE *ExactMsgTypeId);

/* Function to parse command request message                                  */
INT32 ParseCmdMsgReq (RtmpClient *clientPtr, RtmpRecvStrmPkt *recvStrmPkt,
                        RTMP_EXACT_MSG_TYPE *exactMsgTypeId);

/* Function to parse user control request message                             */
INT32 ParseUsrCtrlMsgReq (RtmpClient *clientPtr, RtmpRecvStrmPkt *recvStrmPkt,
                        RTMP_EXACT_MSG_TYPE *exactMsgTypeId);

/* Function to process request message, generate and send response message    */
INT32 ProcessSend (RtmpClient *clientPtr, RtmpRecvStrmPkt *recvStrmPkt,
                    RTMP_EXACT_MSG_TYPE exactMsgTypeId);

/* Function to generate rtmp response message                                 */
RtmpSendStrmPkt* GenerateRtmpMsg (RtmpClient *clientPtr,
                                  RtmpRecvStrmPkt *recvStrmPkt,
                                  RTMP_EXACT_MSG_TYPE exactMsgTypeId);

/* Function to generate response message of command message                   */
RtmpSendStrmPkt *GenerateCmdMsgRes (RtmpClient *clientPtr,
                                    RtmpRecvStrmPkt *recvStrmPkt,
                                    RTMP_EXACT_MSG_TYPE exactMsgTypeId);

/* Function to generate response message of user control message              */
RtmpSendStrmPkt *GenerateUsrCtrlMsgRes (RtmpClient *clientPtr,
                                        RtmpRecvStrmPkt *recvStrmPkt,
                                        RTMP_EXACT_MSG_TYPE exactMsgTypeId);

/* Function to generate response message of protocol control message          */
RtmpSendStrmPkt *GenerateProtoCtrlMsgRes (RtmpClient *clientPtr,
                                          RTMP_EXACT_MSG_TYPE exactMsgTypeId);

/* Function to generate audio/video message                                   */
RtmpSendStrmPkt *GenerateRtmpMediaMsg (RtmpClient *clientPtr,
                                      RTMP_EXACT_MSG_TYPE exactMsgTypeId,
                                      INT32 bufferSize, UINT8 *buffer,
                                      UINT64 timeStamp);


/* Function to generate data message                                          */
RtmpSendStrmPkt *GenerateRtmpDataMsg (RtmpClient *clientPtr,
                                        RtmpRecvStrmPkt *recvStrmPkt,
                                        RTMP_EXACT_MSG_TYPE exactMsgTypeId);

/* Function to send generated message to connected client                     */
INT32 SendMsgToClient (RtmpClient *clientPtr, RtmpSendStrmPkt *sndStrmPkt,
                       RTMP_EXACT_MSG_TYPE exactMsgTypeId);

/* Function to send data message to all connected clients                     */
INT32 SendMsgToConnectedClients (RtmpSession *rtmpSession,
                                 UINT8 *dataBuff, INT32 dataSize,
                                 INT32 deviceFd, VOID *vZeroPtr,
                                 UINT64 timeStamp,
                                 RTMP_EXACT_MSG_TYPE exactMsgTypeId);

/* Function to write data in amf object                                       */
INT32 WriteCmdMsgPayload (RtmpClient *clientPtr, RtmpSendStrmPkt *sndStrmPkt,
                          UINT8 *cmdName, DOUBLE transactionId,
                          UINT8 *fmsVer, DOUBLE capability, DOUBLE mode,
                          UINT8 *levelVal, UINT8 *codeVal, UINT8 *descVal,
                          UINT8 *objEnc, DOUBLE strmId);

/* Function to check whether any client is connected to session or not        */
INT32 IsAnyClientConnected (RtmpSession *rtmpSession);

/* Function to add client in session                                          */
INT32 AddClientInSession (RtmpSession *rtmpSession, RtmpClient *clientPtr);

/* Function to send message to specified socket using sendmsg                 */
INT32 SendMessage (RtmpClient *clientPtr, RtmpData *rtmpData, INT32 dataCount,
                   RtmpSendStrmPkt *sndStrmPkt, UINT8 sendFileFlag);

/* Function to send message to specified socket using sendfile                */
INT32 SendFile (RtmpClient *clientPtr, RtmpData *rtmpData, INT32 rtmpDataCnt,
                   RtmpSendStrmPkt *sndStrmPkt, INT32 deviceFd, VOID *vZeroPtr);

/* Function to generate chunk header                                          */
RtmpData* GenerateChunkHdr (RtmpClient *clientPtr,
                            RtmpSendStrmPkt *sndStrmPkt,
                            RTMP_EXACT_MSG_TYPE exactMsgTypeId,
                            MESSAGE_HEADER_TYPE msgHdrType);

/* Function to generate type-0 message header                                 */
INT32 GenRtmpType0MsgHdr (RtmpSendStrmPkt *sndStrmPkt, RtmpData *outBuff);

/* Function to generate type-1 message header                                 */
INT32 GenRtmpType1MsgHdr (RtmpSendStrmPkt *sndStrmPkt, RtmpData *outBuff);

/* Function to generate type-2 message header                                 */
INT32 GenRtmpType2MsgHdr (RtmpSendStrmPkt *sndStrmPkt, RtmpData *outBuff);

/* Function to generate type-3 message header                                 */
INT32 GenRtmpType3MsgHdr (INT32 strmId, RtmpData *outBuff);

/* Function to create new RtmpClient                                          */
RtmpClient* CreateNewRtmpClient (RtmpServer *rtmpServer, CLIENT_TYPE cType,
                                 SOCKET cSocket, UINT8 *cIp, UINT16 cPort);

/* Function to destroy RtmpClient                                             */
INT32 DestroyRtmpClient (RtmpClient* rtmpClient);

/* Thread to serve connected clients on RTMP                                  */
INT32 ServeRtmpClient (VOID *threadArgs);

/* Thread to serve connected clients on RTMPT                                 */
INT32 ServeRtmptClient (VOID *threadArgs);

/* Function to add audio information in session                               */
INT32 AddAudioInfo (RtmpSession *rtmpSession, RtmpAudioInfo *adInfo);

/* Function to parse audio configuration information                          */
INT32 ParseAdConfigData (RtmpSession *rtmpSession);

/* Function to add video information in session                               */
INT32 AddVideoInfo (RtmpSession *rtmpSession, RtmpVideoInfo *vdInfo);

/* Function to parse video configuration information                          */
INT32 ParseVdConfigData (RtmpSession *rtmpSession);

/* Function to serve client in case of error                                  */
VOID ServeClientForError (RtmpClient *clientPtr, RtmpRecvStrmPkt *recvStrmPkt,
                          INT32 intErrNo, RTMP_EXACT_MSG_TYPE exactMsgTypeId);

/* Function to start sending data after sending pre-required rtmp messages    */
INT32 StartSendingData (RtmpClient *clientPtr, RtmpRecvStrmPkt *recvStrmPkt,
                        RTMP_EXACT_MSG_TYPE exactMsgTypeId);

/* Function to read RTMP message bytes from the given socket                  */
INT32 ReadNoOfBytes (RtmpClient *clientPtr, UINT8 *buffer, INT32 bufferSize);

/* Function to read HTTP message                                              */
INT32 ReadHttpMessage (RtmpClient *clientPtr, UINT8 *dataBuff, INT32 *dataSize);

/* Function to identify HTTP method                                           */
TUNNEL_RTMP_METHOD IdentifyMethod (UINT8 *buffer);

/* Function to find specific character in buffer                              */
INT32 FindCharInBuffer (UINT8 *buffer,  UINT8 findChar,
                         UINT8 endChar,  INT32 bufLen);

/* Function to parse HTTP request message                                     */
TunnelRTMPReqMsg* ParseHttpReqMsg (RtmpClient *clientPtr, UINT8 *dataBuff,
                                    INT32 dataSize);

/* Initialize Mutex                                                           */
INT32 RtmpMutexInit (LOCK_HANDLE *CriticalSection);

/* Function to lock critical section                                          */
INT32 RtmpMutexLock (LOCK_HANDLE *CriticalSection);

/* Function to unlock critical section                                        */
INT32 RtmpMutexUnlock (LOCK_HANDLE *CriticalSection);

/* Destroy Mutex                                                              */
INT32 RtmpMutexDestroy (LOCK_HANDLE *CriticalSection);

/* Get IP address of the given interface                                      */
INT32 RtmpGetIP (RtmpLog *log, UINT8 *nwInterface, UINT8 *ipAddr);

/* Close the specified socket                                                 */
INT32 RtmpCloseSocket (RtmpLog *log, SOCKET *socketId);

/* Function to create thread                                                  */
INT32 RtmpThreadCreate (THREAD_HANDLE *threadHandle, THREAD_FUNC threadFunction,
                       VOID* threadArg);

/* Function to exit the thread                                                */
INT32 RtmpThreadExit (THREAD_HANDLE threadHandle);

INT32 RtmpThreadCancel (THREAD_HANDLE threadHandle);

/* Function to detach the thread                                              */
INT32 RtmpThreadDetach (THREAD_HANDLE ThreadHandle);

/* Function to wait for thread completion                                     */
INT32 RtmpWaitForThreadCompletion (THREAD_HANDLE threadHandle);

/* Function to get self thread handle                                         */
THREAD_HANDLE RtmpGetSelfThreadHandle ();

/* Function to receive given nomber of bytes from socket                      */
INT32 RtmpReadBytes (RtmpLog *log, SOCKET clientSocket, UINT8 *buffer, INT32 bufferSize);

/* Function to send given number of bytes to the socket                       */
INT32 RtmpSendBytes (RtmpLog *log, SOCKET clientSocket, UINT8 *buffer, INT32 bufferSize);

/* Function to send data pointed by msg header to the socket                  */
INT32 RtmpSendMsgBytes (RtmpLog *log, SOCKET clientSocket, struct msghdr *msgPtr, INT32 flags);

/* Fuction to get system time                                                 */
UINT64 RtmpGetSystemTime();

/* Function to ignore SIGPIPE in case of socket error                         */
void HandleSignal(void);

extern INT32 TeradekSyslog(char *module, char *fmt, ...);

void RtmpLogPrint(RtmpLog *logPtr, UINT8 level, const char *format, ...);

#endif  /* _INCLUDE_SERVER_COMMON_H */

/*******************************************************************************
 * Copyright (c) 2010 eInfochips - All rights reserved.
 *
 * This software is authored by eInfochips and is eInfochips' intellectual
 * property, including the copyrights in all countries in the world.
 * This software is provided under a license to use only with all other rights,
 * including ownership rights, being retained by eInfochips.
 *
 * This file may not be distributed, copied, or reproduced in any manner,
 * electronic or otherwise, without the written consent of eInfochips.
 ******************************************************************************/
