
#ifndef _INCLUDE_SERVER_COMMON_DEFINES_H
#define _INCLUDE_SERVER_COMMON_DEFINES_H

#include "errno.h"

/*********************
 * Defines and Macros
 *********************/

#define DBG_LEVEL_1 1
#define DBG_LEVEL_2 2
#define DBG_LEVEL_3 3
#define DBG_LEVEL_4 4
#define DBG_LEVEL_5 5

#define RTMP_DEBUG_PRINT(dbgLevel, format, ...) \
            if(dbgLevel <= DEBUG_LEVEL) \
                fprintf (stdout, "RTMPWriter: DEBUG: " format, ## __VA_ARGS__);

/* Macros for logging                                                         */
#define RTMP_LOG_ERROR(log, fmt, ...) RtmpLogPrint(log, LOG_ERR, "ERROR: "fmt, ## __VA_ARGS__)
#define RTMP_LOG_INFO(log, fmt, ...) RtmpLogPrint(log, LOG_INFO, "INFO: "fmt, ## __VA_ARGS__)

#define SET_ERROR_CODE(errorCode, errorNo) \
                        if(errorCode) \
                            *errorCode = errorNo;

#define FREE(pointer)   if(pointer) \
                        {\
                            free (pointer);\
                            pointer = NULL;\
                        }

/* Macros to read from buffer                                                 */
#define READ_INT16(data) ((*(data) << 8) | *(data+1))
#define READ_INT24(data) ((*(data) << 16) | (*(data+1) << 8) | *(data+2))
#define READ_INT32(data) ((*(data) << 24) | (*(data+1) << 16) | (*(data+2) << 8) | *(data+3))

/* Macros to write in buffer                                                  */
#define WRITE_INT32_LE(data, buffer) { *(buffer)    = (data & 0x000000ff);\
                                       *(buffer+1)  = (data & 0x0000ff00) >> 8;\
                                       *(buffer+2)  = (data & 0x00ff0000) >> 16;\
                                       *(buffer+3)  = (data & 0xff000000) >> 24; \
                                     }

#define WRITE_INT32(data, buffer)    { *(buffer)    = (data & 0xff000000) >> 24;\
                                       *(buffer+1)  = (data & 0x00ff0000) >> 16;\
                                       *(buffer+2)  = (data & 0x0000ff00) >> 8;\
                                       *(buffer+3)  = (data & 0x000000ff); \
                                     }


#define WRITE_INT24(data, buffer)    { *(buffer)    = (data & 0x00ff0000) >> 16;\
                                       *(buffer+1)  = (data & 0x0000ff00) >> 8;\
                                       *(buffer+2)  = (data & 0x000000ff); \
                                     }

#define WRITE_INT16(data, buffer)    { *(buffer)    = (data & 0x0000ff00) >> 8;\
                                       *(buffer+1)  = (data & 0x000000ff); \
                                     }

#define GET_CHUNK_BASIC_HDR_SIZE(chunkStrmId, headerSize) \
                                   if ((chunkStrmId > 0) && (chunkStrmId < 64))\
                                    {\
                                        headerSize =  CHUNK_BASIC_HDR0_SIZE;\
                                    }\
                                    else if (chunkStrmId <= 319)\
                                    {\
                                        headerSize = CHUNK_BASIC_HDR1_SIZE;\
                                    }\
                                    else if (chunkStrmId <= 65599)\
                                    {\
                                        headerSize = CHUNK_BASIC_HDR2_SIZE;\
                                    }\
                                    else\
                                    {\
                                        headerSize = RTMP_FAIL;\
                                    }

#define INVALID_SOCKET              (SOCKET)(~0)    /* Invalid socket value   */
#define SOCKET_ERROR                (-1)            /* Socket error value     */

#define TRUE                        0               /* Macro for True         */
#define FALSE                       1               /* Macro for False        */

#define DFT_STOC_CHUNK_SIZE         128             /* Default server to client
                                                        chunk size            */
#define MAX_STOC_CHUNK_SIZE         4096            /* Max server to client
                                                        chunk size            */
#define DFT_CTOS_CHUNK_SIZE         128             /* Default client to server
                                                        chunk size            */
#define MAX_LISTEN_CLIENT           10              /* Maximum listen clients */
#define SMALL_BUFFER_SIZE           64              /* Small buffer size      */
#define MEDIUM_BUFFER_SIZE          1024            /* Medium buffer size     */
#define MAX_IP_LEN                  32              /* Length of IP           */
#define IN_ADDR_ANY                 "0.0.0.0"       /* IP address any         */
#define MAX_HEADER_COUNT            16              /* Maximum header counts  */
#define MAX_RTMP_HDR_SIZE           18              /* RTMP header size       */
#define HANDSHAKE_PACKET_SIZE       1536            /* Handshake packet size  */
#define IDENTIFIER_LEN              8               /* Length of identifier   */
#define DFT_VBITRATE                6000000         /* Default bitrate        */
#define BITRATE_VARIATION           500000          /* Bitrate variation      */
#define MAX_TIME_STAMP              0xFFFFFF        /* Max Time stamp         */
#define BW_LIMIT_TYPE               2               /* Bandwidth limit - soft */
#define INITIAL_CHUNK_STRM_ID       4               /* Start with this stream
                                                        id after play received*/
#define PROTO_CTRL_STREAM_ID        0x02            /* Stream ID for
                                                        protocol control
                                                        messages              */
#define SUPPORT_VID_ALL             0x00FF          /* Supports all video codecs*/
#define SUPPORT_VID_H264            0x0080          /* Supports H264 video    */
#define SUPPORT_SND_ALL             0x0FFF          /* Supports all audio
                                                       codecs                 */
#define SUPPORT_SND_AAC             0x0400          /* Supports AAC audio     */
#define NAL_SPS                     7               /* NAL type - SPS         */
#define NAL_PPS                     8               /* NAL type - PPS         */

#define MESSAGE_TYPE0_HDR_SIZE      11              /* Message Type 0         */
#define MESSAGE_TYPE1_HDR_SIZE      7               /* Message Type 1         */
#define MESSAGE_TYPE2_HDR_SIZE      3               /* Message Type 2         */
#define MESSAGE_TYPE3_HDR_SIZE      0               /* Message Type 3         */

#define CHUNK_BASIC_HDR0_SIZE       1               /* Header size of type 0  */
#define CHUNK_BASIC_HDR1_SIZE       2               /* Header size of type 1  */
#define CHUNK_BASIC_HDR2_SIZE       3               /* Header size of type 2  */


#define RESULT                      "_result"       /* _result                */
#define ERROR                       "_error"        /* _error                 */
#define ONSTATUS                    "onStatus"      /* onStatus               */
#define VOD                         "vod"           /* vod                    */
#define LIVE                        "live"          /* live                   */
#define ONBANDWIDTHDONE             "onBWDone"      /* onBWDone               */
#define RELEASE_STREAM              "releaseStream" /* releaseStream          */
#define ONFCPUBLISH                 "onFCPublish"   /* onFCPublish            */
#define RTMPSAMPLEACCESS            "|RtmpSampleAccess" /* |RtmpSampleAccess  */
#define CONNECT                     "connect"       /* connect                */
#define CREATESTREAM                "createStream"  /* createStream           */
#define PUBLISH                     "publish"       /* publish                */
#define PLAY                        "play"          /* play                   */
#define PAUSE                       "pause"         /* pause                  */
#define DELETESTREAM                "deleteStream"  /* deleteStream           */
#define APP                         "app"           /* app                    */
#define FLASHVER                    "flashVer"      /* flashVer               */
#define SWFURL                      "swfUrl"        /* swfUrl                 */
#define TCURL                       "tcUrl"         /* tcUrl                  */
#define FPAD                        "fpad"          /* fpad                   */
#define AUDIOCODECS                 "audioCodecs"   /* audioCodecs            */
#define VIDEOCODECS                 "videoCodecs"   /* videoCodecs            */
#define PAGEURL                     "pageUrl"       /* pageUrl                */
#define OBJECTENCODING              "objectEncoding"/* objectEncoding         */
#define SETCHALLENGE                "setChallenge"  /* setChallenge           */
#define CLIENTLOGIN                 "ClientLogin"   /* clientLogin            */
#define ONCLIENTLOGIN               "onClientLogin" /* onClientLoing          */
#define CHECKBW                     "_checkbw"      /* _checkbw               */

#define USTREAM_PING                "ping"          /* ustream ping           */
#define USTREAM_PONG                "pong"          /* ustream pong           */
#define MODULE_INFO                 "moduleInfo"    /* moduleInfo             */
#define START_PUBLISH               "startPublish"  /* startPublish           */
#define STOP_PUBLISH                "stopPublish"   /* stopPublish            */

// Ustream Recording
#define USTREAM_RECORD_STARTED          "recordStarted"
#define USTREAM_RECORD_READY            "recordReady"
#define USTREAM_RECORD_ERROR            "recordError"
#define USTREAM_RECORD_TIMER_WARNING    "recordTimerWarning"
#define USTREAM_RECORD_LIMIT_EXCEEDED   "recordLimitExceeded"
#define USTREAM_RECORD_SAVED            "recordSaved"

#define FMSVER                      "fmsVer"        /* fmsVer                 */
#define FLASH_SERVER_VERSION        "FMS/3,5,1,516" /* FMS/3,5,1,516          */
#define CAPABILITIES                "capabilities"  /* capabilities           */
#define CAPABILITY_VALUE            31.0            /* Value of capability    */
#define MODE                        "mode"          /* mode                   */
#define MODE_VALUE                  1.0             /* Value of mode          */
#define LEVEL                       "level"         /* level                  */
#define LEVEL_STATUS                "status"        /* Value of level - status*/
#define LEVEL_ERROR                 "error"         /* Value of level - error */
#define CODE                        "code"          /* code                   */
#define DESCRIPTION                 "description"   /* description            */
#define DETAILS                     "details"       /* details                */
#define CLIENT_ID                   "clientid"      /* clientid               */

#define CODE_CONN_SUCCESS_VALUE     "NetConnection.Connect.Success"
                                                    /* Netconnection Success  */
#define CODE_CONN_CLOSE_VALUE       "NetConnection.Connect.Close"
                                                    /* Netconnection Close    */
#define CODE_CONN_FAIL_VALUE        "NetConnection.Connect.Failed"
                                                    /* Netconnection Failed   */
#define CODE_STRM_PLAY_RESET        "NetStream.Play.Reset" /* Play reset      */
#define CODE_STRM_DATA_START        "NetStream.Data.Start" /* Data start      */
#define CODE_STRM_PLAY_START        "NetStream.Play.Start" /* Play start      */
#define CODE_STRM_PLAY_STOP         "NetStream.Play.Stop"  /* Play stop       */
#define CODE_STRM_NOT_FOUND         "NetStream.Play.StreamNotFound"
                                                          /* Stream not found */
#define CODE_STRM_PAUSE_NOTIFY      "NetStream.Pause.Notify"
                                                          /* Stream paused    */
#define CODE_STRM_UNPAUSE_NOTIFY    "NetStream.Unpause.Notify"
                                                          /* Stream unpaused  */

#define PLAYING_AND_RESETTING       "Playing and resetting"/* Description     */
#define STARTED_PLAYING             "Started playing"      /* Playing started */
#define STOPPED_PLAYING             "Stopped playing"      /* Playing stopped */
#define STREAM_NOT_FOUND            "Stream not found"     /* Stream not found*/
#define SUCCESS_DESC_VALUE          "Connection succeeded" /* Connection
                                                                    succeeded */
#define CLOSE_DESC_VALUE            "Connection closed" /* Connection
                                                                    closed    */
#define FAIL_DESC_VALUE             "Connection failed" /* Connection
                                                                       failed */
#define PAUSE_DESC_VALUE            "Pausing"       /* Pausing stream         */
#define UNPAUSE_DESC_VALUE          "Unpausing"     /* Unpausing stream       */
#define RTMPT_OPEN                  "open"          /* RTMPT open             */
#define RTMPT_IDLE                  "idle"          /* RTMPT close            */
#define RTMPT_SEND                  "send"          /* RTMPT send             */
#define RTMPT_CLOSE                 "close"         /* RTMPT send             */

/*********************
 * Typedefs and Enums
 *********************/

#ifdef LINUX
#ifndef _INCLUDE_RTPRTSP_H
#ifndef COMMON_TYPEDEF
typedef INT32  (*THREAD_FUNC) (void *);
#endif
#endif
#endif

typedef enum _RTMP_SESSION_STATE
{
    SESSION_INIT_STATE = 0,                 /* Session State - Initialization */
    SESSION_READY_STATE,                    /* Session State - Ready state    */
    SESSION_INVALID_STATE                   /* Session State - Invalid state  */

} RTMP_SESSION_STATE;

typedef enum _SESSION_TYPE
{
    LIVE_SESSION,                           /* Session Type - LIVE session    */
    VOD_SESSION                             /* Session Type - VOD session     */

} SESSION_TYPE;

typedef enum _AV_FLAG
{
    AUDIO_ONLY,                             /* Flag - Audio only              */
    VIDEO_ONLY,                             /* Flag - Video only              */
    AUDIO_VIDEO_BOTH,                       /* Flag - Audio Video both        */
    INVALID                                 /* Flag - Invalid                 */

} AV_FLAG;

typedef enum _CLIENT_TYPE
{
    RTMP_CLIENT,                            /* Type of Client - RTMP          */
    RTMP_TUNNEL_CLIENT                      /* Type of Client - RTMPT         */

} CLIENT_TYPE;

typedef enum _CLIENT_STATE
{
    CLIENT_INIT_STATE,                      /* Client State - Initialization  */
    CLIENT_VERSION_SENT_DONE,               /* Client State - Version sent    */
    CLIENT_HANDSHAKE_DONE,                  /* Client State - Handshake done  */
    CLIENT_CONNECT_SENT,                    /* Client State - Connect sent    */
    CLIENT_CONNECT_DONE,                    /* Client State - Connect done    */
    CLIENT_CREATESTREAM_SENT,               /* Client State - Createstream sent*/
    CLIENT_CREATESTREAM_DONE,               /* Client State - Createstream done*/
    CLIENT_PUBLISH_SENT,                       /* Client State - Publish sent*/
    CLIENT_PUBLISH_DONE,                       /* Client State - Publish done*/
    CLIENT_PLAY_DONE,                       /* Client State - Play done       */
    CLIENT_PAUSE_DONE,                      /* Client State - Pause done      */
    CLIENT_STOP_DONE,                       /* Client State - Stop done       */
    CLIENT_DELETESTREAM_DONE,               /* Client State - DeleteStream done*/
    CLIENT_DESTROY_STATE,                   /* Client State - Destroying      */
    CLIENT_INVALID_STATE                    /* Client State - Invalid         */

} CLIENT_STATE;

typedef enum _CLIENT_RTMPT_STATE
{
    CLIENT_RTMPT_OPEN_DONE,                 /* RTMPT Client state - Open done */
    CLIENT_RTMPT_IDLE_DONE,                 /* RTMPT Client state - Idle done */
    CLIENT_RTMPT_SEND_DONE,                 /* RTMPT Client state - Send done */
    CLIENT_RTMPT_CLOSE_DONE,                /* RTMPT Client state - Close done*/
    CLIENT_RTMPT_INVALID_STATE              /* RTMPT Client state - Invalid   */

} CLIENT_RTMPT_STATE;

typedef enum _PACKET_STATUS
{
    PACKET_INCOMPLETE,                      /* Status of packet - Incomplete  */
    PACKET_COMPLETE,                        /* Status of packet - Complete    */
    PACKET_PROCESSED                        /* Status of packet - Processed   */

} PACKET_STATUS;

typedef enum _MESSAGE_HEADER_TYPE
{
    MSG_HEADER_TYPE0  = 0,                  /* Message Header Type 0          */
    MSG_HEADER_TYPE1  = 1,                  /* Message Header Type 1          */
    MSG_HEADER_TYPE2  = 2,                  /* Message Header Type 2          */
    MSG_HEADER_TYPE3  = 3                   /* Message Header Type 3          */

} MESSAGE_HEADER_TYPE;

typedef enum _BASIC_HEADER_TYPE
{
    BASIC_HEADER_TYPE0,                     /* Basic Header Type 0            */
    BASIC_HEADER_TYPE1,                     /* Basic Header Type 1            */
    BASIC_HEADER_TYPE2                      /* Basic Header Type 2            */

} BASIC_HEADER_TYPE;

typedef enum _RTMP_MESSAGE_TYPE
{
    RTMP_FAIL = -1,
    AMF0_SET_CHUNK_SIZE             = 0x1,  /* Message - Set chunk size       */
    AMF0_ABORT_MESSAGE              = 0x2,  /* Message - Abort message        */
    AMF0_ACKNOWLEDGEMENT            = 0x3,  /* Message - Acknowledgement      */
    AMF0_USER_CONTROL_MESSAGE       = 0x4,  /* Message - User control message */
    AMF0_WINDOW_ACKNOWLEDGEMENT_SIZE= 0x5,  /* Message - Win Ack size         */
    AMF0_SET_PEER_BANDWIDTH         = 0x6,  /* Message - Set peer bandwidth   */
    AMF0_AUDIO_MESSAGE              = 0x8,  /* Message - Audio message        */
    AMF0_VIDEO_MESSAGE              = 0x9,  /* Message - Video message        */
    AMF0_DATA_MESSAGE               = 0x12, /* Message - Data message         */
    AMF0_SHARED_OBJECT_MESSAGE      = 0x13, /* Message - Shared object message*/
    AMF0_COMMAND_MESSAGE            = 0x14, /* Message - Command message      */
    AMF0_AGGREGATE_MESSAGE          = 0x16  /* Message - Aggregate message    */

} RTMP_MESSAGE_TYPE;

typedef enum _RTMP_EXACT_MSG_TYPE
{
    COMMAND_MSG_RESULT,                    /* Command Message - RESULT           */
    COMMAND_MSG_ERROR,                    /* Command Message - ERROR            */
    COMMAND_MSG_CONNECT,                /* Command Message - CONNECT          */
    COMMAND_MSG_ONSTATUS,                /* Command Message - ONSTATUS          */
    COMMAND_MSG_CREATE_STREAM,          /* Command Message - CREATESTREAM     */
    COMMAND_MSG_RELEASE_STREAM,
    COMMAND_MSG_FCPUBLISH,              /* Command Message - FCPUBLISH          */
    COMMAND_MSG_ONFCPUBLISH,              /* Command Message - ONFCPUBLISH      */
    COMMAND_MSG_PUBLISH,                  /* Command Message - PUBLISH          */
    COMMAND_MSG_PLAY,                   /* Command Message - PLAY             */
    COMMAND_MSG_PAUSE,                  /* Command Message - PAUSE            */
    COMMAND_MSG_ON_BW_DONE,             /* Command Message - ON BANDWIDTH DONE*/
    COMMAND_MSG_PLAY_RESET_DONE,        /* Command Message - PLAY RESET       */
    COMMAND_MSG_PLAY_START_DONE,        /* Command Message - PLAY START       */
    COMMAND_MSG_PLAY_STOP_DONE,         /* Command Message - PLAY STOP        */
    COMMAND_MSG_STREAM_NOT_FOUND,       /* Command Message - STREAM NOT FOUND */
    COMMAND_MSG_CONNECTION_FAILED,      /* Command Message - UNSUPPORTED      */
    COMMAND_MSG_DELETESTREAM,           /* Command Message - DELETESTREAM     */
    COMMAND_MSG_SET_CHALLENGE,          /* Command Message - SET CHALLENGE    */
    COMMAND_MSG_CLIENT_LOGIN,           /* Command Message - CLIENT LOGIN     */
    COMMAND_MSG_ON_CLIENT_LOGIN,        /* Command Message - ON CLIENT LOGIN  */
    COMMAND_MSG_CHECK_BW,               /* Command Message - CHECK BW  */

    // Ustream commands
    COMMAND_MSG_USTREAM_PING,           /* Command Message - USTREAM PING     */
    COMMAND_MSG_USTREAM_PONG,           /* Command Message - USTREAM PONG     */
    COMMAND_MSG_START_PUBLISH,          /* Command Message - START_PUBLISH    */
    COMMAND_MSG_STOP_PUBLISH,           /* Command Message - STOP_PUBLISH     */
    COMMAND_MSG_USTREAM_RECORD_START,
    COMMAND_MSG_USTREAM_RECORD_STOP,
    COMMAND_MSG_USTREAM_RECORD_SAVE,
    COMMAND_MSG_USTREAM_RECORD_CANCEL,
    COMMAND_MSG_USTREAM_RECORD_STARTED,
    COMMAND_MSG_USTREAM_RECORD_ERROR,
    COMMAND_MSG_USTREAM_RECORD_READY,
    COMMAND_MSG_USTREAM_RECORD_TIMER_WARNING,
    COMMAND_MSG_USTREAM_RECORD_LIMIT_EXCEEDED,
    COMMAND_MSG_USTREAM_RECORD_SAVED,


    USR_CTRL_MSG_STREAM_BEGIN,          /* User Control Message - STREAM BEGIN*/
    USR_CTRL_MSG_STREAM_EOF,            /* User Control Message - STREAM EOF  */
    USR_CTRL_MSG_STREAM_DRY,            /* User Control Message - STREAM DRY  */
    USR_CTRL_MSG_SET_BUFFER_LEN,        /* User Control Message - SET BUFFER LEN  */
    USR_CTRL_MSG_STREAM_IS_RECORDED,    /* User Control Message - STREAM IS RECORDED    */
    USR_CTRL_MSG_PING_REQUEST,          /* User Control Message - PING REQUEST*/
    USR_CTRL_MSG_PING_RESPONSE,         /* User Control Message -PING RESPONSE*/


    PROTO_CTRL_MSG_SET_CHUNK_SIZE,      /* Protocol Control Message - SET CHUNK SIZE    */
    PROTO_CTRL_MSG_ABORT,               /* Protocol Control Message - ABORT   */
    PROTO_CTRL_MSG_ACKNOWLEDGEMENT,     /* Protocol Control Message - ACKNOWLEDGEMENT   */
    PROTO_CTRL_MSG_WIN_ACK_SIZE,        /* Protocol Control Message - WIN ACK SIZE      */
    PROTO_CTRL_MSG_SET_PEER_BANDWIDTH,  /* Protocol Control Message - SET PEER BANDWIDTH    */

    DATA_MSG_DATA_START,                /* Data Message - Data Start          */
    DATA_MSG_METADATA,                  /* Data Message - MetaData            */
    DATA_MSG_RTMP_SAMPLE_ACCESS,        /* Data Message - RTMP Sample Access  */
    MEDIA_MSG_VIDEO,                    /* Media Message - Video              */
    MEDIA_MSG_AUDIO,                    /* Media Message - Audio              */
    NOT_SUPPORTED_MSG_TYPE              /* Not supported message type         */

} RTMP_EXACT_MSG_TYPE;

typedef enum _RTMP_PUBLISH_TRANSACTION_ID
{
    RTMP_PUBLISH_TRANSACTION_ID_RELEASE_STREAM = 2,
    RTMP_PUBLISH_TRANSACTION_ID_FCPUBLISH = 3,
    RTMP_PUBLISH_TRANSACTION_ID_CREATE_STREAM = 4,
    RTMP_PUBLISH_TRANSACTION_ID_START_PUBLISH = 5
} RTMP_PUBLISH_TRANSACTION_ID;

typedef enum _USR_CTRL_MSGS
{
    STREAM_BEGIN        = 0x00,         /* STREAM BEGIN                       */
    STREAM_EOF          = 0x01,         /* STREAM EOF                         */
    STREAM_DRY          = 0x02,         /* STREAM DRY                         */
    SET_BUFFER_LEN      = 0x03,         /* SET BUFFER LENGTH                  */
    STREAM_IS_RECORDED  = 0x04,         /* STREAM IS RECORDED                 */
    PING_REQUEST        = 0x06,         /* PING REQUEST                       */
    PING_RESPONSE       = 0x07          /* PING RESPONSE                      */

} USR_CTRL_MSGS;

typedef enum _TUNNEL_RTMP_METHOD
{
    RTMP_METHOD_OPEN = 0,               /* RTMPT METHOD - OPEN                */
    RTMP_METHOD_IDLE,                   /* RTMPT METHOD - IDLE                */
    RTMP_METHOD_SEND,                   /* RTMPT METHOD - SEND                */
    RTMP_METHOD_CLOSE,                  /* RTMPT METHOD - CLOSE               */
    RTMP_METHOD_INVALID                 /* RTMPT METHOD - INVALID             */

} TUNNEL_RTMP_METHOD;

typedef enum _INTERNAL_ERROR_CODES
{
    INVALID_STREAM_NAME = -111,         /* Invalid stream name                */
    UNSUPPORTED_MESSAGE_TYPE            /* Message not supported              */

} INTERNAL_ERROR_CODES;

typedef enum _PublishLogLevel {
    PUBLISH_LOG_OFF = 0,
    PUBLISH_LOG_LEVEL_1,
    PUBLISH_LOG_LEVEL_2,
    PUBLISH_LOG_LEVEL_3
}PublishLogLevel;

typedef enum _PublishState {
    PUBLISH_STATE_READY = 0,
    PUBLISH_STATE_CONNECTING,
    PUBLISH_STATE_AUTHENTICATING,
    PUBLISH_STATE_CONNECTED,
    PUBLISH_STATE_FORCE_RECONNECT,
    PUBLISH_STATE_DISCONNECTED,
    PUBLISH_STATE_RECONNECTING,
    PUBLISH_STATE_CLOSING,
    PUBLISH_STATE_CLOSE,
    PUBLISH_STATE_WARNING,
    PUBLISH_STATE_ERROR,
    PUBLISH_STATE_INVALID
}PublishState;

typedef enum _PublishPointType {
  PUBLISH_POINT_PRIMARY = 0,
  PUBLISH_POINT_BACKUP = 1
}PublishPointType;

typedef enum _PublishSenderType {
    PUBLISH_SENDER_NORMAL = 0,
    PUBLISH_SENDER_THREADED = 1
}PublishSenderType;


#endif  /* _INCLUDE_SERVER_COMMON_DEFINES_H */
