/*******************************************************************************
 * Copyright (c) 2010 eInfochips - All rights reserved.
 *
 * This software is authored by eInfochips and is eInfochips' intellectual
 * property, including the copyrights in all countries in the world.
 * This software is provided under a license to use only with all other rights,
 * including ownership rights, being retained by eInfochips.
 *
 * This file may not be distributed, copied, or reproduced in any manner,
 * electronic or otherwise, without the written consent of eInfochips.
 ******************************************************************************/

/*******************************************************************************
 *
 * File Name    :   ServerFlv.h
 *
 * Description  :   This file contains all defines, enums, structures,
 *                  functions declaration used in FLV module
 *
 * History:
 *  MAY/13/2010, ARJ,   Created the file
 *  MAY/14/2010, PRP,   Started RTMP implementation
 ******************************************************************************/

#ifndef _INCLUDE_SERVER_FLV_H
#define _INCLUDE_SERVER_FLV_H

/*********************
 * Include Files
 *********************/
#include "ServerInclude.h"
#include "ServerBasicTypes.h"
#include "ServerCommon.h"

/*********************
 * Defines and Macros
 *********************/
#define FLV_HDR_SIZE        11                  /* FLV header size            */
#define FLV_H264_HDR_SIZE   5                   /* FLV - H264 tag header size */
#define FLV_AAC_HDR_SIZE    2                   /* FLV - AAC tag header size  */

#define VIDEOCODECID        "videocodecid"      /* videocodecid               */
#define WIDTH               "width"             /* width                      */
#define HEIGHT              "height"            /* height                     */
#define VIDEODATARATE       "videodatarate"     /* videodatarate              */

#define AUDIOCODECID        "audiocodecid"      /* audiocodecid               */
#define AUDIOSAMPLERATE     "audiosamplerate"   /* audiosamplerate            */
#define STEREO              "stereo"            /* stereo                     */

/************************
 * Function declarations
 ************************/

 /* Function to generate FLV meta tag                                         */
INT32 GenerateFlvMetadata (RtmpSession *rtmpSession);

/* Function to generate FLV video tag                                         */
INT32 GenerateFlvH264Hdr (RtmpSession *rtmpSession, UINT8 *dataBuff, INT32 dataSize,
                          UINT64 timeStamp, UINT8 *flvAVTagHdr, UINT8 flag);

/* Function to generate FLV audio tag                                         */
INT32 GenerateFlvAACHdr (RtmpSession *rtmpSession, UINT8 *flvAVTagHdr, UINT8 flag);

/* Function to create AVCC box from video config data                         */
INT32 CreateAVCCBox (RtmpSession *rtmpSession);

#endif  /* _INCLUDE_SERVER_FLV_H */

/*******************************************************************************
 * Copyright (c) 2010 eInfochips - All rights reserved.
 *
 * This software is authored by eInfochips and is eInfochips' intellectual
 * property, including the copyrights in all countries in the world.
 * This software is provided under a license to use only with all other rights,
 * including ownership rights, being retained by eInfochips.
 *
 * This file may not be distributed, copied, or reproduced in any manner,
 * electronic or otherwise, without the written consent of eInfochips.
 ******************************************************************************/
