/*******************************************************************************
 * Copyright (c) 2010 eInfochips - All rights reserved.
 *
 * This software is authored by eInfochips and is eInfochips' intellectual
 * property, including the copyrights in all countries in the world.
 * This software is provided under a license to use only with all other rights,
 * including ownership rights, being retained by eInfochips.
 *
 * This file may not be distributed, copied, or reproduced in any manner,
 * electronic or otherwise, without the written consent of eInfochips.
 ******************************************************************************/

/*******************************************************************************
 *
 * File Name    :   ServerH264.h
 *
 * Description  :   This file contains all defines, enums, structures,
 *                  functions declaration used in H264 module
 *
 * History:
 *  MAY/13/2010, ARJ,   Created the file
 *  MAY/14/2010, PRP,   Started RTMP implementation
 ******************************************************************************/

#ifndef _INCLUDE_SERVER_H264_H
#define _INCLUDE_SERVER_H264_H

/*********************
 * Include Files
 *********************/
#include "ServerInclude.h"
#include "ServerBasicTypes.h"
#include "ServerCommon.h"

/*********************
 * Defines and Macros
 *********************/
#define     VIDEO_KEYFRAME_FREQ         16      /* Key frame frequency        */

/*********************
 *    Structures
 *********************/

/* Bit structure to store the sps buffer                                      */
typedef struct _BitStructure
{
    UINT8       *startPtr;                      /* Pointer to start           */
    UINT8       *bitPtr;                        /* Bit pointer                */
    UINT8       *endPtr;                        /* Pointer to end             */
    INT32       bitsLeft;                       /* No. of bits left           */

} BitStructure;

/************************
 * Function declarations
 ************************/

/* Parse SPS and PPS data                                                     */
VOID ParseSPSHeader (UINT8 *spsBuff, INT32 spsBuffSize,
                     RtmpVdCodecInfo *codecInfo);

/* Function to extract SPS header from video config data buffer               */
UINT8 *ExtractSpsHeader (RtmpSession *rtmpSession, UINT8 *spsPpsBuff,
                         INT32 spsPpsBuffSize, INT32 *spsBuffSize);

/* Function to extract PPS header from video config data buffer               */
UINT8 *ExtractPpsHeader (RtmpSession *rtmpSession,
                         UINT8 *spsPpsBuff, INT32 spsPpsBuffSize,
                         INT32 *ppsBuffSize);

/* Initialize Bit structure                                                   */
VOID BitStructureInit (BitStructure * bitStructPtr, VOID *spsData,
                                       INT32 spsSize);
/* Skip bits in the buffer                                                    */
VOID BitStructureSkip (BitStructure * bitStructPtr, INT32 counter);

/* Read bits from the buffer                                                  */
UINT32 BitStructureRead (BitStructure * bitStructPtr, INT32 counter);

/* Read one bit from the buffer                                               */
UINT32 BitStructureReadOne (BitStructure * bitStructPtr);

/* Read unsigned integer Exp-Golomb-coded syntax                              */
INT32 BitStructureReadUe (BitStructure * bitStructPtr);

/* Read signed integer Exp-Golomb-coded syntax                                */
INT32 BitStructureReadSe (BitStructure * bitStructPtr);

/* Decode Scaling Matrix                                                      */
VOID DecodeScalingMatrices (BitStructure * bitStructPtr,
                            UINT8 (*scaling_matrix4)[16],
                            UINT8 (*scaling_matrix8)[64]);

/* Decode Scaling list                                                        */
VOID DecodeScalingList (BitStructure *s, UINT8 *factors, INT32 size,
                       const UINT8 *jvtList,
                       const UINT8 *fallbackList);


#endif  /* _INCLUDE_SERVER_H264_H */

/*******************************************************************************
 * Copyright (c) 2010 eInfochips - All rights reserved.
 *
 * This software is authored by eInfochips and is eInfochips' intellectual
 * property, including the copyrights in all countries in the world.
 * This software is provided under a license to use only with all other rights,
 * including ownership rights, being retained by eInfochips.
 *
 * This file may not be distributed, copied, or reproduced in any manner,
 * electronic or otherwise, without the written consent of eInfochips.
 ******************************************************************************/
