/*******************************************************************************
 * Copyright (c) 2010 eInfochips - All rights reserved.
 *
 * This software is authored by eInfochips and is eInfochips' intellectual
 * property, including the copyrights in all countries in the world.
 * This software is provided under a license to use only with all other rights,
 * including ownership rights, being retained by eInfochips.
 *
 * This file may not be distributed, copied, or reproduced in any manner,
 * electronic or otherwise, without the written consent of eInfochips.
 ******************************************************************************/

/*******************************************************************************
 *
 * File Name    :   ServerHandShake.h
 *
 * Description  :   This file contains all defines, enums, structures,
 *                  functions declaration used in Handshake module
 *
 * History:
 *  MAY/13/2010, ARJ,   Created the file
 *  MAY/14/2010, PRP,   Started RTMP implementation
 ******************************************************************************/

/*********************
 * Defines and Macros
 *********************/
#define HANDSHAKE_PACKET_SIZE       1536    /* Packet size of handshake       */

/************************
 * Function declarations
 ************************/

/* This functions performs handshaking with the connected clients             */
INT32 PerformHandshake (RtmpClient *clientPtr);

/* This functions performs handshaking with the connected clients             */
INT32 PerformPublishHandshake (RtmpClient *clientPtr);

/* This functions calculates digest of the handshake packet using the key     */
INT32 CalculateDigest (UINT32 digestPosition, UINT8 *handshakeMessage,
                    const UINT8 *fmsKey, INT32 fmsKeyLen, UINT8 *digest);

/* This function calculates the position of the digest in handshake message
   as per the Handshake Message Format 1                                      */
INT32 GetDigestPosition1 (UINT8 *handshakeBuff);

/* This function calculates the position of the digest in handshake message
   as per the Handshake Message Format 2                                      */
INT32 GetDigestPosition2 (UINT8 *handshakeBuff);

/* This function calculates the digest of the message                         */
INT32 HMACsha256 (UINT8 *tempBuffer, INT32 tempBufferSize, const UINT8 *key,
                  INT32 keyLen, UINT8 *digest);

/* This function calculates digest of the handshake message received and compares
   it with the digest present in the message                                  */
BOOL CompareDigest (UINT32 digestPosition, UINT8 *tempBuffer, const UINT8 *key,
                    INT32 keyLen);

/*******************************************************************************
 * Copyright (c) 2010 eInfochips - All rights reserved.
 *
 * This software is authored by eInfochips and is eInfochips' intellectual
 * property, including the copyrights in all countries in the world.
 * This software is provided under a license to use only with all other rights,
 * including ownership rights, being retained by eInfochips.
 *
 * This file may not be distributed, copied, or reproduced in any manner,
 * electronic or otherwise, without the written consent of eInfochips.
 ******************************************************************************/
