/*
 * Copyright, 2012, Teradek
 *
 * All rights reserved.
 * Contact Teradek for licensing.
 */

#ifndef _FACEBOOK_API_H_
#define _FACEBOOK_API_H_

#ifdef __cplusplus
extern "C" {
#endif

#define FACEBOOK_API_ENDPOINT             "https://graph.facebook.com/v2.5/me"
#define FACEBOOK_OAUTH_ENDPOINT           "https://graph.facebook.com/oauth/device"

#define FACEBOOK_PROXY_CONF_FILE          "/etc/proxy.conf"
#define FACEBOOK_API_CONF_FILE            "/etc/facebook/api"
#define FACEBOOK_SETTINGS_FILE            "/etc/oauth/facebook"
#define FACEBOOK_API_AUTH_FILE            "/etc/oauth/facebook"

#define FACEBOOK_AUTHENTICATION_HOOK      "/usr/share/tdapi/facebook_authentication.lua"
#define FACEBOOK_APPLICATION_ID           "994530827288401"

#define FACEBOOK_SSL_VERIFYPEER           0
#define FACEBOOK_SSL_VERIFYHOST           2

#define FACEBOOK_CONF_MAX_LEN             256

typedef struct
{
    CURL *curl;
} FB_API_HANDLE;

typedef struct
{
    char account_id[FACEBOOK_CONF_MAX_LEN];
    char account_name[FACEBOOK_CONF_MAX_LEN];
} FB_API_ACCOUNT_INFO;

typedef struct
{
    char stream_id[FACEBOOK_CONF_MAX_LEN];
    char stream_url[FACEBOOK_CONF_MAX_LEN];
    char stream_secondary_urls[FACEBOOK_CONF_MAX_LEN];
} FB_API_PUB_PT;

enum
{
    FACEBOOK_API_ERR_NONE = 0,
    FACEBOOK_API_ERR_NO_MEM = -1,
    FACEBOOK_API_ERR_INVALID_PARAM = -2,
    FACEBOOK_API_ERR_INVALID_RESPONSE = -3,
    FACEBOOK_API_ERR_API_ERROR = -4,
    FACEBOOK_API_ERR_INVALID_CODE = -5,
    FACEBOOK_API_ERR_PERMISSION = -6,
};

FB_API_HANDLE *facebook_api_create_handle(void);
void facebook_api_destroy_handle(FB_API_HANDLE *handle);

char *facebook_api_conf_get(char *key, char *value, char *conffile);
int facebook_api_conf_set(char *key, char *value, char *conffile);

void facebook_api_set_verbose(int verbose);
void facebook_api_set_timeout(int connect_timeout, int request_timeout);
void facebook_api_read_endpoints(void);
int facebook_api_set_endpoints(char *api_endpt, char *oauth_endpt);

int facebook_api_request_device_code(FB_API_HANDLE *h, char *client_id, char *scope, char **reply);
int facebook_api_save_device_code(char *reply);
int facebook_api_check_authorization(FB_API_HANDLE *h, char *client_id, char *device_code, char **reply);
int facebook_api_save_access_token(char *reply);
int facebook_api_get_account_info(FB_API_HANDLE *h, char *access_token, char **reply);
int facebook_api_get_publish_point(FB_API_HANDLE *h, char *access_token, char **reply);
int facebook_api_save_publish_point(char *reply);
int facebook_api_deauthorize(char **reply);

int facebook_get_error_string(char *reply, char *errorMsg);
int facebook_api_make_request(FB_API_HANDLE *h, char *http_method, char *url, char **reply, char * contentHeader, char *params, long *httpcode);
void facebook_api_request_free(char **reply);

int facebook_api_get_account_info_struct(FB_API_HANDLE *h, char *access_token, FB_API_ACCOUNT_INFO *acntPt, char *errorMsg);
int facebook_api_get_publish_point_struct(FB_API_HANDLE *h, char *access_token, FB_API_PUB_PT *pubPt, char *errorMsg);

#ifdef __cplusplus
}
#endif

#endif /* _FACEBOOK_API_H_ */
