/*
 * Copyright, 2012, Teradek
 *
 * All rights reserved.
 * Contact Teradek for licensing.
 */

#ifndef _LIVESTREAM_API_H_
#define _LIVESTREAM_API_H_

#ifdef __cplusplus
extern "C" {
#endif

#define LIVESTREAM_API_ENDPOINT             "https://api.new.livestream.com"
#define LIVESTREAM_OAUTH_ENDPOINT           "https://oauth.new.livestream.com"
#define LIVESTREAM_SIO_ENDPOINT             "https://sio-1.new.livestream.com"
#define LIVESTREAM_FW_ENDPOINT              "https://cdn.livestream.com/broadcasterpro/firmware/latest.json"
#define LIVESTREAM_TIME_ENDPOINT            "http://api.new.livestream.com/ntp"

#define PROXY_CONF_FILE                     "/etc/proxy.conf"
#define LIVESTREAM_API_CONF_FILE            "/etc/livestream/api"
#define LIVESTREAM_SETTINGS_FILE            "/etc/oauth/livestream"
#define LIVESTREAM_API_AUTH_FILE            "/etc/oauth/livestream"
#define LIVESTREAM_API_SERIAL_FILE          "/etc/livestream/serial_number"

#define LIVESTREAM_AUTHENTICATION_HOOK      "/usr/share/tdapi/livestream_authentication.lua"

#define LIVESTREAM_SSL_VERIFYPEER           0
#define LIVESTREAM_SSL_VERIFYHOST           2

#define LIVESTREAM_CONF_MAX_LEN             256

typedef struct
{
    CURL *curl;
} LS_API_HANDLE;

struct ls_auth_client {
    char *platform;
    char *id;
    char *secret;
};

typedef struct
{
    char id[LIVESTREAM_CONF_MAX_LEN];
    char stream_id[LIVESTREAM_CONF_MAX_LEN];
    char username[LIVESTREAM_CONF_MAX_LEN];
    char password[LIVESTREAM_CONF_MAX_LEN];
    char broadcast_url[2][LIVESTREAM_CONF_MAX_LEN];
} LS_API_PUB_PT;

enum
{
    LIVESTREAM_API_ERR_NONE = 0,
    LIVESTREAM_API_ERR_NOMEM = -1,
    LIVESTREAM_API_ERR_UNAUTHORIZED = -2,
    LIVESTREAM_API_ERR_REAUTHORIZED = -3,
    LIVESTREAM_API_ERR_INVALID_PARAM = -4,
    LIVESTREAM_API_ERR_API_ERROR = -5,
    LIVESTREAM_API_ERR_INVALID_RESPONSE = -6
};

LS_API_HANDLE *livestream_api_create_handle(void);
void livestream_api_destroy_handle(LS_API_HANDLE *handle);

char *livestream_conf_get(char *key, char *value, char *conffile);
int livestream_conf_set(char *key, char *value, char *conffile);

void livestream_api_set_verbose(int verbose);
void livestream_api_set_timeout(int connect_timeout, int request_timeout);
void livestream_api_read_endpoints(void);
int livestream_api_set_endpoints(char *api_endpt, char *oauth_endpt, char *sio_endpt, char *fw_endpt, char *time_endpt);

int livestream_api_get_fw_info(LS_API_HANDLE *h, char **reply);
int livestream_api_get_time(LS_API_HANDLE *h, char **reply);
int livestream_api_get_sio_endpoints(LS_API_HANDLE *h, char **reply);
int livestream_api_ping(void);

int livestream_api_get_account(LS_API_HANDLE *h, char *access_token, char *account_id, char **reply);
int livestream_api_unpair_account(LS_API_HANDLE *h, char *access_token,  char **reply);

int livestream_api_set_client_plat(char *client_plat);
struct ls_auth_client *livestream_api_get_auth_client(char *plat);
int livestream_api_request_shortcode(LS_API_HANDLE *h, char *client_id, char *client_secret, char **reply);
int livestream_api_poll_pair_request(LS_API_HANDLE *h, char *device_code, char **reply);
int livestream_api_grant_device_code(LS_API_HANDLE *h, char *client_id, char *client_secret, char *device_code, char **reply);
int livestream_api_get_access_token(LS_API_HANDLE *h, char *client_id, char *client_secret, char *username, char *password, char **reply);
int livestream_api_save_access_token(char *reply);
int livestream_api_refresh_access_token(LS_API_HANDLE *h, char *refresh_token, char **reply);
int livestream_api_refresh_access_token_buf(LS_API_HANDLE *h, char *refresh_token, char *access_token_buf, char *errorStr);
int livestream_api_update_access_token(char *reply);
int livestream_api_get_token_related_info(LS_API_HANDLE *h, char *access_token, char **reply);
int livestream_api_deauthorize(char **reply);

int livestream_api_create_device(LS_API_HANDLE *h, char *access_token, char *name, char **reply);
int livestream_api_get_devices(LS_API_HANDLE *h, char *access_token, char **reply);
int livestream_api_update_device(LS_API_HANDLE *h, char *access_token, char *device_id, char *config, char **reply);
int livestream_api_get_device(LS_API_HANDLE *h, char *access_token, char *device_id, char **reply);
int livestream_api_send_device_serial(LS_API_HANDLE *h, char *access_token, char *device_id, char **reply);

int livestream_api_get_publishing_points(LS_API_HANDLE *h, char *access_token, char *account_id, char **reply);
int livestream_api_get_publishing_point(LS_API_HANDLE *h, char *access_token, char *account_id, LS_API_PUB_PT *pubPt, char *errorMsg);
int livestream_api_create_broadcast(LS_API_HANDLE *h, char *access_token, char *account_id,
                char *title, int notify_followers, char *event_id, char **reply);
int livestream_api_create_broadcast_id(LS_API_HANDLE *h, char *access_token, char *account_id,
                char *title, int notify_followers, char *event_id, char *broadcast_id, char *video_id, char *errorMsg);
int livestream_api_stop_broadcast(LS_API_HANDLE *h, char *access_token, char *broadcast_id, char **reply);
int livestream_api_stop_broadcast_error(LS_API_HANDLE *h, char *access_token, char *broadcast_id, char *errorMsg);
int livestream_api_cleanup_broadcast(LS_API_HANDLE *h, char *access_token, char *account_id, char *broadcast_id, char **reply);

int livestream_api_get_events(LS_API_HANDLE *h, char *access_token, char *account_id, int page, int older, int newer, char **reply);
int livestream_api_get_draft_events(LS_API_HANDLE *h, char *access_token, char *account_id, int page, char **reply);
int livestream_api_create_stream(LS_API_HANDLE *h, char *access_token, char *account_id,
                char *broadcast_id, char *event_id,
                char *publishing_point_id,char *stream_name, char* quality, char **reply);
int livestream_api_create_stream_error(LS_API_HANDLE *h, char *access_token, char *account_id,
                char *broadcast_id, char *event_id,
                char *publishing_point_id,char *stream_name, char* quality, char *errorMsg);

int livestream_api_delete_video(LS_API_HANDLE *h, char *access_token, char *account_id, char *event_id, char *video_id, char **reply);
int livestream_api_delete_video_error(LS_API_HANDLE *h, char *access_token, char *account_id, char *event_id, char *video_id, char *errorMsg);
int livestream_api_save_video(LS_API_HANDLE *h, char *access_token, char *account_id, char *event_id, char *video_id, int draft, int notify_followers, char **reply);
int livestream_api_save_video_error(LS_API_HANDLE *h, char *access_token, char *account_id, char *event_id, char *video_id, int draft, int notify_followers, char *errorMsg);

int livestream_api_post_thumbnail(LS_API_HANDLE *h, char *access_token, char *device_id, char *thumbnail_path, char **reply);
int livestream_api_post_thumbnail_error(LS_API_HANDLE *h, char *access_token, char *device_id, char *thumbnail_path, char *errorMsg);
int livestream_api_post_live_thumbnail(LS_API_HANDLE *h, char *access_token, char *broadcast_id, int broadcast_time, char *thumbnail_path, char **reply);
int livestream_api_post_live_thumbnail_error(LS_API_HANDLE *h, char *access_token, char *broadcast_id, int broadcast_time, char *thumbnail_path, char *errorMsg);

int livestream_api_create_event(LS_API_HANDLE *h, char * access_token, char *account_id, char *full_name,
        char* short_name, char* description, char** reply);

void livestream_api_request_free(char **reply);

int livestream_get_error_string(char *reply, char *errorStr);
int livestream_make_request(LS_API_HANDLE *h, char *http_method, char *access_token, char *url, char **reply, char * contentHeader, char *params, long *httpcode);

#ifdef __cplusplus
}
#endif

#endif /* _LIVESTREAM_API_H_ */
