/*******************************************************************************
 * Copyright (c) 2009 eInfochips - All rights reserved.
 *
 * This software is authored by eInfochips and is eInfochips' intellectual
 * property, including the copyrights in all countries in the world.
 * This software is provided under a license to use only with all other rights,
 * including ownership rights, being retained by eInfochips.
 *
 * This file may not be distributed, copied, or reproduced in any manner,
 * electronic or otherwise, without the written consent of eInfochips.
 ******************************************************************************/

/*******************************************************************************
 *
 * File Name    :   MessageDigest.h
 *
 * Description  :   This file defines Macros, structures and function Definitions
                    required by MessageDigest.c
 *
 * History:
 *  MAY/07/2009, ARJ,   RTSP over HTTP feature added
 *
 ******************************************************************************/
/*********************
 * Include Files
 *********************/
#include "BasicTypes.h"

/*********************
 * Defines and Macros
 *********************/

/* PROTOTYPES should be set to one if and only if the compiler supports
   function argument prototyping.
   The following makes PROTOTYPES default to 0 if it has not already
   been defined with C compiler flags.                                         */

#ifndef PROTOTYPES
#define PROTOTYPES 0
#endif

/* PROTO_LIST is defined depending on how PROTOTYPES is defined above. If using 
   PROTOTYPES, then PROTO_LIST returns the list, otherwise it returns
    an empty list.                                                            */

#if PROTOTYPES
#define PROTO_LIST(list) list
#else
#define PROTO_LIST(list) ()
#endif

#define HASHLEN 16
typedef char HASH[HASHLEN];
#define HASHHEXLEN 32
typedef char HASHHEX[HASHHEXLEN+1];
#define IN
#define OUT

#define S11 7
#define S12 12
#define S13 17
#define S14 22
#define S21 5
#define S22 9
#define S23 14
#define S24 20
#define S31 4
#define S32 11
#define S33 16
#define S34 23
#define S41 6
#define S42 10
#define S43 15
#define S44 21

/* F, G, H and I are basic MD5 functions.                                     */
#define F(x, y, z) (((x) & (y)) | ((~x) & (z)))
#define G(x, y, z) (((x) & (z)) | ((y) & (~z)))
#define H(x, y, z) ((x) ^ (y) ^ (z))
#define I(x, y, z) ((y) ^ ((x) | (~z)))

/* ROTATE_LEFT rotates x left n bits.                                         */
#define ROTATE_LEFT(x, n) (((x) << (n)) | ((x) >> (32-(n))))

/* FF, GG, HH, and II transformations for rounds 1, 2, 3, and 4.
   Rotation is separate from addition to prevent recomputation.               */
#define FF(a, b, c, d, x, s, ac) { \
 (a) += F ((b), (c), (d)) + (x) + (UINT32)(ac); \
 (a) = ROTATE_LEFT ((a), (s)); \
 (a) += (b); \
  }
#define GG(a, b, c, d, x, s, ac) { \
 (a) += G ((b), (c), (d)) + (x) + (UINT32)(ac); \
 (a) = ROTATE_LEFT ((a), (s)); \
 (a) += (b); \
  }
#define HH(a, b, c, d, x, s, ac) { \
 (a) += H ((b), (c), (d)) + (x) + (UINT32)(ac); \
 (a) = ROTATE_LEFT ((a), (s)); \
 (a) += (b); \
  }
#define II(a, b, c, d, x, s, ac) { \
 (a) += I ((b), (c), (d)) + (x) + (UINT32)(ac); \
 (a) = ROTATE_LEFT ((a), (s)); \
 (a) += (b); \
  }

/**********************
 * Globals & Structures
 **********************/
/* MD5 context.                                                               */
typedef struct
{
  UINT32 state[4];           /* state (ABCD)                                  */
  UINT32 count[2];           /* number of bits, modulo 2^64 (lsb first)       */
  UINT8 buffer[64];          /* input buffer                                  */
  
} MD5_CTX;

/************************
 * Function declarations
 ************************/
void MD5Init PROTO_LIST ((MD5_CTX *));
void MD5Update PROTO_LIST((MD5_CTX *, UINT8 *, UINT32));
void MD5Final PROTO_LIST ((UINT8 [16], MD5_CTX *));
void hmac_md5 PROTO_LIST ((UINT8 *, INT32, UINT8 *, INT32, caddr_t));

/* calculate H(A1) as per HTTP Digest spec                                    */
void DigestCalcHA1(IN INT8 *pszAlg, INT8 *pszUserName, IN INT8 *pszRealm,
                   IN INT8 *pszPassword, IN INT8 *pszNonce, IN INT8 *pszCNonce,
                   OUT HASHHEX SessionKey);

/* calculate request-digest/response-digest as per HTTP Digest spec           */
void DigestCalcResponse(IN HASHHEX HA1, IN INT8 *pszNonce, IN INT8 *pszNonceCount,
                        IN INT8 *pszCNonce, IN INT8 *pszQop, IN INT8 *pszMethod,
                        IN INT8 *pszDigestUri, IN HASHHEX HEntity,
                        OUT HASHHEX Response);
                        
/*  Wrapper containing DigestCalcHA1 and DigestCalcResponse                   */
INT32 GenerateDigestResponse(INT8 *pszNonce, INT8 *pszCNonce, INT8 *pszUser,
                             INT8 *pszRealm,INT8 *pszPass, INT8 *pszAlg,
                             INT8 *szNonceCount, INT8 *pszMethod, INT8 *pszQop,
                             INT8 *pszURI,INT8 *Response);

/*******************************************************************************
 * Copyright (c) 2007 eInfochips - All rights reserved.
 *
 * This software is authored by eInfochips and is eInfochips' intellectual
 * property, including the copyrights in all countries in the world.
 * This software is provided under a license to use only with all other rights,
 * including ownership rights, being retained by eInfochips.
 *
 * This file may not be distributed, copied, or reproduced in any manner,
 * electronic or otherwise, without the written consent of eInfochips.
 ******************************************************************************/

