/*******************************************************************************
 * Copyright (c) 2008 eInfochips - All rights reserved.
 *
 * This software is authored by eInfochips and is eInfochips' intellectual
 * property, including the copyrights in all countries in the world.
 * This software is provided under a license to use only with all other rights,
 * including ownership rights, being retained by eInfochips.
 *
 * This file may not be distributed, copied, or reproduced in any manner,
 * electronic or otherwise, without the written consent of eInfochips.
 ******************************************************************************/

/*******************************************************************************
 *
 * File Name    :   RtpRtspClient.h
 *
 * Description  :   This file contains API list which is exported to application
 *
 * History:
 *  AUG/22/2008, ARJ,   Created the file
 ******************************************************************************/

#ifndef _INCLUDE_RTPRTSPCLIENT__H
#define _INCLUDE_RTPRTSPCLIENT__H

/*********************
 * Include Files
 *********************/
#include "RtpRtspInclude.h"
#include "RtpRtspBasicTypes.h"

/*********************
 * Defines and Macros
 *********************/

// Start with Blocking code
#define  STACK_BLOCKINGCODE

/* Error codes return by RTP/RTSP APIs                                        */
#define RTP_RTSP_OK                 (0)     /* Status ok                      */
#define RTP_RTSP_ERROR              (-1)    /* Status error                   */
#define MAX_IP_LENGTH               32      /* Maximum IP Length              */
#define MAX_DECODE_SIZE             512     /* Maximum Size of decoded SPS & PPS */


/*********************
 * Typedefs and Enums
 *********************/
typedef enum _ErrorCodes
{
    RTP_RTSP_INVALID_INPUT_VALUE = -99,     /* Invalid input argument         */
    RTP_RTSP_MEMORY_ALLOCATION_FAIL,        /* Memory allocate failed         */
    RTP_RTSP_INVALID_HEADER,                /* Header is Invalid              */
    RTP_RTSP_HEADERLIST_NOT_VALID,          /* Header list is Invalid         */
    RTP_RTSP_CLIENT_NOT_CONNECTED,          /* Client is not connected to server*/
    RTP_RTSP_REQUEST_MESSAGE_NOT_GENERATED, /* Unable to generate request message */
    RTP_RTSP_SEND_REQUEST_FAILED,           /* Failed in sending request message*/
    RTP_RTSP_RESPONSE_NOT_RECEIVED_IN_TIME, /* Response not received in time  */
    RTP_RTSP_NULL_POINTER_ASSIGNMENT,       /* Null pointer assignment        */
    RTP_RTSP_PARSING_RESPONSE_FAIL,         /* Problem in parsing response message */
    RTP_RTSP_SERVER_ERROR_FROM_SERVER,      /* Server Error on server side    */
    RTP_RTSP_SERVER_AUTHENTICATION_REQUESTED,/* Server requested authentication */
    RTP_RTSP_CLIENT_ERROR_FROM_SERVER,      /* Client Error on server side    */
    RTP_RTSP_REDIRECTIONAL_SERVER_ERROR,    /* Redirectional error on server side */
    RTP_RTSP_SMALL_BUFF_FROM_APPLICATION,   /* Small Buffer passed from application */
    RTP_RTSP_METHOD_INVALID_IN_THIS_STATE,  /* Method is invalid in this state*/
    RTP_RTSP_INVALID_SESSION,               /* Invalid Session found          */
    RTP_RTSP_SOCKET_CREATION_FAIL,          /* Socket creation failed         */
    RTP_RTSP_SOCKET_BIND_FAIL,              /* Socket bind failed             */
    RTP_RTSP_SOCKET_CONNECT_FAIL,           /* Socket connection failed       */
    RTP_RTSP_LOCK_INITIALIZATION_FAIL,      /* Lock Initialization failed     */
    RTP_RTSP_LOCK_CREATION_FAILED,          /* Lock Creating Failed           */
    RTP_RTSP_LOCK_FAIL,                     /* Locking Failed                 */
    RTP_RTSP_UNLOCK_FAIL,                   /* Unlocking Failed               */
    RTP_RTSP_LOCK_DESTROY_FAIL,             /* Destroying Lock Failed         */
    RTP_RTSP_INVALID_IP_VERSION,            /* Invalid IP Version found       */
    RTP_RTSP_ATOI_FAIL,                     /* atoi() conversion failed       */
    RTP_RTSP_CLIENT_NOT_FOUND,              /* Client Not Found               */
    RTP_RTSP_MULTICAST_ADD_FAIL             /* Multicast Add Failed           */

} ErrorCodes;

typedef enum _RTSPTransportProto
{
    UDP_CONNECTION,                         /* UDP Transport Protocol         */
    TCP_CONNECTION                          /* TCP Transport Protocol         */
}RTSPTransportProto;

/* Video Codec enums                                                          */
typedef enum _SupportedCodecs
{
    CODEC_H264 = 100,                            /* Video Codec H264          */
    CODEC_MPEG4V,                                /* Video Codec Mpeg4 Video   */
    CODEC_MJPEG,                                 /* Video Codec MJPEG         */
    CODEC_MP2T,                                  /* MPEG2 Transport stream    */
    CODEC_G711A = 200,                           /* Audio Codec G711 ALAW     */
    CODEC_G711U,                                 /* Audio Codec G711 ULAW     */
    CODEC_L16,                                   /* Audio Codec PCM  	      */
    CODEC_AAC,                                   /* Audio Codec AAC           */
    CODEC_G726,                                  /* Audio Codec g726          */
    CODEC_MPA,                                   /* Audio Codec MPA           */
    INVALID_CODEC                                /* Invalid Audio Codec       */

} SupportedCodecs;


/***************
 * Structures
 ***************/

typedef struct _StreamInputParams
{
    BYTE                codecType;          /*  0 = Audio, 1 = Video Codec    */
    BYTE                encodingName[MAX_IP_LENGTH]; /* Codec Encoding Name   */
    SupportedCodecs     codecName;          /*  Codec Type eNum               */
    INT32               payloadNumber;      /*  Codec Payload Number          */
    INT32               clockRate;          /*  Codec Clock Rate              */
    FLOAT               frameRate;          /*  Codec FrameRate(#.# format)   */
    INT32               noOfChannels;       /*  Number of Audio Channels      */
    INT32               bandWidth;          /*  Codec Bandwidth               */
    INT32               formatDataSize;     /*  Format Params size            */
    BYTE                *formatDataPtr;     /*  Format Params from/to SDP     */
    INT32               spsDataSize;        /*  Decoded SPS Data Size         */
    BYTE                spsData[MAX_DECODE_SIZE];/*Decoded SPS Data Buffer    */
    INT32               ppsDataSize;        /*  Decoded PPS Data Size         */
    BYTE                ppsData[MAX_DECODE_SIZE];/*Decoded PPS Data Buffer    */
    INT32               aacConfigDataSize;  /* AAC Config Data Size           */
    BYTE                aacConfigData[MAX_DECODE_SIZE]; /* AAC Config Data    */

}StreamInputParams;

typedef struct _RTPHeader
{
    UINT16              rtpVersion      : 2;    /* Version Of RTP             */
    UINT16              paddingBit      : 1;    /* Padding Bit of Packet      */
    UINT16              extensionBit    : 1;    /* Extension Bit of Packet    */
    UINT16              csrcCount       : 4;    /* CSRC Count of Packet       */
    UINT16              markerBit       : 1;    /* Marker Bit of Packet       */
    UINT16              payloadType     : 7;    /* Payload Type of Packet     */
    UINT16              sequenceNumber;         /* Sequence number of packet  */
    UINT32              timestamp;              /* Time Stamp of packet       */
    UINT32              ssrc;                   /* SSRC of packet             */
    UINT32              csrc[16];               /* CSRC List                  */

} RTPHeader;

/* This structure is used for storing RTP data sent or received.              */
struct _RTPStreamData
{
    SupportedCodecs         dataCodecType;      /* RTP Data codec type        */
    RTPHeader               *rtpHeader;         /* RTP Header of this packet  */
    INT32                   dataSize;           /* RTP Data size              */
    BYTE                    *dataBuffer;        /* RTP Data buffer pointer    */
    UINT16                  headerExtensionSize;/* Size of Header Extension   */
    UINT16                  headerExtensionType;/* Type of Header Extension   */
    BYTE                    *headerExtension;   /* Pointer to Header Extension*/
    INT32                   packetSize;         /* Size of packet             */
    BYTE                    *packetBuffer;      /* Pointer to entire RTP packet*/
    INT32                   packetLostFlag;     /* Packet is lost before this packet*/
    UINT64                  pktCounter;         /* Packet counter             */                            
    struct _RTPStreamData   *next;              /* Pointer to next Stream data*/
    struct _RTPStreamData   *prev;              /* Pointer to previous Stream data*/
};

/* TypeDef for RTPStreamData                                                  */
typedef struct _RTPStreamData RTPStreamData;

/***************
 * Callbacks
 ***************/
typedef INT32 (*CLIENT_MTU_CALLBACK) (HANDLE, INT32);

/* Thread Function Pointer                                                    */
#ifdef WIN32
typedef INT32  (WINAPI *THREAD_FUNC) (void *);
#endif


/************************
 * Function declarations
 ************************/

/* API to Create the RTSP Client Instance                                     */
HANDLE RTSPClientCreate ();

/* API to Create New Header List                                              */
HANDLE RTSPCreateHeaderList();

/* API to Add Header in created header list                                   */
INT32 RTSPAddHeaderToList ( HANDLE headerHandle, BYTE  *headerName,
                            BYTE   *headerVal,   BYTE  customHeaderFlag );

/* API to Remove Header from header list                                      */
INT32 RTSPRemoveHeaderFromList ( HANDLE headerHandle, BYTE *headerName );

/* API to Modify Header in the header list                                    */
INT32 RTSPModifyHeaderInList ( HANDLE headerHandle, BYTE *headerName,
                               BYTE   *headerValue );

/* API to Destroy Header in the header list                                   */
INT32 RTSPDestroyHeaderList(HANDLE headerHandle);

/* API to connect the client to the specified server                          */
INT32 RTSPClientConnect (HANDLE clientHandle,   BYTE *serverIp,
                         UINT16 serverPort,     BYTE *sessionName,
                         INT32 dataProto,       CLIENT_MTU_CALLBACK mtuCallBack);

/* API to generate OPTIONS request and send it to server                      */
INT32 RTSPClientSendOptionsRequest (HANDLE  clientHandle,   HANDLE  optionsHeaderList,
                                    BYTE    *responseBuff,  INT32   *responseSize);

/* API to generate DESCRIBE request and send it to server                     */
INT32 RTSPClientSendDescribeRequest (HANDLE  clientHandle, HANDLE describeHeaderList,
                                     BYTE   *responseBuff, INT32  *responseSize);

/* API to get the total number os streams present in SDP                      */
INT32 RTSPClientGetNbStreams(HANDLE clientHandle);

/* API to get the SDP Information as received from the server                 */
StreamInputParams  *RTSPClientGetSDPInfo(HANDLE clientHandle, INT32 streamCount);

/* API to get the SDP field information as received from the server           */
INT32 RTSPClientGetSDPFieldInfo (HANDLE clientHandle, INT32 streamCount,
                                 BYTE *sdpFieldName,  BYTE  *sdpFieldValue,
                                 INT32 *sdpFieldSize);

/* API to generate SETUP request and send it to server                        */
INT32 RTSPClientSendSetupRequest(HANDLE clientHandle,   INT32 streamCount,
                                 HANDLE setupHeaderList,
                                 BYTE   *responseBuff,  INT32 *responseSize);

/* API to generate PLAY request and send it to server                         */
INT32 RTSPClientSendPlayRequest(HANDLE clientHandle, INT32  streamCount,
                                HANDLE playHeaderList,
                                BYTE  *responseBuff, INT32  *responseSize);

/* API to generate TEARDOWN request and send it to server                     */
INT32 RTSPClientSendTeardownRequest (HANDLE clientHandle,   HANDLE teardownHeaderList,
                                     BYTE   *responseBuff,  INT32  *responseSize);

/* API to generate PAUSE request and send it to server                        */
INT32 RTSPClientSendPauseRequest(HANDLE clientHandle,   INT32   streamCount,
                                 HANDLE pauseHeaderList,
                                 BYTE   *responseBuff,  INT32   *responseSize);

/* API to automatically connect to server, generate and send all message to server */
INT32 RTSPClientAutoRTSPConnect(HANDLE clientHandle,    BYTE    *serverIp,
                                UINT16  serverPort,     BYTE    *sessionName,
                                BYTE  *username,     	BYTE    *password,
                                INT32   dataProto,      HANDLE  optionsHeaderList,
                                HANDLE  describeHeaderList,
                                HANDLE  setupHeaderList,
                                HANDLE  playHeaderList,
                                CLIENT_MTU_CALLBACK     mtuCallBack,
                                BYTE    *responseBuff,  INT32   *responseSize);

/* API to Get the Source Synchronization Identifier of particular stream      */
INT32 RTSPClientGetSSRC (HANDLE clientHandle, INT32 streamCount,
                         UINT32 *SSRC);

/* API to destroy the Client                                                  */
INT32 RTSPClientDestroy(HANDLE clientHandle);

/* API to set the MTU Size of the Stack                                       */
INT32 RTSPClientSetMTUSize( HANDLE clientHandle, UINT32 MTUSize );


INT32 RTPClientGetTSOffset(HANDLE clientHandle, SupportedCodecs codecName, INT32 *offset, INT32 audioClockRate);

INT32 RTPClientPeekRTPStreamData (HANDLE clientHandle,RTPStreamData **streamData,
                                 SupportedCodecs codecName);
/* API to get the stream data from the Stack buffer                           */
INT32 RTPClientGetRTPStreamData (HANDLE clientHandle, RTPStreamData **streamData,
                                 SupportedCodecs codecName);

/* API to free the stream data buffer present in stack                        */
INT32 RTPClientFreeRTPStreamData (HANDLE clientHandle,RTPStreamData *streamData,
                                 SupportedCodecs codecName);

/* API to set the value of latency in milliseconds                            */
INT32 RTPClientConfigLatency (HANDLE clientHandle, INT32 jbTol,
                              INT32 jbUnd, INT32 jbOvr);

/* API to set Audio and Video Ring Buffer Sizes 							  */
INT32 RTPClientSetAudioVideoBufferSize (HANDLE clientHandle ,
										INT32 videoBufferSize, INT32 audioBufferSize);

/* API to start third party client for receiving data directly on RTP 		  */
INT32 StartThirdPartyClient (HANDLE clientHandle, INT32 adVdFlag,
							 SupportedCodecs videoCodecName, UINT16 videoPort,
							 INT32 videoRBsize, SupportedCodecs audioCodecName,
							 UINT16 audioPort, INT32 audioRBsize,
							 INT32 tgtLat, INT32 udfLat, INT32 ovfLat);

/* This API gives the number of packets present in Ring buffer of particular 
  stream.							  										  */
INT32 RTPClientGetNbOfPktInBuffer(HANDLE clientHandle, SupportedCodecs codecName);

/* API to return global packet count 						  				  */
UINT64 RTPClientGetStreamPktCount (HANDLE clientHandle, SupportedCodecs codecName);

#endif  /* _INCLUDE_RTPRTSPCLIENT__H */

/*******************************************************************************
 * Copyright (c) 2008 eInfochips - All rights reserved.
 *
 * This software is authored by eInfochips and is eInfochips' intellectual
 * property, including the copyrights in all countries in the world.
 * This software is provided under a license to use only with all other rights,
 * including ownership rights, being retained by eInfochips.
 *
 * This file may not be distributed, copied, or reproduced in any manner,
 * electronic or otherwise, without the written consent of eInfochips.
 ******************************************************************************/
