#ifndef _INCLUDE_RTSPANNOUNCE_H
#define _INCLUDE_RTSPANNOUNCE_H

/*********************
 * Include Files
 *********************/
#include <stdio.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <sys/un.h>

#include <netinet/in.h>
#include <netinet/tcp.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <unistd.h>
#include <fcntl.h>
#include <stdlib.h>
#include <errno.h>
#include <ctype.h>
#include <time.h>
#include <sys/timeb.h>
#include <sys/time.h>

#include <linux/if.h>
#include <sys/wait.h>
#include <sys/ioctl.h>

#include "RtpRtsp.h"
#include "RtspServer.h"
#include "MessageDigest.h"

#include "DbgPrint.h"

/*********************
 * Defines and Macros
 *********************/

#define RTSP_TERADEK_USER_AGENT "Teradek v1"

#define RTP_RTSP_OK                 (0)     /* Status ok */
#define RTP_RTSP_ERROR              (-1)    /* Status error */
#define RTP_RTSP_AUTH_ERROR         (-2)    /* Authentication error */
#define RTP_RTSP_FORBIDDEN_ERROR    (-3)    /* Forbidden error */

#define LED_SEQ_PRIORITY 2
#define LED_RECORDING    "recording"
#define LED_RECONNECTING "reconnecting"
#define LED_STREAMING    "streaming"

#define TDWD_DAEMON_ID "tdwd"
#define TDCD_DAEMON_ID "tdcd"
#define TDBOARD_DAEMON_ID "tdbd"

#define RUNNING_FLAG_FILE_NAME "/tmp/rtsp_announce_streaming"

/*********************
 * Typedefs and Enums
 *********************/

typedef enum _RtspAnnounceError {
	NO_ERROR = 0,
	CONNECTION_ERROR,
	ANNOUNCE_ERROR
}RtspAnnounceError;

/************************
 * Function declarations
 ************************/

/* Teradek daemons communication */
extern int daemon_buf_send_command(char *daemon, char *cmd_string);
extern int rtsp_report_error(RtspAnnounceError type, char *error_str, ...);
extern int rtsp_report_status(AnnounceState state, int recordFlagIsSet);


/* Misc */
int announce_lookup_host(const char *host, char *ip);
INT32 AnnounceLog(RTSPClient *rtspClient, INT8 *str, ... );
RTSPClient* FindRTSPClient(RTSPSession *RtspSession, AnnounceClient *announce);
INT32 announceChangeState(RTSPClient *rtspClient, AnnounceState state);

/* Functions to interact with announce clients */
INT32 announceConnectClient(AnnounceClient *announce, BYTE *session, INT32 exp_stream_cnt, BYTE *ret);
INT32 announceDisconnectClient(AnnounceClient *announce, BYTE *session, BYTE *ret);
INT32 announceCloseClient(AnnounceClient *announce, BYTE *session, BYTE *ret);
INT32 announceReconnectClient(AnnounceClient *announce, BYTE *session, BYTE *ret);

/* Local handles for RTSP States */
INT32 RTSPAnnounceDescribeHandle( RTSPClient * ClientConn );
INT32 RTSPAnnounceSetupHandle( RTSPClient * ClientConn,  INT8 trackID);
INT32 RTSPAnnouncePlayHandle( RTSPClient * ClientConn);
INT32 RTSPAnnouncePauseHandle( RTSPClient * ClientConn);
INT32 RTSPAnnounceTeardownHandle( RTSPClient * ClientConn);

/* Send RTSP Requests to the client */
INT32 RTSPAnnounceSendGetParameterRequest(HANDLE clientHandle, HANDLE headerList, BYTE *responseBuff,  INT32  *responseSize);
INT32 RTSPAnnounceSendOptionsRequest(HANDLE clientHandle, HANDLE headerList, BYTE *responseBuff,  INT32  *responseSize);
INT32 RTSPAnnounceSendAnnounceRequest(HANDLE clientHandle, HANDLE headerList, BYTE *responseBuff,  INT32  *responseSize);
INT32 RTSPAnnounceSendSetupRequest(HANDLE clientHandle, HANDLE headerList, BYTE *responseBuff,  INT32  *responseSize);
INT32 RTSPAnnounceSendRecordRequest(HANDLE clientHandle, HANDLE headerList, BYTE *responseBuff,  INT32  *responseSize);
INT32 RTSPAnnounceSendTeardownRequest(HANDLE clientHandle, HANDLE headerList, BYTE *responseBuff,  INT32  *responseSize);


#endif  /* _INCLUDE_RTSPANNOUNCE_H */
