/*
 * Copyright, 2009, Teradek
 * 
 * All rights reserved.
 * Contact Teradek for licensing.
 */

#ifndef _DAEMON_H_
#define _DAEMON_H_

/** <h3>Teradek Libraries</h3> 
 * \addtogroup libtd
 * \{
 */

/** Daemonize a process
 * 
 * This command will allow the process to run "headless".  It will not have
 * any input or output.  It will not hang the terminal it is run in.  It will
 * not lock the directory it is run in.  It will not output anything to the shell.
 */
extern void td_daemonize(void);

/** Create a PID file
 * 
 * This function will create a file that contains the PID of the process running.
 * This can be used to keep track of running process and allow you to kill any
 * running process.
 * 
 * \param pidFile Path to the file to create
 */
extern void td_create_pid_file(const char *pidFile);

/** Remove the PID file
 * 
 * This is the complement of the function above.  When the process is exiting, it
 * should clean up after itself.
 * 
 * \param pidFile Path to the file to clean up
 */
extern void td_remove_pid_file(char *pidFile);

/** \} */

#endif /* _DAEMON_H_ */
