/*
 * Copyright, 2008, Teradek
 * 
 * All rights reserved.
 * Contact Teradek for licensing.
 * 
 */

#ifndef _LOG_H_
#define _LOG_H_

#include <syslog.h>
#include <pthread.h>

/** <h3>Teradek Libraries</h3> 
 * \addtogroup libtd
 * \{
 */

/** Log levels
 * Different logging level
 */
#define TD_LOG_ERR          LOG_ERR
#define TD_LOG_WARN         LOG_WARNING
#define TD_LOG_INFO         LOG_NOTICE
#define TD_LOG_CHITCHAT     LOG_INFO
#define TD_LOG_DEBUG        LOG_DEBUG
#define TD_LOG_VERBOSE      (LOG_DEBUG+1)
#define TD_LOG_NUM_LEVELS   6

/** Open the logging system
 * \param level The log level to display. (0: Err ~ 4: Debug)
 * \param timestamp Enable timestamping on the messages.
 * \param use_console Enable message to console
 * \param file File to write the log messages.
 * \param syslog Send message to syslog
 * \param ident String prepend to syslog messages.
 */
void td_log_start(int level, int timestamp, int use_console, 
                        char *file, int syslog, char *ident);

/** Stop the logging system
 */
void td_log_stop(void);

/** Write a message to the logging system
 * \param level The log level
 * \param fmt Printf-like formatting of message.
 * \param ... Parameters for the formatted message.
 */
void td_log_print(int level, char *fmt, ...);

/** Change the log level that is displayed.
 * \param level The log level to display. (0: Err ~ 4: Debug)
 */
void td_log_change_level(int level);

/** \} */

#endif // _LOG_H_

