/*******************************************************************************
*
* Teradek(C) 2014
*
*
*******************************************************************************/
#pragma once

#include "writestreamer.h"
#include "ioctrl.h"
#include "tdprofiler.h"

#define TDS_SHORT_STRING_LENGTH 128
#define TDS_MEDIUM_STRING_LENGTH 1024
#define TDS_LONG_STRING_LENGTH 4096

#define TDS_SERVER_NUMBACKLOG 10

#define TDS_PING_TIMEOUT_MS 5000

#define TDS_HEADER_SIZE 8
#define TDS_DEFAULT_PACKET_SIZE 1300
#define TDS_MAX_PACKET_SIZE (10*1024)

typedef enum
{
    TDS_PROTO_UDP,
    TDS_PROTO_TCP,
    TDS_PROTO_MULTICAST,
} TDS_PROTO;

typedef enum
{
    TDS_PACKET_WRITER_PROTO_ID = 0x49,
    TDS_PACKET_WRITER_MESSAGE_PROTO_ID= 0x4a,
} TDS_PACKET_WRITER;
typedef enum
{
    TDS_PACKET_READER_PROTO_ID = 0x50,
    TDS_PACKET_READER_PING = 0x24,
    TDS_PACKET_READER_PING_W_DELAY = 0x25,
    TDS_PACKET_READER_CONNECT = 0x26,
    TDS_PACKET_READER_DISCONNECT = 0x27,
    TDS_PACKET_READER_SETTINGS = 0x28,
    TDS_PACKET_READER_FORCE_CONNECT = 0x29,
} TDS_PACKET_READER;

typedef enum
{
    TDS_WRITER_IOCTL_ENABLE_STREAMING = 0x01,
    TDS_WRITER_IOCTL_GET_DELAY = 0x02,
    TDS_WRITER_IOCTL_SEND_MESSAGE = 0x03,
    TDS_WRITER_IOCTL_GET_CLIENT_STATE = 0x04,
    TDS_WRITER_IOCTL_CHANGE_BUFFER = 0x05,
    TDS_WRITER_IOCTL_CHANGE_HOST = 0x06,
    TDS_WRITER_IOCTL_ADD_HOST = 0x07,
    TDS_WRITER_IOCTL_REMOVE_HOST = 0x08,
    TDS_WRITER_IOCTL_GET_USER_AGENTS = 0x09,
    TDS_WRITER_IOCTL_SET_STREAM_ID = 0x0A
} TDS_WRITER_IOCTL;

typedef enum
{
    TDS_WRITER_STATUS_CONNECTED_CLIENTS = 0x01,
    TDS_WRITER_STATUS_NEW_CLIENT = 0x02,
    TDS_WRITER_STATUS_CLIENT_LEFT = 0x03,
    TDS_WRITER_STATUS_SETTINGS = 0x04,
} TDS_WRITER_STATUS;

typedef enum
{
    TDS_PACKET_RESPONSE_CODE_OK = 0x00,
    TDS_PACKET_RESPONSE_CODE_ERROR = 0x01,
} TDS_PACKET_RESPONSE_CODE;

typedef struct
{
    int port;
    int socket;

    pthread_t threadId;
    int threadStarted;
} tdsServer;

typedef struct
{
    struct sockaddr_in server;
    int sock;

    char host[TDS_SHORT_STRING_LENGTH];
    char user_agent[TDS_SHORT_STRING_LENGTH];
    int port;
    TDS_PROTO protocol;
    uint8_t multicastTTL;

    int videoStreamId;
    int audioStreamId;

    int send_timeout; // timeout in seconds
    struct timeval last_connected;
    int reconnect_timeout_ms;

    uint8_t enabled;
    uint8_t keepalive;
    uint8_t connected;
    uint8_t cont_counter;

    int dscp;
    uint8_t received_ping;
    uint64_t last_ping_received_time_ms;
    int delay_ms;
    uint8_t updated_delay;
} tdsClient;

typedef struct
{
    int print_stats;
    int tick_120;  // use 120kHz ticks
    int dscp;
    int dscp_audio;
    char name[TDS_SHORT_STRING_LENGTH];
    void* avcencHandle;
    int bufferLength; // in ticks
    int maxPacketSize;
    uint32_t cube_id;
    int server_port;
    int max_clients;
    int promiscuous;
    int self_id;
    int source_id;
} tdsWriterConfiguration;

typedef struct
{
    int skipping_frames;
    uint32_t session;

    tdsClient clientZero;
    tdsClient newClientZero;

    tdsClient *clients;
    uint8_t *clientsOn;
    pthread_mutex_t clientListLock;

    tdsServer server;

    tdProfiler profiler;

    ioctlCallbackFunction ioctlCallback;
    void* ioctlCallbackHandler;

    tdsWriterConfiguration cfg;
} TDSWRITER_HANDLE;

/* Bitstream Writer Functions */
BITSTREAMWRITER_HANDLE tdsWriterCreate();
int tdsWriterOpen(BITSTREAMWRITER_HANDLE handle, const char *configStr);
int tdsWriterClose(BITSTREAMWRITER_HANDLE handle);
int tdsWriterNewBlock(BITSTREAMWRITER_HANDLE handle, char *data, int length);
int tdsWriterNewBlockUMA(BITSTREAMWRITER_HANDLE handle, char *data, int length, int srcFD, void *virtualZero);
int tdsWriterNewBlockUnified(BITSTREAMWRITER_HANDLE handle, char *data, int length, int zeroCopy, int srcFD, void *virtualZero);
int tdsWriterIOCTL(BITSTREAMWRITER_HANDLE handle, unsigned int cmd, void *);
int tdsWriterShow(BITSTREAMWRITER_HANDLE handle);
int tdsWriterPwrite(BITSTREAMWRITER_HANDLE handle, char *buf, int count, int offset);
int tdsWriterConfig(BITSTREAMWRITER_HANDLE handle, int param, int value);
int tdsWriterCallback(void *arg, unsigned int *msg, unsigned int *rmsg);
int tdsWriterSetCallback(BITSTREAMWRITER_HANDLE handle, BITSTREAMWRITER_CALLBACK fn, void *arg);
int tdsWriterSwitchFile(BITSTREAMWRITER_HANDLE handle, char *name);
int tdsWriterNewMetadata(BITSTREAMWRITER_HANDLE handle, char *buf, int length, int type, int streamID);

/* Utility/Convenience Functions */
uint32_t tdsElapsed_ms(struct timeval last_time);
uint64_t tdsCurrent_ms();
int tdsHeaderCreate(char *hdr, int type, int i_frame, int first_packet, int last_packet, int tick_120, int continuityCounter, int session);
int tdsParseURL(char *url, TDS_PROTO *proto, char *ip, int *port);
int tdsTcpServerSocketSetup(int port);
int tdsNotify(TDSWRITER_HANDLE *h, int cmd, char *msg, char *misc);
int tdsMessage(TDSWRITER_HANDLE *h, int cmd, char *msg, char *misc);

/* Server/Client Functions */
int tdsClientInit(tdsClient *client);
int tdsClientParseArgs(tdsClient *client, char *configStr);
void tdsClientPrint(tdsClient *client);
int tdsClientConnect(tdsClient *client);
int tdsClientSend(tdsClient *client, char *hdr, int hdfSize, int dscp, char *data, int length, int zeroCopy, int srcFD, void *virtualZero);
int tdsClientReceive(tdsClient *client);
int tdsClientCopy(tdsClient *client, tdsClient *from);
int tdsClientClose(tdsClient *client);

int tdsClientSendResponse(int fd, TDS_PACKET_RESPONSE_CODE code, char *message);

int tdsClientFind(TDSWRITER_HANDLE *h, tdsClient *cli);
int tdsClientAdd(TDSWRITER_HANDLE *h, tdsClient *cli);
int tdsClientForceAdd(TDSWRITER_HANDLE *h, tdsClient *cli);
int tdsClientRemove(TDSWRITER_HANDLE *h, tdsClient *cli);

int tdsServerLog(TDSWRITER_HANDLE *h);
int tdsServerInit(TDSWRITER_HANDLE *h);
int tdsServerShutdown(tdsServer *server);
