/*******************************************************************************
*
* Teradek(C) 2013
* TDS stream reader
*
*******************************************************************************/
#ifndef __TDSREADER_H
#define __TDSREADER_H

/// \defgroup rsgroup_c TDS Bitstream Reader
/// \{

////////////////////////////////////////////////////////////////////////////////
///
/// \page udpreader TDS Bitstream Reader
/// \latexonly
/// \chapter{TDS Bitstream Reader}
/// \endlatexonly
///
/// The TDS bitstream reader is designed to receive TDS streams via a UDP
/// socket.
///
/// The TDS reader takes a list of comma-separated arguments in its filename
/// as part of the Open() call.  The arguments are as follows.
///
/// - port: The UDP port on which to listen
///

#include "refdesign.h"
#include "readstreamer.h"
#include "bsreader.h"

/// \brief Create a TDS reader
/// \param srcId Pointer to an array of stream Id's which will be remapped
/// \param codecId Pointer to an array of stream Id's which are sent to the codec
/// \param nStream Number of stream IDs in the arrays
/// \return Handle to a bitstream reader
BITSTREAMREADER_HANDLE tdsReaderCreate   (int *srcId, int *codecId, int nStreams);

/// \brief Open a bitstream reader.  This function opens the file and starts prefetching
/// bitstream data in anticipation of sending data.
/// \param name Filename to be opened.  The filename is a set of comma separated
/// arguments.  The caller must specify "port" and "bufferlen".
/// \return 1 for success, 0 for failure
int                    tdsReaderOpen     (BITSTREAMREADER_HANDLE h, const char *name);

/// \brief Close a bitstream reader's open file.
/// \param h Handle to a bitstream reader
/// \return 1 for success, 0 for failure
int                    tdsReaderClose    (BITSTREAMREADER_HANDLE h);

/// \brief Request a data block from a bitstream reader.
/// \param h Handle to a bitstraem reader
/// \return buf Pointer to the data buffer
/// \return len Pointer to the length of data in the buffer
/// \return au Pointer to the number of access units in the data block
/// \return type Pointer to the type of data in the data block
/// \return sid Pointer to the stream ID of the block
int                    tdsReaderGetBlock (BITSTREAMREADER_HANDLE h, char **buf, int *len, int *au,
                                          int *type, int *sid);

/// \brief Configure the bitstream reader
/// \param param Parameter index to configure
/// \param value Parameter value to set
/// \return 1 for Success, 0 for failure
int                    tdsReaderConfig   (BITSTREAMREADER_HANDLE h, int param, long long value);

/// \brief This operation is not supported on the UDP reader
int                    tdsReaderSwitchFile(BITSTREAMREADER_HANDLE h, int *srcId, int *codecId, int nStreams, int diffFile);

int                    tdsReaderShow     (BITSTREAMREADER_HANDLE h);

/// \brief Set callback function pointer for the reader
/// \param h Handle to an open reader
/// \param fn Function pointer
/// \param arg Arguments for the callback function
int                    tdsReaderSetCallback(BITSTREAMREADER_HANDLE h, BITSTREAMREADER_CALLBACK fn, void *arg);

/// \brief Ioctl for UDP reader
int                    tdsReaderIOCTL    (BITSTREAMREADER_HANDLE h, int cmd, void *arg);

#endif /* __TDSREADER_H */
