/*
 * Copyright, 2014, Teradek
 *
 * All rights reserved.
 * Contact Teradek for licensing.
 */

#ifndef _TWITCH_API_H_
#define _TWITCH_API_H_

#include <curl/curl.h>

#define TWITCH_API_ENDPOINT     "https://api.twitch.tv/kraken"
#define TWITCH_OAUTH_ENDPOINT   "https://api.twitch.tv/kraken/oauth2/token"

#define PROXY_CONF_FILE            "/etc/proxy.conf"
#define TWITCH_API_CONF_FILE       "/etc/twitch/api"
#define TWITCH_API_AUTH_FILE       "/etc/oauth/twitch"
#define TWITCH_AUTHENTICATION_HOOK "/usr/share/tdapi/twitch_authentication.lua"

#define TWITCH_CLIENT_ID        "pnzjcqm0t084g44u1ufn06jtwsfa63o"
#define TWITCH_CLIENT_SECRET    "gnmahcemw1n6krjyvfem5nhthtdatoo"

#define TWITCH_CONF_MAX_LEN     256

typedef struct
{
    CURL *curl;
} TWITCH_API_HANDLE;

enum
{
    TWITCH_API_ERR_NONE = 0,
    TWITCH_API_ERR_NOMEM = -1,
    TWITCH_API_ERR_UNAUTHORIZED = -2,
    TWITCH_API_ERR_INVALID_PARAM = -3,
    TWITCH_API_ERR_API_ERROR = -4
};

// conf.c
char *twitch_conf_get(char *key, char *value, char *conffile);
int twitch_conf_set(char *key, char *value, char *conffile);

// api.c
void twitch_api_set_verbose(int verbose);
void twitch_api_set_timeout(int connect_timeout, int request_timeout);
int twitch_api_set_endpoints(char *api_endpt, char *oauth_endpt);
TWITCH_API_HANDLE *twitch_api_create_handle(void);
void twitch_api_destroy_handle(TWITCH_API_HANDLE *handle);
void twitch_api_request_free(char **reply);

// authentication.c
int twitch_api_get_access_token(TWITCH_API_HANDLE *h, char *client_id, char *client_secret, char *username, char *password, char *scope, char **reply);
int twitch_api_save_access_token(char *reply);
int twitch_api_deauthorize(char **reply);

// users.c
int twitch_api_get_user_info(TWITCH_API_HANDLE *h, char *access_token, char **reply);
int twitch_api_save_user_info(char *reply);

// broadcast.c
int twitch_api_get_channel(TWITCH_API_HANDLE *h, char *access_token, char **reply);
int twitch_api_get_stream_key(TWITCH_API_HANDLE *h, char *access_token, char *stream_key, char *errorMsg);
int twitch_api_update_channel_status(TWITCH_API_HANDLE *h, char *access_token, char *status, char *game, char **reply);

// ingest.c
int twitch_api_raw_ingest(TWITCH_API_HANDLE *h, char **reply);
int twitch_api_list_ingest(TWITCH_API_HANDLE *h, char **reply);
int twitch_api_best_ingest(TWITCH_API_HANDLE *h, char **reply);

#endif /* _TWITCH_API_H_ */
