/*******************************************************************************
* 
* Teradek(C) 2009
* MPEG2 transport stream through SPI writer
* 
*******************************************************************************/

#ifndef __UDPTSWRITER_H
#define __UDPTSWRITER_H


// The MPEG2 UDP TS bitstream writer is capable of streaming over UDP (or TCP) transport video
// stream when receiving a AVC & MP2 QBOX stream from the MG3500.

// This bitstream writer has a bunch of arguments, including:
/// - protocol : Either 'udp' or 'tcp'
/// - max : Maximum packet size in bytes, 1472 for ethernet
/// - host : Hostname of the destination client as either a hostname or IP address
/// - port : Port number for the destination client 
 
// The writer does not support any IOCTL calls.

#include "writestreamer.h"
#include "refdesign.h"
#include "qboxutil.h"

/// \brief Create an instance of the bitstream writer
/// \return Handle to the bitstream writer
BITSTREAMWRITER_HANDLE udpTsWriterCreate   ();

/// \brief Open the bitstream writer
/// \param h Handle to the bitstream writer
/// \param name The name of the destination file
///
/// This function is used to open a bitstream writer.  Note that this function
/// is not called directly from the application but instead called from the
/// writestreamer's open function.
int                    udpTsWriterOpen     (BITSTREAMWRITER_HANDLE h, const char *name);

/// \brief Close the bitstrea writer and its open file
/// \param h Handle to the bitstream writer
/// \return 1 for success, 0 for failure
int                    udpTsWriterClose    (BITSTREAMWRITER_HANDLE h);

/// \brief Send a new block of data to the writer
/// \param h Handle to the bitstream writer
/// \param buf Pointer to the compressed data block (qbox)
/// \param len Length in bytes of the compressed data block
///
int                    udpTsWriterNewBlock (BITSTREAMWRITER_HANDLE h, char *buf, int len);

/// \brief Call an IOCTL on the writer
/// \brief h Handle to the writer
/// \param cmd Command value of the IOCTL
/// \param arg Void pointer representing an argument to the command
/// 
/// The bitstream writer does not support any IOCTL calls.
int                    udpTsWriterIOCTL    (BITSTREAMWRITER_HANDLE h, unsigned int cmd, void *arg);

/// \brief Display verbose information about the writer
/// \param h Handle to the bitstream writer
/// \return 1 for success, 0 for failure
int                    udpTsWriterShow     (BITSTREAMWRITER_HANDLE h);

/// \brief Configure writer parameters
/// \param h Handle to the bitstream writer
/// \param param Parameter index to configure
/// \param value Parameter value to set
/// \return 1 for success, 0 for failure
int                    udpTsWriterConfig   (BITSTREAMWRITER_HANDLE h, int param, int value);

/// \}
#endif
