/*******************************************************************************
* 
* Teradek(C) 2013
* 
*******************************************************************************/

#ifndef __UPLOADER_H
#define __UPLOADER_H

#include "writestreamer.h"
#include "refdesign.h"
#include "qboxutil.h"

/// \brief Create an instance of the bitstream writer
/// \return Handle to the bitstream writer
BITSTREAMWRITER_HANDLE fileuploaderWriterCreate   ();

/// \brief Open the bitstream writer
/// \param h Handle to the bitstream writer
/// \param name The name of the destination file
///
/// This function is used to open a bitstream writer.  Note that this function
/// is not called directly from the application but instead called from the
/// writestreamer's open function.
int                    fileuploaderWriterOpen     (BITSTREAMWRITER_HANDLE h, const char *name);

/// \brief Close the bitstrea writer and its open file
/// \param h Handle to the bitstream writer
/// \return 1 for success, 0 for failure
int                    fileuploaderWriterClose    (BITSTREAMWRITER_HANDLE h);

/// \brief Send a new block of data to the writer
/// \param h Handle to the bitstream writer
/// \param buf Pointer to the compressed data block (qbox)
/// \param len Length in bytes of the compressed data block
///
int                    fileuploaderWriterNewBlock (BITSTREAMWRITER_HANDLE h, char *buf, int len);

/// \brief Call an IOCTL on the writer
/// \brief h Handle to the writer
/// \param cmd Command value of the IOCTL
/// \param arg Void pointer representing an argument to the command
/// 
/// The bitstream writer does not support any IOCTL calls.
int                    fileuploaderWriterIOCTL    (BITSTREAMWRITER_HANDLE h, unsigned int cmd, void *arg);

/// \}
#endif
