/*
 * Copyright, 2012, Teradek
 * 
 * All rights reserved.
 * Contact Teradek for licensing.
 */

#ifndef _USTREAM_API_H_
#define _USTREAM_API_H_

#ifdef __cplusplus
extern "C" {
#endif

#include <curl/curl.h>

#define USTREAM_OAUTH_ENDPOINT  "https://www.ustream.tv/oauth2/token"
#define USTREAM_API_ENDPOINT    "https://api.ustream.tv"
#define USTREAM_REDIRECT_URL    "https://ustream.tv/oauth2/redirect"
#define USTREAM_PING_URL        "http://api.ustream.tv/json/system/status/ping"

#define PROXY_CONF_FILE         "/etc/proxy.conf"
#define USTREAM_API_CONF_FILE 	"/etc/ustream/api"
#define USTREAM_SETTINGS_FILE   "/etc/oauth/ustream"
#define USTREAM_API_AUTH_FILE  	"/etc/oauth/ustream"

#define USTREAM_AUTHENTICATION_HOOK "/usr/share/tdapi/ustream_authentication.lua"

#define USTREAM_CLIENT_KEY      "5b0ead85b705fb1b3ec87b75a8008c62563cdb3f"
#define USTREAM_CLIENT_SECRET   "b7c046ee14d20daa0aba2e0666bfd842dd99e28b"

#define USTREAM_CONF_MAX_LEN     256

typedef struct
{
    CURL *curl;
} US_API_HANDLE;

typedef struct
{
    char id[USTREAM_CONF_MAX_LEN];
    char title[USTREAM_CONF_MAX_LEN];
    char broadcast_url[2][USTREAM_CONF_MAX_LEN];
} US_API_CHANNEL;

enum
{
    USTREAM_API_ERR_NONE = 0,
    USTREAM_API_ERR_NOMEM = -1,
    USTREAM_API_ERR_UNAUTHORIZED = -2,
    USTREAM_API_ERR_INVALID_PARAM = -3,
    USTREAM_API_ERR_INVALID_RESPONSE = -4,
    USTREAM_API_ERR_ERROR_RESPONSE = -5
};

US_API_HANDLE *ustream_api_create_handle(void);
void ustream_api_destroy_handle(US_API_HANDLE *handle);

char *ustream_conf_get(char *key, char *value, char *conffile);
int ustream_conf_set(char *key, char *value, char *conffile);

void ustream_api_set_verbose(int verbose);
void ustream_api_set_timeout(int connect_timeout, int request_timeout);
int ustream_api_set_endpoints(char *api_endpt, char *oauth_endpt);

int ustream_api_get_access_token(US_API_HANDLE *h, char *client_id, 
                char *client_secret, char *username, char *password, char **reply);
int ustream_api_save_access_token(char *reply);
int ustream_api_deauthorize(char **reply);

int ustream_api_get_streaming_key(US_API_HANDLE *h, char *access_token, char *channel_id, char **reply);
int ustream_api_get_streaming_key_string(US_API_HANDLE *h, char *access_token, char *channel_id, char *key, char *errorMsg);

int ustream_api_get_channels(US_API_HANDLE *h, char *access_token, char **reply);
int ustream_api_get_channel(US_API_HANDLE *h, char *access_token, char *channel_id, US_API_CHANNEL *ch, char *errorMsg);
int ustream_api_create_channel(US_API_HANDLE *h, char *access_token,  char *title, char *description, char **reply);

int ustream_api_get_password(US_API_HANDLE *h, char *access_token, char *channel_id, char **reply);
int ustream_api_set_password(US_API_HANDLE *h, char *access_token, char *channel_id, char *password, char **reply);
int ustream_api_rem_password(US_API_HANDLE *h, char *access_token, char *channel_id, char **reply);

int ustream_api_get_user_info(US_API_HANDLE *h, char *access_token, char **reply);
int ustream_api_save_user_info(char *reply);

int ustream_api_ping(void);

void ustream_api_request_free(char **reply);

#ifdef __cplusplus
}
#endif

#endif /* _USTREAM_API_H_ */
