/*
 * Copyright, 2014, Teradek
 *
 * All rights reserved.
 * Contact Teradek for licensing.
 */

#ifndef _WOWZA_API_H
#define _WOWZA_API_H

#include <curl/curl.h>

#define WOWZA_API_ENDPOINT          "https://api.cloud.wowza.com/api/v1/token"

#define WOWZA_PROXY_CONF_FILE       "/etc/proxy.conf"
#define WOWZA_API_CONF_FILE         "/etc/wowza/api"
#define WOWZA_SETTINGS_FILE         "/etc/oauth/wowza"
#define WOWZA_API_AUTH_FILE         "/etc/oauth/wowza"

#define WOWZA_AUTHENTICATION_HOOK   "/usr/share/tdapi/wowza_authentication.lua"

#define WOWZA_API_KEY               "5SnsiQM2hLxO0lKUt8rTzvXtdwr2esKgxeHKnxAWRVIfmuGZ1EIV3uWByHRAXbSa"

#define WOWZA_CONF_MAX_LEN          256

typedef struct
{
    CURL *curl;
} WOWZA_API_HANDLE;

typedef struct
{
    int depth;
    char url[WOWZA_CONF_MAX_LEN];
    char stream[WOWZA_CONF_MAX_LEN];
    char username[WOWZA_CONF_MAX_LEN];
    char password[WOWZA_CONF_MAX_LEN];
    char *last_content;
    const char *error_string;
} wowza_parse_t;

typedef struct
{
    int id;
    char name[WOWZA_CONF_MAX_LEN];
    char protocol[WOWZA_CONF_MAX_LEN];
    char domain_name[WOWZA_CONF_MAX_LEN];
    int port;
    char application_name[WOWZA_CONF_MAX_LEN];
    char stream_name[WOWZA_CONF_MAX_LEN];
    char publish_username[WOWZA_CONF_MAX_LEN];
    char publish_password[WOWZA_CONF_MAX_LEN];
    char state[WOWZA_CONF_MAX_LEN];
} WOWZA_API_TRANSCODER;

enum
{
    WOWZA_API_ERR_NONE = 0,
    WOWZA_API_ERR_NO_MEM = -1,
    WOWZA_API_ERR_INVALID_URL = -2,
    WOWZA_API_ERR_INVALID_XML = -3,
    WOWZA_API_ERR_INVALID_PARAMS = -4,
    WOWZA_API_ERR_XML_PARSE = -5,
    WOWZA_API_ERR_UNAUTHORIZED = -6,
    WOWZA_API_ERR_TOKEN_SAVE = -7,
    WOWZA_API_ERR_INVALID_RESPONSE = -8,
    WOWZA_API_ERR_INT_SERVER = -9,
    WOWZA_API_ERR_ERROR_RESPONSE = -10,
    WOWZA_API_ERR_INVALID_CODE = -11
};

WOWZA_API_HANDLE *wowza_api_create_handle(void);
void wowza_api_destroy_handle(WOWZA_API_HANDLE *handle);

char *wowza_api_conf_get(char *key, char *value, char *conffile);
int wowza_api_conf_set(char *key, char *value, char *conffile);

void wowza_api_set_verbose(int verbose);
void wowza_api_set_timeout(int connect_timeout, int request_timeout);

//transcoders.c
int wowza_api_transcoders_validate(WOWZA_API_HANDLE *h,char *short_token, char **reply);
int wowza_api_transcoders_info(WOWZA_API_HANDLE *h, char *access_token, char **reply);
int wowza_api_transcoders_get_info(WOWZA_API_HANDLE *h, char *access_token, WOWZA_API_TRANSCODER *transcoder, char *errorMsg);
int wowza_api_transcoders_save_info(WOWZA_API_HANDLE *h, char *access_token);
int wowza_api_transcoders_state(WOWZA_API_HANDLE *h, char *access_token, char **reply);
int wowza_api_transcoders_state_string(WOWZA_API_HANDLE *h, char *access_token, char *state, char *errorMsg);
int wowza_api_transcoders_start(WOWZA_API_HANDLE *h, char *access_token, char **reply);
int wowza_api_transcoders_stop(WOWZA_API_HANDLE *h, char *access_token, char **reply);
int wowza_api_transcoders_reset(WOWZA_API_HANDLE *h, char *access_token, char **reply);

int wowza_api_read_xml(char *filename, wowza_parse_t *wowza_xml);
int wowza_api_read_url(char *url, wowza_parse_t *wowza_url);

int wowza_api_save_access_token(char *reply);
int wowza_api_deauthorize(char **reply);

void wowza_api_free_request(char **reply);

 #endif /* _WOWZA_API_H */
