global.__ROOT__ = __dirname;

var _                = require('underscore'),
    fs               = require('fs'),
    daemon           = require('daemon'),
    redis            = require('redis'),
    webserver        = require('./app/index'),
    pid_util         = require('./utils/pid'),
    configFileReader = require('./config/config'),
    logger,
    pid,
    argv;


process.on('uncaughtException', function (err) {
    console.error('Caught exception: ', err.stack || err.toString());
});

/* --- Clean up after yourself if you are killed --- */

process.on('exit', function(signal) {
    pid.killall()(function() {
        process.exit();
    });
});

process.on('SIGINT', function(signal) {
    pid.killall()(function() {
        process.exit();
    });
});

/* --- Parse arguments --- */

argv = _.reduce(process.argv.slice(2), function(memo, arg) {
    if (arg.charAt(0) === '-') {
        var parts = arg.slice(1).split('=');
        memo[parts[0]] = (parts.length === 2) ? parts[1] : '';
    }
    return memo;
}, {});

pid = pid_util(argv.pid || __dirname + '/sputnik-www.pid');

var defaults = {
     web_port   : argv.port  || 1957
    ,redis_port : argv.redis || 6379
    ,view_only  : Boolean(argv['view-only'])
    ,outFile    : argv.log     || '/tmp/sputnik-node-output.log'
};




configFileReader.mergeNewConfig()
                .then(function(){
        configFileReader.get()(
            function( config ){
                var fullConfig = _.defaults( defaults, config );
                startProcess( fullConfig );
            },
            function(){
                console.log('Unable to read the config file. Starting with default options.');
                startProcess(defaults);
        });

});

var startProcess = function(config) {
    logger = require('./utils/logger')(config);

    if (process.env.__daemon) {
        pid.add(process.pid);
        new webserver(config);
    } else {

        var arg = process.argv[2];

        if (arg === 'start' || arg === 'restart') {
            pid.killall()(function() {
                daemon({
                    //stdout : fs.openSync(argv.log || '/tmp/sputnik-out.log', 'a'),
                    //stderr : fs.openSync(argv.log || '/tmp/sputnik-err.log', 'a')
                });
            });
        } else if (arg === 'starti') {
            new webserver(config);
        } else if (arg === 'stop') {
            pid.killall()(function() {
                process.exit();
            });
        } else {
            console.log('app.js available arguments: start, starti, restart, stop, [port=XXXX]');
            process.exit();
        }
    }

}