var  https = require('https')
    ,http  = require('follow-redirects').http
    ,fs = require('fs')
    ,os = require('os')
    ,url_parser = require('url')
    ,uname = require('uname')
    ,exec = require('child_process').exec
    ,spawn = require('child_process').spawn
    ,targz = require('tar.gz')
    ,properties = require('properties')
    ,url = require('url')
    ,_ = require('underscore')
    ,deferred = require('deferred');

var CORE_HOST = null;
var CACHE = {};
var CERT_DIR = '/usr/share/sputnik-core-client/certs/';
var CERT_FILE = CERT_DIR + 'sputnik.crt';
var KEY_FILE = CERT_DIR + 'sputnik.key';
var CORE_CONF = '/etc/sputnik-core.conf';
var SPUTNIK_CONF = '/etc/sputnik.conf';
//note that the core server API version may not match the API version of the Teradek ID server.
var DEFAULT_CORE_URL_PREFIX = 'https://api-core.teradek.com/api/v1.0';
var DEFAULT_DEV_TERADEK_ID_URL_PREFIX = 'https://id-dev.teradek.com:8443/api/v1.1';
var DEFAULT_PROD_TERADEK_ID_URL_PREFIX = 'https://api-id.teradek.com/api/v1.1';
var DEFAULT_PROD_CORE_HOST = 'core.teradek.com';
var DEFAULT_DEV_PORT = 8443;

module.exports = function(self, app, checkAdmin) {
    if( app.enabled('development_mode') ){
        var UPDATE_SERVER_URL = 'http://krypton.teradek.priv/update.teradek.com/';
        var CORE_CLIENT_SERVER_URL ='https://teradek-firmdata.s3.amazonaws.com/Public/SputnikCoreClient/Scripts/version.json';

    }
    else{
        var UPDATE_SERVER_URL ='http://update.teradek.com/json/';
        var CORE_CLIENT_SERVER_URL ='https://teradek-firmdata.s3.amazonaws.com/Public/SputnikCoreClient/Scripts/version.json';
    }

    var JSONFormattedErrorMsg = function(message) {
        this.name = 'JSONFormattedErrorMsg';
        this.message = message;
        this.stack = (new Error()).stack;
    }
    JSONFormattedErrorMsg.prototype = new Error();
    JSONFormattedErrorMsg.constructor = JSONFormattedErrorMsg;

    var getCoreHostURLPrefix = function(){
        var def = deferred();

        if( !( 'coreURLPrefix' in CACHE ) ){
            CACHE['coreURLPrefix'] = DEFAULT_CORE_URL_PREFIX;

            readCoreConf().then(function(settings){
                if( 'core_api_url' in settings ){
                    var core_host_prefix = settings['core_api_url'].replace(/"/g, '');
                    CACHE['coreURLPrefix'] = core_host_prefix;
                }
                //handle dev server versions
                else if( 'core_host' in settings && settings['core_host'] != DEFAULT_PROD_CORE_HOST ){
                    var parsedURL = url.parse( DEFAULT_CORE_URL_PREFIX );
                    var core_host = settings['core_host'].replace(/"/g, '');
                    if( 'core_api_port' in settings ){
                       parsedURL.port = parseInt(settings['core_api_port']);
                    }
                    else{
                        parsedURL.port = DEFAULT_DEV_PORT;
                    }
                    parsedURL.hostname = core_host;
                    delete parsedURL.host;
                    CACHE['coreURLPrefix'] = url.format(parsedURL);
                }

                def.resolve(CACHE['coreURLPrefix']);

            }, function(){

                def.resolve(CACHE['coreURLPrefix']);
                //def.reject(new Error("unable to read the sputnik core configuration file"));
            });
        }
        else{
            def.resolve(CACHE['coreURLPrefix']);
        }
        return def.promise;
    }

    var getTeradekIDServerURLPrefix = function(){
        var def = deferred();
        if( !( 'teradekIDServerURLPrefix' in CACHE ) ){
            CACHE['teradekIDServerURLPrefix'] = DEFAULT_PROD_TERADEK_ID_URL_PREFIX;

            readCoreConf().then(function(settings){
                if( 'core_host' in settings ){
                    if( 'teradek_id_url' in settings ){
                        CACHE['teradekIDServerURLPrefix'] = settings['teradek_id_url'];
                    }
                    //if core_host is dev, then core_id_host is also dev
                    else if( settings['core_host'].match(/dev/) ){
                        CACHE['teradekIDServerURLPrefix'] = DEFAULT_DEV_TERADEK_ID_URL_PREFIX;
                    }
                }
                def.resolve(CACHE['teradekIDServerURLPrefix']);

            }, function(){
                def.resolve(CACHE['teradekIDServerURLPrefix']);
                //def.reject(new Error("unable to read the sputnik core configuration file"));
            });
        }
        else{
            def.resolve(CACHE['teradekIDServerURLPrefix']);
       }
        return def.promise;
    }

    var download = function( url, binary ){
        var def = deferred();
        console.log('download url', url.replace(/password=[^&]+/, 'password=[redacted]'));
        var url_components = url_parser.parse( url );
        var options = {
             host: url_components.hostname
            ,port: url_components.port
            ,path: url_components.path
        };

        if( url_components.protocol == 'http:' ){
            var httpClient = http;
        }
        else{
            var httpClient = https;
        }

        var req = httpClient.get(options, function( response ){
              var res_data = '';
              if( binary ){
                  response.setEncoding('binary');
              }
              response.on('data', function( chunk ){
                res_data += chunk;
              });

              response.on('end', function() {
                def.resolve(res_data);
              });
        });

        req.on('error', function(err) {
            console.log("Request error: " + err.message);
            def.resolve(new Error(err.message));
        });
        return def.promise;
    }

    var rpmPkgCheck = function(){
        var def = deferred();
        exec('rpm -q teradek-sputnik-core', function( error, stdout, stderr ){
            console.log("RPM CHECK OUTPUT", arguments)
            if (error !== null || stderr != '' || stdout.match(/is not installed/g) ) {
                console.log("rpm resolve as false")
              def.resolve(false);
            }
            else{
               def.resolve(true);
            }

        });
         return def.promise;
    }

    var debPkgCheck = function(){
        var def = deferred();
        exec('dpkg -s teradek-sputnik-core', function( error, stdout, stderr ){
            console.log("DEBCHECK OUTPUT", arguments)

            if (error !== null ) {
                console.log("deb resolve as false")
                def.resolve(false);
            }
            else if( stdout.match(/^Status: install ok installed/gm) ){
                def.resolve(true);
            }
            else{
                def.resolve(false);
            }
        });
        return def.promise;
    }

    var readCoreConf = function( enableCoreClient ){
        var def = deferred();

          properties.parse( CORE_CONF, {path: true},
          function( error, settings ){
            if( error ){
                console.log('Error reading file:', CORE_CONF, error);
                properties.parse( SPUTNIK_CONF, {path: true}, function(error, settings){
                   if( error ){
                       console.log('Error reading file:', SPUTNIK_CONF, error);
                       def.resolve(new Error(error)); }
                   else{
                       def.resolve(settings);
                   }
                });
            }
            else{
                def.resolve(settings);
            }
        });
        return def.promise;
    };

    var coreConf = function( coreClientState ){

        var def = deferred();
        fs.readFile( CORE_CONF, 'utf8', function ( err, data ){
            if( err ){
                def.resolve(false);
            }
            if( !(coreClientState) ){
                if( typeof data == 'string' && data.match(/^core_client=yes$/gm) ){
                     def.resolve(true);
                }
                else{
                    def.resolve(false);
                }
            }
            else{
              if( coreClientState == 'disable' ){
                  data = data.replace(/^core_client=.*$/gm, 'core_client=no');
              }
              else{
                data = data.replace(/^core_client=.*$/gm, 'core_client=yes');
              }
              fs.writeFile(CORE_CONF, data, 'utf8', function (err) {
                 if (err){
                    console.log(err);
                    def.resolve(err);
                 }
                 def.resolve(data);
              });
            }
        });
        return def.promise;
    };

    var isCoreClientRunning = function(){
        try{
            var pidFile = fs.readFileSync('/var/run/sputnik-core.pid');
            var pid = pidFile.toString();
            var kill = process.kill(pid, 0);

            return true;
        }
        catch(e){
            if( e.code == 'ENOENT' ){ //pid file doesn't exist
                return false;
            }
            if( e.code == 'ESRCH' ){ //pid exists, process not running
                return false;
            }

        }
    }

    var readyForCore = function(){
        var status = {};
        var def = deferred();

        //do the ssl certificates exist
        if( fs.existsSync( CERT_FILE )
                && fs.existsSync( KEY_FILE ) ){
            status.certs = true;
        }
        else{
            status.certs = false;
        }

        status.core_client = false;
        //is core client running?

        if( isCoreClientRunning() ){
            status.core_client_running = true;
            status.core_client_installed = true;
            status.core_client = true;
            def.resolve(status);
        }
        else{
            status.core_client_running = false;
            status.core_client = false;
            //is core client installed (rpmPkgCheck, debPkgCheck)
            //and are sputnik settings enabled to run core client (coreConf)
            deferred(rpmPkgCheck(), debPkgCheck(), coreConf())(function (result) {
              status.core_client_enabled = result.pop();

              var installed = false;
              for( var i in result ){
                  installed = installed || result[i];
              };
              status.core_client_installed = installed;
              def.resolve(status);
            });
        }
        return def.promise;
    }

    //step 1
    // company is optional and is only passed when an account is ssociated with multiple companies
    // and a user must choose the company to proceed
    var getAuthToken = function( email, passwd, user_selected_company ){
        var def = deferred();
        email = encodeURIComponent(email);
        passwd = encodeURIComponent(passwd);

        getTeradekIDServerURLPrefix()(function( url_prefix ){
           return download( url_prefix+'/auth/login?email='+email+'&password='+passwd);
        }).then(function( auth_token_data ){
            try{
                auth_token_data = JSON.parse(auth_token_data);

                if( auth_token_data.meta.status == "ok" ){
                    var auth_token = auth_token_data.response.auth_token;
                    def.resolve({auth_token:auth_token, user_selected_company:user_selected_company});
                }
                else{
                    def.resolve( new Error('Failed to get Authorization Token'));
                }
            }
            catch(e){
                def.resolve( new Error('Failed to get Authorization Token: Invalid JSON format') );
            }
        }, function(e){
             //request error
             def.resolve( e );
        });
        return def.promise;
    }


    // step 2
    var getCompanyID = function( options ){
        var def = deferred();
        if( 'user_selected_company' in options && options.user_selected_company ){
            def.resolve( options );
        }
        else{
            getTeradekIDServerURLPrefix().then( function( url_prefix ){
                auth_token = options.auth_token;
                download( url_prefix + '/me/companies?auth_token='+auth_token ).then( function( data ){
                    try{
                        data = JSON.parse( data );
                    }
                    catch( e ){
                        def.resolve( new Error( "Invalid JSON format when trying to get Company ID" ) );
                        return false;
                    }

                    if( data.meta.status != 'ok' ){
                        def.resolve( new JSONFormattedErrorMsg( data ) );
                        return false;
                    }

                    var companies = data.response;

                    if( companies.length >= 1 ){
                        options.companies = companies;
                        def.resolve( options );
                    }
                    else{
                        // no account associated yet
                        console.log('Core Authentication failed to get Companies', data);
                        def.resolve( new Error("No Account associated yet") );
                    }


                }, function(){
                    def.resolve( new Error("Failed to get Company Info") );
                });
            });
        }

        return def.promise;
    }

    //step 3
    var downloadCertificates = function( options ){
        var def = deferred();

        //multiple companies user must choose from first
        if( 'companies' in options && options.companies.length > 1 ){
            def.resolve( options );
        }
        else{
            var auth_token = options.auth_token;
            var company = ( 'user_selected_company' in options
                                    && options.user_selected_company ) ? options.user_selected_company : options.companies[0].slug;

            getCoreHostURLPrefix()(function( url_prefix ){
                var base_url = url_prefix + '/'+company+'/sputniks/certificate';
                var url = base_url +'?auth_token='+auth_token;
                download( url, 'binary' )
                .then(function( data ){
                    try{
                        //detect json error
                        var jsonError = JSON.parse(data);
                        if( 'meta' in jsonError && 'error' in jsonError.meta ){
                            def.resolve( new Error( jsonError.meta.error.message, jsonError ) );
                        }
                        else{
                            def.resolve( new Error( "Could not download certificates." ) );
                        }

                    }
                    catch(e){
                        if( data.search(/^Cannot GET/) != -1 ){
                            def.resolve( new Error( "Unable to reach URL:"+ base_url ) );
                            return false;
                        }
                        var tmpFile = os.tmpdir()+'/sputnik-core-certs.tar.gz';
                        fs.writeFile( tmpFile, data, 'binary', function(){
                            try{
                                var compress = new targz().extract( tmpFile, CERT_DIR, function(err){
                                    if( err ){
                                        console.log(err);
                                        def.resolve( new Error( "Could not extract certificates tar.gz ", tmpFile, 'to ', CERT_DIR ) );
                                    }
                                    def.resolve( { companies:[company], certs:'successfully installed certificates' } );
                                    fs.unlinkSync( tmpFile );
                                });
                            }
                            catch(e){
                                def.resolve(e);
                            }
                        });
                    }
                });
            });
        }

        return def.promise;
    };

    app.post( '/core/install/certs', checkAdmin, function(req, res){
         var email = req.param('email');
         var passwd = req.param('password');
         var user_selected_company = req.param('company');

         getAuthToken( email, passwd, user_selected_company )
                .then( getCompanyID )
                .then( downloadCertificates )
                .then( function( data ){
                    delete data.auth_token;
                    res.json({meta:{status:'ok', error:null}, response:data});
                },
                function( error ){
                        if( error.name == 'JSONFormattedErrorMsg' ){
                            res.json( error.message );
                        }
                        else{
                            res.json( {meta:{ status:"error", "error":{ message:error.message }}} );
                        }
                    });
    });

    var serialize = function(obj){
      var str = [];
      for( var p in obj ){
        if( obj.hasOwnProperty(p) ){
          str.push(encodeURIComponent(p) + "=" + encodeURIComponent(obj[p]));
        }
      }
      return str.join("&");
    }

    var getUpdates = function( opts ){
        _.defaults( opts, {v:2});
        var q = serialize(opts);
        if( opts.product == 'sputnik-core-client' ){
            return download( CORE_CLIENT_SERVER_URL+'?'+q );
        }
        else{
            return download( UPDATE_SERVER_URL+'?'+q );
        }

    };

    var getMyLinuxFlavor = function(){
        var linuxFlavors = [
             {type:'amzn', id_file:'/etc/system-release'}
            ,{type:'rpm', id_file:'/etc/redhat-release'}
            ,{type:'deb', id_file:'/etc/debian_version'}
        ];

        for( var i in linuxFlavors ){
            if( fs.existsSync( linuxFlavors[i].id_file ) ){
                var type = linuxFlavors[i].type;
                var id = fs.readFileSync(linuxFlavors[i].id_file).toString();
            }
        }
        if( !(id) ){
            console.log('unknown version of linux');
            id = 'unknown';
        }
        var unameData = uname.uname();
        unameData['releaseFileType'] = type;
        unameData['releasefile'] = id.replace(/(?:(?:^|\n)\s+|\s+(?:$|\n))/g,'').replace(/\s+/g,' ');
        return unameData;
    };


    var downloadCoreClient = function( url ){
        try{
            var def = deferred();
            var filenameMatches = url.match(/[^\/]+$/);
            if( filenameMatches ){
                var filename = filenameMatches[0];
                var extension = /[^.]+$/.exec(filename);
            }
            download(url, 'binary')(function(data){
                var tmpFile = os.tmpdir()+'/'+filename;

                fs.writeFile( tmpFile, data, 'binary', function(){
                        if( extension == 'rpm' ){
                          exec('rpm -ivh '+tmpFile, function( error, stdout, stderr ){
                              console.log(error, stdout, stderr );
                              if (error !== null || stderr != '' ){
                                    def.resolve(false);
                              }
                              else{
                                  def.resolve(true);
                              }
                              fs.unlinkSync(tmpFile);

                          });
                        }
                        else{

                          exec('dpkg --force-confmiss -i '+tmpFile, function( error, stdout, stderr ){
                              console.log( error, stdout, stderr );
                              if (error !== null ){
                                    def.resolve(new Error( error ) );
                              }
                              else{
                                  def.resolve(true);
                              }
                              fs.unlinkSync(tmpFile);

                          });
                        }
                    });
              });
        }
        catch(e){
            console.log(e);
            def.resolve(e);
        }

        return def.promise;
    };

    var isRPMBased = function(){
        var def = deferred();
        exec('rpm -q rpm', function( error, stdout, stderr ){
            if (error !== null ) {
              def.resolve(false);
            }
            else{
               def.resolve(true);
            }
        });
         return def.promise;
    }

    var isDEBBased = function(){
        var def = deferred();
        exec('/usr/bin/dpkg --search /usr/bin/dpkg', function( error, stdout, stderr ){
            if (error !== null ) {
                def.resolve(false);
            }
            else{
                def.resolve(true);
            }
        });
        return def.promise;
    }

var findCoreClientUpdateForMyPlatform = function( platform, sysInfo ){
        var def = deferred();
        var sysInfo = getMyLinuxFlavor();
        getUpdates( {product:platform, sysInfo:JSON.stringify(sysInfo)} )(function(data){
            data = JSON.parse(data);

            var updateInfoObj = {};
            var coreData = data[platform];
            var latestVersion = coreData.latest_version;
            _.each(coreData.downloads.latest, function(v, k){
                updateInfoObj[k] = {};
                updateInfoObj[k]['latest'] = {
                    version: latestVersion
                    ,url:v
                }
            });

            if( 'releaseFileType' in sysInfo ){ //linux server products should have a 'releaseFileType'
                //guess on my own
                var fileType = sysInfo['releaseFileType'];
                var machine = ( sysInfo['machine'].match(/64/) ) ? 'amd64' : 'i386';
                var combinedArchAndFileType = machine+fileType;
                if( combinedArchAndFileType in updateInfoObj ){
                    def.resolve( updateInfoObj[combinedArchAndFileType] );
                }
                else if( fileType in updateInfoObj ){
                    def.resolve( updateInfoObj[fileType] );
                }
                else{
                    deferred( isRPMBased(), isDEBBased() )(function( result ){
                        console.log('isRPMBased', result[0]);
                        console.log('isDebBased', result[1]);
                        if( result[0] ){
                            def.resolve( updateInfoObj['i686rpm'] );
                        }
                        else if( result[1] ){
                            var combinedArchAndFileType = machine+'deb';
                            if( combinedArchAndFileType in updateInfoObj ){
                                def.resolve( updateInfoObj[combinedArchAndFileType] );
                            }
                            else{
                                def.resolve( new Error("Unable to find update for platform") );
                            }

                        }
                        else{
                            def.resolve( new Error("Unable to find update for platform") );
                        }
                    })


                }
            }
            else{
                def.resolve( new Error("system information not available to figure out which platform") );
            }
        });

        return def.promise;
    }


    var findUpdateForMyPlatform = function( platform, sysInfo ){
        var def = deferred();
        var sysInfo = getMyLinuxFlavor();
        getUpdates( {product:platform, sysInfo:JSON.stringify(sysInfo)} )(function(data){
            data = JSON.parse(data);
            var updateInfoObj = data[platform];
            var downloadLinkKeys = Object.keys(updateInfoObj);

            //if update server eliminated the choices to just 1
            if( Object.keys(updateInfoObj).length == 1 ){
                def.resolve( updateInfoObj[downloadLinkKeys[0]] );
            }
            else if( 'releaseFileType' in sysInfo ){ //linux server products should have a 'releaseFileType'
                //guess on my own
                var fileType = sysInfo['releaseFileType'];
                var machine = ( sysInfo['machine'].match(/64/) ) ? '64bit' : '32bit';
                var combinedArchAndFileType = machine+fileType;
                if( combinedArchAndFileType in updateInfoObj ){
                    def.resolve( updateInfoObj[combinedArchAndFileType] );
                }
                else if( fileType in updateInfoObj ){
                    def.resolve( updateInfoObj[fileType] );
                }
                else{
                    deferred( isRPMBased(), isDEBBased() )(function( result ){
                        console.log('isRPMBased', result[0]);
                        console.log('isDebBased', result[1]);
                        if( result[0] ){
                            if( 'rpm' in updateInfoObj ){
                                def.resolve( updateInfoObj['rpm'] );
                            }
                            else{
                                def.resolve( new Error("Unable to find update for platform") );
                            }
                        }
                        else if( result[1] ){
                            var combinedArchAndFileType = machine+'deb';
                            if( combinedArchAndFileType in updateInfoObj ){
                                def.resolve( updateInfoObj[combinedArchAndFileType] );
                            }
                            else{
                                def.resolve( new Error("Unable to find update for platform") );
                            }

                        }
                        else{
                            def.resolve( new Error("Unable to find update for platform") );
                        }
                    })

                }
            }
            else{
                def.resolve( new Error("system information not available to figure out which platform") );
            }
        });

        return def.promise;
    }

    var getNewCoreClientUpdateURL = function(){
        var def = deferred();
        var myLinuxSysInfo = getMyLinuxFlavor();
        var updateData = findCoreClientUpdateForMyPlatform( 'sputnik-core-client', myLinuxSysInfo )(function( updateData ){
            if( 'latest' in updateData ){
                def.resolve(updateData.latest.url);
            }
            else{
                def.resolve(false);
            }
        });
        return def.promise;
    }


    app.delete('/core/install/certs', checkAdmin, function(req, res){
        try{
            fs.unlinkSync(CERT_FILE);
            fs.unlinkSync(KEY_FILE);
        }
        catch(e){
            res.json({'error':'failed to delete certs'});
        }
        res.json({'success':'woot'});

    });

    app.get( '/sputnik/updates', checkAdmin, function(req, res){
        var myLinuxSysInfo = getMyLinuxFlavor();
        findUpdateForMyPlatform( 'sputnik', myLinuxSysInfo )(function( updateData ){
            if( updateData ){
                res.json({status:'success', data:updateData});
            }
            else{
                res.json({status:'error'});
            }
        }, function(e){
                    res.json({status:'error', data:e.message});
            });
    });

    app.get( '/core/updates', checkAdmin, function(req, res){
        var myLinuxSysInfo = getMyLinuxFlavor();
        findCoreClientUpdateForMyPlatform( 'sputnik-core-client', myLinuxSysInfo )(function( updateData ){
            if( updateData ){
                res.json({status:'success', data:updateData});
            }
            else{
                res.json({status:'error'});
            }
        }, function(e){
                    res.json({status:'error', data:e});
            });
    });

    app.get( '/core/version', checkAdmin, function(req, res){
        fs.readFile('/opt/teradek/sputnik-core/VERSION', 'utf8', function( err, data ){
          if (err) {
            console.log(err);
            res.json({status:'error', data:err});
          }
          else{
              res.json({status:'success', data:data});
          }
        });
    });

    app.get( '/core/install/core-client', checkAdmin, function(req, res){

            var url = getNewCoreClientUpdateURL()(function(url){
            //figured out the right version to download
            if( url ){
                downloadCoreClient(url)(function(){
                    console.log('downloaded the core client')
                    res.json({'success':'woot'});
                }, function(error){
                    console.log('Error: Failed to download Core Client -', error);
                    if( error.name == 'JSONFormattedErrorMsg' ){
                        res.json( error.message );
                    }
                    else{
                        res.json( {meta:{ status:"error", "error":{ message:error.message }}} );
                    }
                    return false;
                });
            }
            else{
                //unable to guess if it's a debian or rpm based
                res.json(new Error(data));
            }
        });
    });

    app.get( '/core/install/enable-core-client', checkAdmin, function(req, res){
        coreConf('enable')(function(data){
            res.json({status:'ok', data:data});
        });
    });

    app.get( '/core/install/disable-core-client', checkAdmin, function(req, res){
        coreConf('disable')(function(data){
            res.json({status:'ok', data:data});
        });
    });

    app.get( '/core/install/status', checkAdmin, function(req, res){
        readyForCore().then(function( status ){
            res.json( status );
        });
    });

    app.get('/core/restart', checkAdmin, function(req, res){
        console.log('restarting core client');
        exec('/opt/teradek/sputnik-core/core -k', function( error, stdout, stderr ){
            if (error !== null || stderr != '' ) {
                console.log("Unable to kill core client:", stderr);
            }
            //spawns long running core client process
            var core_client = spawn('/opt/teradek/sputnik-core/core', ['-s'] );
            var checkCoreClientAttempts = 0;
            var checkCoreClientIntervalID = setInterval(function(){
                checkCoreClientAttempts++;
                if( isCoreClientRunning() ){
                    res.json({status:'ok'});
                    console.log('core client successfully restarted');
                    clearInterval(checkCoreClientIntervalID);
                }
                if( checkCoreClientAttempts > 20 ){
                    clearInterval(checkCoreClientIntervalID);
                    res.json({status:'error'});
                    console.log('unable to restart core client');
                }
            }, 500);

        });
    });
};