var fs = require('fs')
 ,path = require('path')
 ,  _ = require('underscore')
 ,exec = require('child_process').exec
 ,archiver = require('archiver');


module.exports = function(self, app, checkAdmin, config) {
    if( 'upload_path' in config ){
        var upload_path = config.upload_path;
    }
    else{
        console.log('Warning:The configured upload path is not configured. Using default.');
        var upload_path = '/opt/teradek/sputnik/upload/';
        process.env.SPUTNIK_UPLOADS_PATH = upload_path;
        exec('export SPUTNIK_UPLOADS_PATH='+upload_path);
    }
    if( !(fs.existsSync(config.upload_path)) ){
        console.log('Warning:The configured upload path does not exist.');
    }

    if(upload_path.substr(-1) != '/'){
        upload_path += '/';
    }

    console.log('fileTransfer(): using this upload path', upload_path);
    console.log(process.env.SPUTNIK_UPLOADS_PATH)

    app.get('/json/local/files/download', checkAdmin, function(req, res){
        /*var dir = req.query.dir;

        if( dir.match(/\/\.\./) ){
            res.json({error:'illegal file path'});
        }
        else{
            var file = upload_path + dir;
        }*/

        try{

            var files = _.map( JSON.parse(req.query.files), function( v ){
                var filepath = v.path;
                if( filepath.match(/\/\.\./) ){
                    return null;
                }
                if( filepath.charAt(0) == '/'){
                    filepath = filepath.substr(1);
                }
                return {expand:true, cwd:upload_path, src:filepath};
            });

            if( files.length > 1){
                var archive = archiver('zip');

                archive.bulk(files);

                res.header("Content-Disposition", 'attachment; filename="download.zip"');
                res.header('Content-Type', 'application/zip');
                archive.pipe(res);
                archive.finalize();

                archive.on('error', function(err) {
                  throw err;
                });
            }
            else if( files.length == 1 ){
                var file = files[0].cwd + files[0].src;
                var filename = path.basename(file);
                res.header("Content-Disposition", 'attachment; filename="'+filename+'"');
                res.sendfile(filename, {root: path.dirname(file)});
            }

        }catch(e){
            console.log(e);
        }


    });

    app.get('/json/local/files', checkAdmin, function(req, res){try{
            var path = req.query.dir || '';
            if( path.match(/\/\.\./) ){
                res.json({error:'illegal file path'});
            }
            var filepath = upload_path;
            filepath += path.substr(1);
            var files = fs.readdirSync(filepath);
            var results = files.map(function(file){

                //is hidden file
                var isHidden = false;
                if( (/(^|.\/)\.+[^\/\.]/g).test(file) ){
                    isHidden = true;
                }

                var f = {
                     name: (isHidden) ? file : file.replace(/\.[^/.]+$/, "")
                };
                if(filepath.substr(-1) != '/'){
                    filepath += '/';
                }

                if( path != '/' ){
                    f.path = path+'/'+file;
                }
                else{
                    f.path = '/'+file;
                }

                try{
                    var stats = fs.statSync( filepath+file );
                    if( stats.isFile() ){
                        f.size = stats.size;
                        f.modified = stats.mtime.toISOString();
                        if( isHidden ){
                            f.type = '';
                        }
                        else{
                            f.type = (/[.]/.exec(file)) ? /[^.]+$/.exec(file)[0] : 'unknown';
                        }
                    }
                    else if( stats.isDirectory() ){
                        f.type = 'directory';
                    }
                    else{

                    }
                }
                catch(e){
                    console.log('Error: stat file fail', filepath, file, e );
                }

               return f;
            })
            if( path != '' && path != '/' ){
                results.unshift({name:"..", path:path+'/..', type:'directory' })
            }
            res.send(results);
            }catch(e){console.log("ERROR:", e)}
        });


        app.get('/json/:sessID/files/devices', checkAdmin, function(req, res){

           var sid = req.params.sessID;

           if( !(self._sessions[sid]) ){
               res.json({error:'session does not exist'});
               return false;
           }

           var sess = self.sess(sid);
           sess.iface('C').enqueueOne('dev:emit-getdevices', function (session) {
           session.getDevices().then(function(data){
                res.json(JSON.parse(data));
            }, function(err){
                 res.json({error:'Failed to get files', message:err.message});
                });
           });
       })

        app.get('/json/:sessID/files/:device', checkAdmin, function(req, res){
           var sid = req.params.sessID;
           var device = req.params.device;
           var dir = req.query.dir || '/';
           if( !(self._sessions[sid]) ){
               res.json({error:'session does not exist'});
               return false;
           }

           var sess = self.sess(sid);
           sess.iface('C').enqueueOne('dev:emit-getfiles', function (session) {
           session.getFiles(device, dir).then(function(data){
                res.json(JSON.parse(data));
            }, function(err){
                 res.json({error:'Failed to get files', message:err.message});
                });
           });
       })

        app.get('/json/:sessID/fileTransferQ', checkAdmin, function(req, res){
           var sid = req.params.sessID;

           if( !(self._sessions[sid]) ){
               res.json({error:'session does not exist'});
               return false;
           }
           var sess = self.sess(sid);
           sess.iface('C').enqueueOne('dev:emit-getfileTransferQ', function (session) {
           session.getFileTransferQ().then(function(data){
                res.json(JSON.parse(data));
            }, function(err){
                 res.json({error:'Failed to get files', message:err.message});
                });
           });
       });

        app.post('/json/:sessID/fileTransferQ', checkAdmin, function(req, res){
           var sid = req.params.sessID;
           var device = req.body.device;
           var path = req.body.path;

           if( !(self._sessions[sid]) ){
               res.json({error:'session does not exist'});
               return false;
           }
           var sess = self.sess(sid);
           sess.iface('C').enqueueOne('dev:emit-addfileTransferQ', function (session) {
           session.addFileTransferQ(device, path).then(function(data){
                res.json(JSON.parse(data));
            }, function(data){
                 res.json({error:'Failed to get files', data:arguments});
                });
           });
       })


        app.delete('/json/:sessID/fileTransferQ', checkAdmin, function(req, res){
           var sid = req.params.sessID;
           var device = req.body.device;
           var path = req.body.path;

           if( !(self._sessions[sid]) ){
               res.json({error:'session does not exist'});
               return false;
           }
           var sess = self.sess(sid);
           sess.iface('C').enqueueOne('dev:emit-removefileTransferQ', function (session) {
           session.removeFileTransferQ(device, path).then(function(data){
                res.json(JSON.parse(data));
            }, function(data){
                 res.json({error:'Failed to get files', data:arguments});
                });
           });
       })

}