/*
 * Copyright, 2012, Teradek
 * 
 * All rights reserved.
 * Contact Teradek for licensing.
 */

var	redis = require('redis'),
	cli = require('Node-CLI/node-cli.js'),
	sessions = require("./sessions"),
	logs = require("./logs");

var maxServers = 1
var dbconfig;

function config_db(config) {
	dbconfig = config;
}

function update_servers() {
	var numServers = 0
	var db = redis.createClient(dbconfig.port, dbconfig.host)

	db.smembers("servers", function(err, servers) {
		servers.forEach(function(server, i) {
			var sid = "server:" + server + ":info";
			
			// Show the session stats
			db.exists(sid, function(err, stats) {
				db.hget(sid, "starttime", function(err, res){

					cli.move(3, 4).clearNext(80).write("Servers ".bold.blue + (numServers+1));
					sessions.check(dbconfig, server, numServers)

					if (++numServers >= maxServers)
						return;
				});
			});
			
			// Show the log messages
			logs.show(dbconfig, server, 40)
		});
		
	});
}

function check_new_servers() {
	
	var db = redis.createClient(dbconfig.port, dbconfig.host)

	db.subscribe("servers")
	
	db.on("message", function(channel, message) {
		if (message == "add") {
			update_servers()
		}
		else if (message == "rem") {
			update_servers()
		}
	});
}

module.exports = {
	"config" : config_db,
	"check" : check_new_servers,
	"update" : update_servers
}